//
//  DSIntervalPickerCell.h
//  DSInterval
//
//  Created by David Sinclair on 2008-07-11.
//  Copyright 2008 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//

#import <Cocoa/Cocoa.h>
#import "DSInterval.h"


typedef enum _DSIntervalPickerRangeFiltering
{
    DSIntervalAnyRange = 0,
    DSIntervalFirstLessThanSecond = 1,
    DSIntervalFirstLessOrEqualToSecond = 2,
    DSIntervalFirstGreaterOrEqualToSecond = 3,
    DSIntervalFirstGreaterThanSecond = 4
} DSIntervalPickerRangeFiltering;


@interface DSIntervalPickerCell : NSActionCell
{
    // All ivars are private; these are used internally:
    NSTextFieldCell *_firstAmountCell;
    NSTextFieldCell *_rangeSeparatorCell;
    NSTextFieldCell *_secondAmountCell;
    NSTextFieldCell *_unitsCell;
    NSStepperCell *_stepperCell;
    NSTextFieldCell *_selectedCell;
    NSMutableArray *_includedUnitsMapping;
    NSRect _pickerFrame;
    NSRect _firstAmountFrame;
    NSRect _rangeSeparatorFrame;
    NSRect _secondAmountFrame;
    NSRect _unitsFrame;
    NSRect _stepperFrame;
    BOOL _shouldAppendEdit;
    BOOL _autoPopMenu;
    
    // These ivars have public properties:
    DSInterval *_interval;
    DSIntervalPickerRangeFiltering _rangeFiltering;
    NSInteger _minimumAmount;
    NSInteger _maximumAmount;
    BOOL _includeSeconds;
    BOOL _includeMinutes;
    BOOL _includeHours;
    BOOL _includeDays;
    BOOL _includeWeeks;
    BOOL _includeMonths;
    BOOL _includeQuarters;
    BOOL _includeYears;
    BOOL _includeForever;
    BOOL _includeNever;
}

@property(copy) DSInterval *interval;
@property NSInteger firstAmount;
@property NSInteger secondAmount;
@property NSInteger amount;
@property DSIntervalUnits units;
@property(readonly) NSTimeInterval firstTimeInterval;
@property(readonly) NSTimeInterval secondTimeInterval;
@property(readonly) NSTimeInterval amountTimeInterval;
@property BOOL usingRange;
@property DSIntervalPickerRangeFiltering rangeFiltering;
@property NSInteger minimumAmount;
@property NSInteger maximumAmount;
@property BOOL includeSeconds;
@property BOOL includeMinutes;
@property BOOL includeHours;
@property BOOL includeDays;
@property BOOL includeWeeks;
@property BOOL includeMonths;
@property BOOL includeQuarters;
@property BOOL includeYears;
@property BOOL includeForever;
@property BOOL includeNever;

- (NSSize)idealSize;

- (void)setupForShortTerm;
- (void)setupForLongTerm;
- (void)setupForAllUnits;

- (void)cellBecomeFirstResponderInDirection:(NSSelectionDirection)selectionDirection;

@end

