//
//  DSBezierPathCategories.h
//  Shared categories
//
//  Created by David Sinclair on Fri Oct 29 2004.
//  Copyright (c) 2004 - 2007 Dejal Systems, LLC. All rights reserved.
//
//  Portions based on sample code from <http://www.harmless.de/cocoa.html#rollover>.
//  Portions based on sample code from <http://www.cocoadev.com/index.pl?GradientFill>.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//


typedef enum
{
	DSTriangleUp = 0,
	DSTriangleDown,
	DSTriangleLeft,
	DSTriangleRight
} DSTriangleOrientation;


@interface NSBezierPath (DSBezierPathCategories)

+ (NSBezierPath *)bezierPathWithPlateInRect:(NSRect)rect;
- (void)appendBezierPathWithPlateInRect:(NSRect)rect;

+ (NSBezierPath *)bezierPathWithRoundedRect:(NSRect)rect cornerRadius:(float)radius;
- (void)appendBezierPathWithRoundedRect:(NSRect)rect cornerRadius:(float)radius;

+ (NSBezierPath *)bezierPathWithTriangleInRect:(NSRect)aRect orientation:(DSTriangleOrientation)orientation;
- (void)appendBezierPathWithTriangleInRect:(NSRect)aRect orientation:(DSTriangleOrientation)orientation;

- (void)customVerticalFillWithCallbacks:(CGFunctionCallbacks)functionCallbacks firstColor:(NSColor *)firstColor secondColor:(NSColor *)secondColor;

- (void)linearGradientFillWithStartColor:(NSColor *)startColor endColor:(NSColor *)endColor;

- (void)bilinearGradientFillWithOuterColor:(NSColor *)outerColor innerColor:(NSColor *)innerColor;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark CONVENIENCE FUNCTIONS
// ----------------------------------------------------------------------------------------


void DSFrameRoundedRect(NSRect aRect, float radius);
void DSFrameRoundedRectWithWidth(NSRect aRect, float radius, float width);

