//
//  DSTableViewCategories.h
//  Shared categories
//
//  Created by David Sinclair on Mon Aug 12 2002.
//  Copyright � 2002 - 2007 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//


@interface NSTableView (DSTableViewCategories)

- (IBAction)selectNone:(id)sender;

- (NSEnumerator *)rowEnumerator;
- (NSEnumerator *)selectedOrAllRowsEnumerator;
- (NSEnumerator *)multipleSelectedOrAllRowsEnumerator;

- (void)selectFirstRowExtendingSelection:(BOOL)extendSel;
- (void)selectLastRowExtendingSelection:(BOOL)extendSel;

- (void)editColumnWithIdentifier:(NSString *)columnIdentfier row:(NSInteger)row;

- (NSEnumerator *)tableColumnEnumerator;
- (NSTableColumn *)tableColumnAtIndex:(NSUInteger)i;
- (NSUInteger)indexOfFirstEditableTableColumn;
- (NSTableColumn *)firstEditableTableColumn;

- (NSTableColumn *)addTableColumnWithIdentifier:(NSString *)identifier title:(NSString *)title editable:(BOOL)editable resizable:(BOOL)resizable width:(CGFloat)width;
- (NSTableColumn *)addTableColumnWithIdentifier:(NSString *)identifier title:(NSString *)title editable:(BOOL)editable resizable:(BOOL)resizable sortable:(BOOL)sortable ascending:(BOOL)ascending width:(CGFloat)width alignment:(NSTextAlignment)alignment;

- (NSTableColumn *)addTableColumnWithColumnInfo:(NSDictionary *)columnInfo;
- (void)addTableColumnsWithArrayOfColumnInfo:(NSArray *)columns removeAll:(BOOL)removeAll sizeLast:(BOOL)sizeLast;

- (void)insertTableColumn:(NSTableColumn *)aColumn atIndex:(NSUInteger)i;

- (void)removeTableColumns:(NSArray *)columns;

- (void)registerDefaultSortDescriptorForTableColumnWithIdentifier:(NSString *)identifier ascending:(BOOL)ascending;
- (void)registerDefaultSortDescriptorForTableColumn:(NSTableColumn *)tableColumn ascending:(BOOL)ascending;

- (void)setHeaderImageNamed:(NSString *)imageName inTableColumnWithIdentifier:(NSString *)identifier;
- (void)setDataImageNamed:(NSString *)imageName inTableColumnWithIdentifier:(NSString *)identifier;
- (void)setDataProgressCellClass:(Class)class withAltImageNamed:(NSString *)imageName inTableColumnWithIdentifier:(NSString *)identifier;

- (void)setHeaderAndDataImageNamed:(NSString *)imageName inTableColumnWithIdentifier:(NSString *)identifier;
- (void)setHeaderAndDataProgressCellClass:(Class)class withAltImageNamed:(NSString *)imageName inTableColumnWithIdentifier:(NSString *)identifier;

- (NSString *)truncatedString:(NSString *)string withIndicator:(NSString *)indicator forTableColumn:(NSTableColumn *)tableColumn;

- (NSMenu *)menuForEvent:(NSEvent *)event;

- (void)addKey:(NSString *)key withValue:(NSString *)value toDictionary:(NSMutableDictionary *)dict;
- (void)removeSelectedKeyFromDictionary:(NSMutableDictionary *)dict;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface NSTableView (DSTableViewCutCopyPasteDeleteCategories)

- (IBAction)cut:(id)sender;
- (IBAction)copy:(id)sender;
- (IBAction)paste:(id)sender;
- (IBAction)delete:(id)sender;

- (BOOL)validateMenuItem:(NSMenuItem *)item;

- (NSIndexSet *)canCutRowIndexes;
- (NSIndexSet *)canCopyRowIndexes;
- (NSInteger)canPasteBeforeRow;
- (NSIndexSet *)canDeleteRowIndexes;

- (BOOL)deleteRowIndexes:(NSIndexSet *)indexes;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@protocol DSTableViewCutCopyPasteDeleteDelegateMethods
@optional

- (NSIndexSet *)tableView:(NSTableView *)tableView canCutRowIndexes:(NSIndexSet *)indexes;
- (NSIndexSet *)tableView:(NSTableView *)tableView canCopyRowIndexes:(NSIndexSet *)indexes;
- (NSInteger)tableView:(NSTableView *)tableView canPasteBeforeRow:(NSInteger)row;
- (NSIndexSet *)tableView:(NSTableView *)tableView canDeleteRowIndexes:(NSIndexSet *)indexes;

- (BOOL)tableView:(NSTableView *)tableView deleteRowIndexes:(NSIndexSet *)indexes;

- (NSAttributedString *)tableView:(NSTableView *)tableView attributedStringValueForRow:(NSInteger)row;
- (NSString *)tableView:(NSTableView *)tableView stringValueForRow:(NSInteger)row;
- (NSString *)tableView:(NSTableView *)tableView delimiterForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row;

@end

