//
//  DSTreeControllerCategories.m
//  Shared categories
//
//  Created by David Sinclair on 2006-04-17.
//  Copyright (c) 2006 Dejal Systems, LLC. All rights reserved.
//
//  Portions based on code by Wil Shipley, Copyright (c) 2006 Delicious Monster Software, LLC:
//      <http://wilshipley.com/blog/2006/04/pimp-my-code-part-10-whining-about.html>.
//

#import "DSTreeControllerCategories.h"


@interface NSTreeController (DSCategories_Private)

- (NSIndexPath *)_indexPathFromIndexPath:(NSIndexPath *)baseIndexPath inChildren:(NSArray *)children
                              childCount:(NSUInteger)childCount toObject:(id)object;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation NSTreeController (DSCategories)

/*
 indexPathToObject:
 
 Returns an index path for the specified object.
 
 Written by WS 2006-03.
*/

- (NSIndexPath *)indexPathToObject:(id)object;
{
    NSArray *children = [self content];
    return [self _indexPathFromIndexPath:nil inChildren:children childCount:[children count]
                                toObject:object];
}

/*
 setSelectedObjects:
 
 Changes the selected objects to those in the array.
 
 Written by WS 2006-03.
*/

- (void)setSelectedObjects:(NSArray *)newSelectedObjects;
{
    NSMutableArray *indexPaths = [NSMutableArray array];
    for (id selectedObject in newSelectedObjects) {
        NSIndexPath *indexPath = [self indexPathToObject:selectedObject];
        if (indexPath)
            [indexPaths addObject:indexPath];
    }
    
    [self setSelectionIndexPaths:indexPaths];
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation NSTreeController (DSCategories_Private)

- (NSIndexPath *)_indexPathFromIndexPath:(NSIndexPath *)baseIndexPath inChildren:(NSArray *)children
                              childCount:(NSUInteger)childCount toObject:(id)object;
{
    NSUInteger childIndex;
    for (childIndex = 0; childIndex < childCount; childIndex++) {
        id childObject = [children objectAtIndex:childIndex];
        
        NSArray *childsChildren = nil;
        NSUInteger childsChildrenCount = 0;
        NSString *leafKeyPath = [self leafKeyPath];
        if (!leafKeyPath || [[childObject valueForKey:leafKeyPath] boolValue] == NO) {
            NSString *countKeyPath = [self countKeyPath];
            if (countKeyPath)
                childsChildrenCount = [[childObject valueForKey:leafKeyPath] unsignedIntValue];
            if (!countKeyPath || childsChildrenCount != 0) {
                NSString *childrenKeyPath = [self childrenKeyPath];
                childsChildren = [childObject valueForKey:childrenKeyPath];
                if (!countKeyPath)
                    childsChildrenCount = [childsChildren count];
            }
        }
        
        BOOL objectFound = [object isEqual:childObject];
        if (!objectFound && childsChildrenCount == 0)
            continue;
        
        NSIndexPath *indexPath = (baseIndexPath == nil) ? [NSIndexPath indexPathWithIndex:childIndex]
                                                        : [baseIndexPath indexPathByAddingIndex:childIndex];
        
        if (objectFound)
            return indexPath;
        
        NSIndexPath *childIndexPath = [self _indexPathFromIndexPath:indexPath inChildren:childsChildren
                                                         childCount:childsChildrenCount toObject:object];
        if (childIndexPath)
            return childIndexPath;
    }
    
    return nil;
}

@end

