//
//  DSImageViewCategories.m
//  Shared categories
//
//  Created by David Sinclair on Sat Nov 29 2003.
//  Copyright (c) 2003 Dejal Systems, LLC. All rights reserved.
//

#import "DSImageViewCategories.h"


@implementation NSImageView (DSImageViewCategories)

// DJS 2005-05 note: Tiger (10.4) implements -cut:, -copy:, -paste:, and -delete: for NSImageView, so these categories will override that functionality.  These categories are now deprecated.  They may be removed in the future.  These categories are not used by default (via DSCategories.h), so the Tiger ones will be used unless this is explicitly imported.  If other NSImageView categories are added in the future, these should be removed or moved to a separate file.

- (BOOL)_doCopy
{
    // Get image as TIFF:
    NSImage *image = [self image];
    NSData *tiff = [image TIFFRepresentation];
    NSPasteboard *pboard = [NSPasteboard generalPasteboard];
    
    if (tiff)
    {
        // Declare type:
        [pboard declareTypes:[NSArray arrayWithObject:NSTIFFPboardType] owner:self];
        
        // Copy string to pasteboard:
        [pboard setData:tiff forType:NSTIFFPboardType];
        
        return YES;
    }
    else
        return NO;
}

- (IBAction)cut:(id)sender
{
    // Only Delete if the Copy was successful:
    if ([self _doCopy])
        [self delete:sender];
}

- (IBAction)copy:(id)sender
{
    [self _doCopy];
}

- (IBAction)paste:(id)sender
{
    NSImage *image = [[[NSImage alloc] initWithPasteboard:[NSPasteboard generalPasteboard]] autorelease];
    
    if (image)
    {
        [self setImage:image];
        
        [self sendAction:[self action] to:[self target]];
    }
}

- (IBAction)delete:(id)sender
{
    [self setImage:nil];
    
    [self sendAction:[self action] to:[self target]];
}

- (BOOL)validateMenuItem:(NSMenuItem *)item
{
    SEL action = [item action];
    BOOL editable = [self isEditable];
    
    if (action == @selector(copy:))
        return ([self image] != nil);
    else if (action == @selector(cut:) || action == @selector(delete:))
        return (editable && [self image] != nil);
    else if (action == @selector(paste:))
        return (editable && [NSImage canInitWithPasteboard:[NSPasteboard generalPasteboard]]);
    else
        return YES;
}

@end
