//
//  DSArrayCategories.h
//  Shared categories
//
//  Created by David Sinclair on Fri May 30 2003.
//  Copyright (c) 2003 - 2009 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//

#import <Foundation/Foundation.h>


@interface NSArray (DSArrayCategories)

- (BOOL)isValidIndex:(NSUInteger)i;

- (id)objectMatching:(id)match usingKey:(NSString *)key;
- (NSUInteger)indexOfObjectMatching:(id)match usingKey:(NSString *)key;

- (id)objectMatching:(id)match usingSelector:(SEL)selector;
- (id)objectMatching:(id)match orDefault:(id)defaultMatch orFirst:(BOOL)firstIfNotFound usingSelector:(SEL)selector;

- (NSArray *)arrayWithObjectsMatching:(id)match usingKey:(NSString *)key;
- (NSArray *)arrayWithObjectsMatching:(id)match usingSelector:(SEL)selector;

- (NSArray *)arrayUsingSelector:(SEL)selector;

- (NSArray *)arrayByRemovingObjectsInArray:(NSArray *)otherArray;

- (NSArray *)reverseArray;

- (NSArray *)sortedArrayUsingFinderOrder;

- (BOOL)containsObjectIdenticalTo:(id)obj;
- (BOOL)containsObjectEquivalentTo:(id)obj;
- (NSUInteger)indexOfObjectEquivalentTo:(id)obj;

- (id)firstObject;
- (id)penultimateObject;

- (id)deepCopy;
- (id)deepMutableCopy;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface NSMutableArray (DSArrayCategories)

- (NSUInteger)insertObject:(id)object atIndex:(NSUInteger)i orEnd:(BOOL)atEnd;
- (void)insertObjectsFromArray:(NSArray *)array atIndex:(NSUInteger)i;
- (NSUInteger)insertOrMoveObjectsFromArray:(NSArray *)array atIndex:(NSUInteger)i;
- (void)addOrMoveObjectsFromArray:(NSArray *)array;

- (NSMutableArray *)arrayWithIndexEnumerator:(NSEnumerator *)enumerator;

- (void)removeObjectsFromIndexEnumerator:(NSEnumerator *)enumerator;
- (void)removeObjectsMatching:(id)match usingKey:(NSString *)key;

- (NSMutableArray *)reverseArray;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface NSMutableArray (DSToolbarCategories)

- (NSToolbarItem *)toolbarItemForIdentifier:(NSString *)identifier;

- (NSArray *)toolbarItemIdentifiersIncludingStandardItems:(BOOL)inclStd;

- (void)addToolbarItem:(NSString *)identifier
                 label:(NSString *)label
          paletteLabel:(NSString *)paletteLabel
               toolTip:(NSString *)toolTip
                  view:(id)view
              minWidth:(CGFloat)minWidth
              maxWidth:(CGFloat)maxWidth
             minHeight:(CGFloat)minHeight
             maxHeight:(CGFloat)maxHeight
    menuRepresentation:(NSMenuItem *)menuItem
               submenu:(NSMenu *)submenu
               enabled:(BOOL)enable
                target:(id)target
                action:(SEL)action
                   tag:(NSInteger)tag;

- (void)addToolbarItem:(NSString *)identifier
                 label:(NSString *)label
          paletteLabel:(NSString *)paletteLabel
               toolTip:(NSString *)toolTip
            imageNamed:(NSString *)imageName
               enabled:(BOOL)enable
                target:(id)target
                action:(SEL)action
                   tag:(NSInteger)tag;

- (void)addToolbarItem:(NSString *)identifier
                 label:(NSString *)label
          paletteLabel:(NSString *)paletteLabel
               toolTip:(NSString *)toolTip
            imageNamed:(NSString *)imageName
               enabled:(BOOL)enable
                target:(id)target
                action:(SEL)action;

@end
