//
//  DSCarbonCategories.m
//  Shared categories
//
//  Created by David Sinclair on 2005-08-13.
//  Copyright 2004 - 2007 Dejal Systems, LLC. All rights reserved.
//

#import "DSCarbonCategories.h"
#import <unistd.h>
#import <fcntl.h>


@implementation NSString (DSCarbonStringCategories)

/*
 * +stringWithFSRef:
 *  Created by Nathan Day on Sat Aug 03 2002.
 *  Copyright (c) 2002 Nathan Day. All rights reserved.
 */

+ (NSString *)stringWithFSRef:(const FSRef *)aFSRef
{
    UInt8 thePath[PATH_MAX + 1];		// plus 1 for \0 terminator
    
    return (FSRefMakePath(aFSRef, thePath, PATH_MAX) == noErr) ? [NSString stringWithUTF8String:thePath] : nil;
}

/*
 * -getFSRef:
 *  Created by Nathan Day on Sat Aug 03 2002.
 *  Copyright (c) 2002 Nathan Day. All rights reserved.
 */

- (BOOL)getFSRef:(FSRef *)aFSRef
{
    return FSPathMakeRef([self UTF8String], aFSRef, NULL) == noErr;
}

/*
 * -resolveAliasFile
 *  Created by Nathan Day on Sat Aug 03 2002.
 *  Copyright (c) 2002 Nathan Day. All rights reserved.
 */

- (NSString *)resolveAliasFile
{
    FSRef theRef;
    Boolean theIsTargetFolder, theWasAliased;
    NSString *theResolvedAlias = nil;
    
    [self getFSRef:&theRef];
    
    if ((FSResolveAliasFile (&theRef, YES, &theIsTargetFolder, &theWasAliased) == noErr))
    {
        theResolvedAlias = (theWasAliased) ? [NSString stringWithFSRef:&theRef] : self;
    }
    
    return theResolvedAlias;
}

/*
 getFSSpec:createIfMissing:
 
 Given a path in the receiver, creates a FSSpec struct.  If createIfMissing is YES, a file is created if there is none at that path; otherwise, this method will fail.  Returns YES if successful.  Given a declaration of FSSpec fileSpec, pass &fileSpec.
 
 Written by DJS 2004-03.
*/

- (BOOL)getFSSpec:(FSSpec *)fileSpec createIfMissing:(BOOL)createIfMissing
{
    FSRef fsRef;
    
    OSErr err = FSPathMakeRef([self fileSystemRepresentation], &fsRef, NULL);
    
    // If the error is file not found then lets create it for it:
    if (err == fnfErr)
    {
        int fd;
        
        fd = open([self fileSystemRepresentation], O_CREAT | O_RDWR, 0600);
        
        if (fd < 0)
            return NO;
        
        write(fd, " ", 1);
        close(fd);
        
        err = FSPathMakeRef([self fileSystemRepresentation], &fsRef, NULL);
    }
    
    if (err == noErr)
    {
        err = FSGetCatalogInfo(&fsRef, kFSCatInfoNone, NULL, NULL, fileSpec, NULL);
    }
    
    return err == noErr;
}

/*
 pathFileKind
 
 Returns a string describing the kind of file, for the path in the receiver.
*/

- (NSString *)pathFileKind
{
    OSStatus status;
    CFStringRef outKindString;
    
    // KIND string for Finder kind:
    status = LSCopyKindStringForURL((CFURLRef)[NSURL fileURLWithPath:self], &outKindString);
    
    if (status == noErr)
        return [(NSString *)outKindString autorelease];
    else if (status == kLSDataUnavailableErr)
        return @"unknown";
    else
        return @"???";
}

@end
