//
//  DSStringCategories.h
//  Shared categories
//
//  Created by David Sinclair on Sat Aug 17 2002.
//  Copyright � 2002 - 2008 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//


@interface NSString (DSStringCategories)

+ (NSString *)stringWithIntValue:(NSInteger)value;
+ (NSString *)stringWithIntegerValue:(NSInteger)value;
+ (NSString *)stringWithFloatValue:(CGFloat)value places:(NSInteger)places;

+ (NSString *)stringWithIntValue:(NSInteger)value
                            zero:(NSString *)zero
                        singluar:(NSString *)singular
                          plural:(NSString *)plural;

+ (NSString *)stringWithIntegerValue:(NSInteger)value
                                zero:(NSString *)zero
                            singluar:(NSString *)singular
                              plural:(NSString *)plural;

+ (NSString *)stringWithFloatValue:(CGFloat)value
                              zero:(NSString *)zero
                          singluar:(NSString *)singular
                            plural:(NSString *)plural;

+ (NSString *)stringWithSeconds:(NSInteger)seconds
                 minuteSingular:(NSString *)minuteSingular
                  minutesPlural:(NSString *)minutesPlural
                 secondSingular:(NSString *)secondSingular
                  secondsPlural:(NSString *)secondsPlural;

+ (NSString *)stringWithIntValue:(NSInteger)value minLength:(NSUInteger)minLength maxLength:(NSUInteger)maxLength
                      paddedWith:(char)padding padLeft:(BOOL)padLeft useEllipsis:(BOOL)ellipsis;
+ (NSString *)stringWithIntegerValue:(NSInteger)value minLength:(NSUInteger)minLength maxLength:(NSUInteger)maxLength
                          paddedWith:(char)padding padLeft:(BOOL)padLeft useEllipsis:(BOOL)ellipsis;

+ (NSString *)stringWithLeadingZeroesForIntValue:(NSInteger)value digits:(NSInteger)digits;
+ (NSString *)stringWithLeadingZeroesForIntegerValue:(NSInteger)value digits:(NSInteger)digits;

+ (NSString *)stringAsBytesWithInt:(NSInteger)bytes;
+ (NSString *)stringAsBytesWithInteger:(NSInteger)bytes;

+ (NSString *)stringWithPrefix:(NSString *)prefix keyword:(NSString *)keyword suffix:(NSString *)suffix or:(NSString *)alternative;

- (id)or:(id)preferred;

- (BOOL)containsSomething;

- (BOOL)containsString:(NSString *)subString;
- (BOOL)containsStringCaseInsensitive:(NSString *)subString;

- (NSComparisonResult)caseAndSpaceInsensitiveCompare:(NSString *)otherString;

- (NSString *)lowercasedLettersOnly;
- (NSString *)lowercasedLettersOrDigitsOnly;
- (BOOL)containsStringLetters:(NSString *)otherString;
- (BOOL)isLetterEquivalentToString:(NSString *)otherString;

- (NSString *)stringByRemovingCharactersInSet:(NSCharacterSet *)set;
- (NSString *)stringByRemovingDiacriticalMarks;
- (NSString *)stringByRemovingQuotesAndSpaces;

- (NSString *)stringByReplacingAllOccurrencesOf:(NSString *)string1 with:(NSString *)string2;

- (NSString *)stringByDeletingLeadingSpaces:(BOOL)leading trailingSpaces:(BOOL)trailing;

- (NSString *)truncatedStringWithIndicator:(NSString *)indicator forFont:(NSFont *)font withWidth:(NSUInteger)desiredWidth trimLeft:(BOOL)trimLeft;
- (NSString *)truncatedStringWithIndicator:(NSString *)indicator forFont:(NSFont *)font withWidth:(NSUInteger)desiredWidth;

- (NSArray *)componentsSeparatedByLength:(NSUInteger)length;
+ (NSUInteger)lengthOfInteger:(NSInteger)integer;

- (NSInteger)wordCount;

- (NSDictionary *)dictionaryWithVersionComponents;
+ (NSString *)versionWithDictionary:(NSDictionary *)dict;
+ (NSString *)versionWithMajor:(NSInteger)major minor:(NSInteger)minor bug:(NSInteger)bug kind:(NSString *)kind stage:(NSInteger)stage;

- (NSString *)left:(NSInteger)length;
- (NSString *)right:(NSInteger)length;

- (NSString *)from:(NSUInteger)position length:(NSUInteger)length;
- (NSString *)from:(NSUInteger)startPosition to:(NSUInteger)endPosition;

- (NSString *)substringFromString:(NSString *)string;
- (NSString *)substringToString:(NSString *)string;

- (NSString *)reverse;

- (NSString *)mask;
- (NSString *)unmask;

- (NSString *)rot13;

+ (NSString *)universallyUniqueIdentifier;

- (NSInteger)integerMappedFromString:(NSString *)value withDefault:(NSInteger)defaultValue;
- (NSString *)stringMappedFromInteger:(NSInteger)value withDefault:(NSString *)defaultValue;
- (NSString *)mappedFromValue:(id)value withDefault:(id)defaultValue;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface NSString (DSInternetStringCategories)

- (NSString *)stringByReplacingPercentEscapes;
- (NSString *)stringByAddingPercentEscapes;

- (NSString *)stringByMakingURLSafe;

- (NSString *)stringByStrippingHTML;
- (NSString *)stringByReplacingHTMLEntities;

- (NSString *)stringByCleaningURL;
- (NSString *)stringByCleaningURLWithDefaultScheme:(NSString *)scheme;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface NSString (DSFilePathStringCategories)

- (NSString *)stringByCleaningFilenameWithDefault:(NSString *)defaultFilename;
- (NSString *)stringByAppendingPathComponent:(NSString *)dirtyFilename cleanWithDefault:(NSString *)defaultFilename;

- (NSString *)lastPathComponentWithoutExtension;

- (NSString *)backupFilePath;

- (NSString *)uniquePath;
- (NSString *)uniquePathWithPrefix:(NSString *)prefix;

- (NSString *)validatedDirectoryPath;
- (NSString *)validatedFilePath;

- (NSString *)expandedPath;
- (NSString *)abbreviatedPath;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface NSString (DSPropertyListStringCategories)

+ (NSString *)stringWithPoint:(NSPoint)point;
- (NSPoint)pointValue;

+ (NSString *)stringWithBoolValue:(BOOL)value;
- (BOOL)booleanValue;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface NSString (DSAppKitStringCategories)

+ (NSString *)stringWithColor:(NSColor *)color;

- (NSColor *)color;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface NSMutableString (DSStringCategories)

- (void)deleteCharactersInSet:(NSCharacterSet *)set;

- (void)caseInsensitiveReplaceAllOccurrencesOf:(NSString *)string1 with:(NSString *)string2;
- (void)replaceAllOccurrencesOf:(NSString *)string1 with:(NSString *)string2;

- (void)deleteLeadingSpaces:(BOOL)leading trailingSpaces:(BOOL)trailing;

- (void)appendString:(NSString *)string or:(NSString *)alternative;
- (void)appendPrefix:(NSString *)prefix keyword:(NSString *)keyword suffix:(NSString *)suffix or:(NSString *)alternative;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface NSString (DSPascalStringCategories)

+ (NSString *)stringWithPascalString:(void *)strPtr;

- (BOOL)getPascalString:(void *)strPtr maxLength:(NSUInteger)maxLength;

@end

