//
//  DSUserDefaultsCategories.h
//  Shared categories
//
//  Created by David Sinclair on Thu Jul 03 2003.
//  Copyright (c) 2003 - 2008 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//


@interface NSUserDefaults (DSUserDefaultsCategories)

- (void)setObject:(id)anObject forKey:(id)aKey defaultValue:(id)aDefault;
- (void)setObject:(id)anObject forKey:(id)aKey removeIfNil:(BOOL)removeIfNil;

- (NSInteger)incrementIntegerForKey:(NSString *)defaultName;

- (NSPoint)pointForKey:(NSString *)defaultName;
- (void)setPoint:(NSPoint)point forKey:(NSString *)defaultName;

- (NSSize)sizeForKey:(NSString *)defaultName orDefaultSize:(NSSize)defaultSize;
- (void)setSize:(NSSize)size forKey:(NSString *)defaultName;

- (NSColor *)colorForKey:(NSString *)defaultName;
- (void)setColor:(NSColor *)color forKey:(NSString *)defaultName;

- (NSColor *)grayForKey:(NSString *)defaultName;
- (void)setGray:(NSColor *)gray forKey:(NSString *)defaultName;

- (NSDictionary *)factorySettingsForKeys:(NSArray *)keysArray;

- (BOOL)changedFromFactorySettingsForKey:(NSString *)key;
- (BOOL)changedFromFactorySettingsForKeys:(NSArray *)keysArray;

- (void)restoreFactorySettingsForKey:(NSString *)key;
- (void)restoreFactorySettingsForKeys:(NSArray *)keysArray;

- (BOOL)copyObjectForKey:(NSString *)oldKey toKey:(NSString *)newKey replace:(BOOL)replace;
- (BOOL)copyObjectsForKeys:(NSArray *)keyPairsArray replace:(BOOL)replace;

- (void)copyPreferenceWithKey:(NSString *)preferenceKey fromBundleIdentifier:(NSString *)bundleIdentifier removeIfMissing:(BOOL)removeIfMissing;
- (void)copyPreferencesWithKeys:(NSArray *)preferenceKeys fromBundleIdentifier:(NSString *)bundleIdentifier removeIfMissing:(BOOL)removeIfMissing;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark LOGIN SYSTEM PREFERENCES METHODS
// ----------------------------------------------------------------------------------------

- (BOOL)isInLoginItems:(NSString *)path inAnyLocation:(BOOL)inAnyLocation;
- (void)loginItem:(NSString *)path add:(BOOL)add hide:(BOOL)hide;

@end
