//
//  DSInterval.h
//  DSInterval
//
//  Created by David Sinclair on 2007-04-05.
//  Copyright 2007-2008 Dejal Systems, LLC. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification,
//  are permitted provided that the following conditions are met:
//  
//  Redistributions of source code must retain this list of conditions and the following disclaimer.
//  
//  The name of Dejal Systems, LLC may not be used to endorse or promote products derived from this
//  software without specific prior written permission.
//  
//  THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
//  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
//  PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE AUTHORS OR
//  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
//  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
//  SOFTWARE OR THE USE OR OTHER DEALINGS IN THIS SOFTWARE.
//

#import <Cocoa/Cocoa.h>


typedef enum _DSIntervalUnits
{
    DSIntervalNever = 0,
    DSIntervalSeconds = 1,
    DSIntervalMinutes = 2,
    DSIntervalHours = 3,
    DSIntervalDays = 4,
    DSIntervalWeeks = 5,
    DSIntervalMonths = 6,
    DSIntervalQuarters = 7,
    DSIntervalYears = 8,
    DSIntervalForever = 9
} DSIntervalUnits;


@interface DSInterval : NSObject <NSCopying>
{
    BOOL _usingRange;
    CGFloat _firstAmount;
    CGFloat _secondAmount;
    DSIntervalUnits _units;
    id _userInfo;
}

@property BOOL usingRange;
@property CGFloat firstAmount;
@property CGFloat secondAmount;
@property CGFloat amount;
@property DSIntervalUnits units;
@property(copy) id userInfo;
@property(readonly) NSTimeInterval amountTimeInterval;
@property(readonly) NSTimeInterval firstTimeInterval;
@property(readonly) NSTimeInterval secondTimeInterval;
@property(readonly) NSTimeInterval rangeDifferenceInterval;
@property(readonly) NSTimeInterval unitsTimeInterval;

+ (id)intervalWithAmount:(CGFloat)amount units:(DSIntervalUnits)units;
+ (id)intervalWithAmount:(CGFloat)amount units:(DSIntervalUnits)units userInfo:(id)userInfo;
+ (id)intervalWithRangeFirstAmount:(CGFloat)firstAmount secondAmount:(CGFloat)secondAmount units:(DSIntervalUnits)units;
+ (id)intervalWithRangeFirstAmount:(CGFloat)firstAmount secondAmount:(CGFloat)secondAmount units:(DSIntervalUnits)units userInfo:(id)userInfo;

+ (id)intervalWithDictionary:(NSDictionary *)dict;

- (id)initWithRange:(BOOL)usingRange firstAmount:(CGFloat)firstAmount secondAmount:(CGFloat)secondAmount units:(DSIntervalUnits)units userInfo:(id)userInfo;

- (NSDictionary *)dictionary;

- (NSTimeInterval)amountTimeInterval;
- (NSTimeInterval)firstTimeInterval;
- (NSTimeInterval)secondTimeInterval;
- (NSTimeInterval)rangeDifferenceInterval;
- (NSTimeInterval)unitsTimeInterval;

- (CGFloat)amount;
- (void)setAmount:(CGFloat)amount;

- (NSString *)amountWithBriefUnitsName;
- (NSString *)amountWithFullUnitsName;

- (NSString *)briefUnitsName;
- (NSString *)fullUnitsName;

@end

