//
//  DSAgentController.h
//  Shared classes
//
//  Created by David Sinclair on 2007-03-14.
//  Copyright 2007 Dejal Systems, LLC. All rights reserved.
//

#import "DSAgentManager.h"


@interface DSAgentController : NSObject
{
    BOOL _fullyStarted;
    BOOL _quitting;
    BOOL _debugMode;
    DSAgentManager *_ownerAgent;
    NSTimer *_timer;
}

+ (id)mainController;

- (void)someAppTerminated:(NSNotification *)notification;

- (void)debugWithFormat:(NSString *)format, ...;

+ (void)fatalError:(NSString *)message;

- (void)upgradePreferences;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark AGENT DELEGATE METHODS
// ----------------------------------------------------------------------------------------

- (void)agentDidLaunch:(DSAgentManager *)sender withRelease:(NSString *)release;
- (void)agentRequestsTermination:(DSAgentManager *)sender;
- (void)agentJustCheckingIn:(DSAgentManager *)sender;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark OVERRIDE METHODS
// ----------------------------------------------------------------------------------------

- (NSMutableDictionary *)defaultPreferences;
- (int)upgradePreferencesFromVersion:(int)preferencesVersion;

- (void)agentWillSetupOwner;
- (void)agentDidSetupOwner;
- (void)agentDidFinishLaunching;
- (BOOL)agentCanTerminate;
- (void)agentCleanup;

- (NSString *)ownerName;
- (NSString *)ownerBundleIdentifier;
- (DSAgentLocation)ownerLocation;

- (BOOL)shouldTerminateWithOwner;
- (void)someApplicationDidTerminateWithBundleIdentifier:(NSString *)bundleIdentifier;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (void)setMustQuitWithoutConfirmation;

- (BOOL)fullyStarted;
- (void)setFullyStarted:(BOOL)value;

- (BOOL)isQuitting;
- (void)setQuitting:(BOOL)value;

- (BOOL)debugMode;
- (void)setDebugMode:(BOOL)value;

- (DSAgentManager *)ownerAgent;
- (void)setOwnerAgent:(DSAgentManager *)value;

- (NSString *)dataFolderPath;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark CONVENIENCE ACCESSOR METHODS
// Provided for subclasses to use freely
// ----------------------------------------------------------------------------------------

- (NSTimer *)timer;
- (void)setTimer:(NSTimer *)value;

@end

