//
//  DSAgentMainController.m
//  Shared classes
//
//  Created by David Sinclair on 2007-03-29.
//  Copyright 2007 Dejal Systems, LLC. All rights reserved.
//

#import "DSAgentMainController.h"
#import "DSUpdateController.h"


@implementation DSAgentMainController

- (void)dealloc;
{
    [self setSchedulerAgent:nil];
    [self setStatusAgent:nil];
    
    [super dealloc];
}

- (void)relaunchAgent:(NSTimer *)timer;
{
    DSAgentManager *agent = [timer userInfo];
    
    [agent remotelyPerformAgentDidLaunchWithRelease];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark OVERRIDE METHODS
// ----------------------------------------------------------------------------------------

/*
 preferenceClasses
 
 Returns an array of class names (as strings) for the preference pages.
 
 Written by DJS 2007-03.
*/

- (NSArray *)preferenceClasses;
{
    return [NSArray arrayWithObjects:@"DSPrefsGeneral", @"DSAgentPrefsUpdates", @"DSPrefsAdvanced", nil];
}

/*
 checkForUpdates:runModal:
 
 Diverts the update check to the Status agent.
 
 Written by DJS 2007-03.
*/

- (void)checkForUpdates:(int)checkingMode runModal:(BOOL)runModal;
{
    [[self statusAgent] remotelyPerformAgentCheckForUpdates:checkingMode runModal:runModal];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark AGENT DELEGATE METHODS
// ----------------------------------------------------------------------------------------

/*
 agent:wasWrongRelease:
 
 Delegate method received when a remote agent sends this signature to us.  The sender is the DSAgentManager of the agent that sent the message.  Received when the agent is a different release than the receiver (which will have launched it).  Launches the agent again.
 
 Written by DJS 2007-03.
*/

- (void)agent:(DSAgentManager *)sender wasWrongRelease:(NSString *)release;
{
    // Relaunch the agent in 10 seconds, to give it time to quit:
    [NSTimer scheduledTimerWithTimeInterval:10.0 target:self selector:@selector(relaunchAgent:) userInfo:sender repeats:NO];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (DSAgentManager *)schedulerAgent;
{
    return _schedulerAgent;
}

- (void)setSchedulerAgent:(DSAgentManager *)value;
{
    [value retain];
    [_schedulerAgent autorelease];
    _schedulerAgent = value;
}

- (DSAgentManager *)statusAgent;
{
    return _statusAgent;
}

- (void)setStatusAgent:(DSAgentManager *)value;
{
    [value retain];
    [_statusAgent autorelease];
    _statusAgent = value;
}

- (id)updateController;
{
    return [DSUpdateController sharedUpdateController];
}

@end

