//
//  DSAgentManager.h
//  Shared classes
//
//  Created by David Sinclair on 2007-03-20.
//  Copyright 2007 Dejal Systems, LLC. All rights reserved.
//

#import "DSDelegateData.h"


typedef enum _DSAgentLocation
{
    DSAgentIsParent = 0,
    DSAgentIsSibling = 1,
    DSAgentIsChild = 2
} DSAgentLocation;


@interface DSAgentManager : DSDelegateData
{
    NSString *_name;
    NSString *_bundleIdentifier;
    BOOL _isSingleInstance;
    BOOL _shouldStopAgentWithParent;
    DSAgentLocation _relativeLocation;
}

+ (id)agentWithName:(NSString *)name bundleIdentifier:(NSString *)bundleIdentifier relativeLocation:(DSAgentLocation)relativeLocation delegate:(id)delegate;

- (id)initWithName:(NSString *)name bundleIdentifier:(NSString *)bundleIdentifier relativeLocation:(DSAgentLocation)relativeLocation delegate:(id)delegate;

- (BOOL)remotelyPerformSignature:(NSString *)signature;
- (BOOL)remotelyPerformSignature:(NSString *)signature withObject:(id)object;
- (BOOL)remotelyPerformSignature:(NSString *)signature withObject:(id)object1 object:(id)object2;
- (BOOL)remotelyPerformSignature:(NSString *)signature withArguments:(NSArray *)arguments;

- (BOOL)isRunning;
- (void)stop;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark REMOTELY PERFORM CONVENIENCE METHODS
// ----------------------------------------------------------------------------------------

- (BOOL)remotelyPerformAgentDidLaunchWithRelease;
- (BOOL)remotelyPerformAgentWasWrongRelease;
- (BOOL)remotelyPerformAgentRequestsTermination;
- (BOOL)remotelyPerformAgentJustCheckingIn;
- (BOOL)remotelyPerformAgentDidChangeUpdatesPreferences;
- (BOOL)remotelyPerformAgentCheckForUpdates:(int)checkingMode runModal:(BOOL)runModal;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSString *)name;
- (void)setName:(NSString *)value;

- (NSString *)bundleIdentifier;
- (void)setBundleIdentifier:(NSString *)value;

- (BOOL)hasLaunchArguments;

- (BOOL)isSingleInstance;
- (void)setSingleInstance:(BOOL)value;

- (BOOL)shouldStopAgentWithParent;
- (void)setShouldStopAgentWithParent:(BOOL)value;

- (DSAgentLocation)relativeLocation;
- (void)setRelativeLocation:(DSAgentLocation)value;

- (NSString *)executablePath;

@end

