//
//  DSAgentPrefsUpdates.m
//  Shared classes
//
//  Created by David Sinclair on 2007-03-30.
//  Copyright 2007 Dejal Systems, LLC. All rights reserved.
//
//  This subclass of DSPrefsUpdates is for use in the Main app where the updates are processed
//  in a Status agent, to syncronize the user-settable preferences and the check-settable ones.
//


#import "DSAgentPrefsUpdates.h"
#import "MainController.h"


@implementation DSAgentPrefsUpdates

// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark MANAGEMENT METHODS
// ----------------------------------------------------------------------------------------

/*
 prepare
 
 This subclass copies all Updates preferences from the Status agent to the receiver, since the Status agent is where update checks are performed.
 
 Written by DJS 2007-03.
*/

- (BOOL)prepare;
{
    [[NSUserDefaults standardUserDefaults] copyPreferencesWithKeys:SU_ALL_PREFERENCE_KEYS fromBundleIdentifier:[[[MainController mainController] statusAgent] bundleIdentifier] removeIfMissing:NO];
    
    return [super prepare];
}

/*
 maintainControls
 
 This subclass notifies the Status agent to fetch the user-settable preferences from the receiver to its own prefs.
 
 Written by DJS 2007-03.
*/

- (BOOL)maintainControls;
{
    [[[MainController mainController] statusAgent] remotelyPerformAgentDidChangeUpdatesPreferences];
    
    return [super maintainControls];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

/*
 nibName
 
 Returns the Nib name to use for these preferences.  Uses the superclass' name, since that's what the Nib is called.
 
 Written by DJS 2007-03.
*/

- (NSString *)nibName;
{
    return @"DSPrefsUpdates";
}

@end

