//
//  DSExceptionController.m
//  Shared classes
//
//  Created by David Sinclair on 2007-01-31.
//  Copyright (c) 2007 Dejal Systems, LLC. All rights reserved.
//

#import "DSExceptionController.h"
#import <ExceptionHandling/NSExceptionHandler.h>


@implementation DSExceptionController

+ (DSExceptionController *)sharedExceptionController;
{
    static DSExceptionController *sharedExceptionController = nil;
    
    if (sharedExceptionController == nil)
        sharedExceptionController = [[self alloc] init];
    
    return sharedExceptionController;
}

- (id)init;
{
    self = [super init];
    
    [[NSExceptionHandler defaultExceptionHandler] setDelegate:self];
    [[NSExceptionHandler defaultExceptionHandler] setExceptionHandlingMask:NSLogAndHandleEveryExceptionMask];
    
    return self;
}

- (void)dealloc;
{
    [[NSExceptionHandler defaultExceptionHandler] setDelegate:nil];
    
    [super dealloc];
}

/*
 shouldDisplayException:
 
 Returns YES if it is an exception we're interested in, or NO if it is a common one we want to ignore.
 
 Written by DJS 2007-01.
*/

- (BOOL)shouldDisplayException:(NSException *)exception;
{
/*
    NSString* name = [exception name];
    NSString* reason = [exception reason];
    
    if ([name isEqualToString:@"NSImageCacheException"] ||
        [name isEqualToString:@"GIFReadingException"] ||
        [name isEqualToString:@"NSRTFException"] ||
        ([name isEqualToString:@"NSInternalInconsistencyException"] && [reason hasPrefix:
            @"lockFocus"])
        )
    {
        return NO;
    }
*/
    
    return YES;
}

/*
 exceptionHandler:shouldLogException:mask:
 
 NSExceptionHandler delegate method that controls whether to log the exception in the console.  We don't want that, as we log it ourselves.
 
 Written by DJS 2007-01.
*/

- (BOOL)exceptionHandler:(NSExceptionHandler *)sender shouldLogException:(NSException *)exception 
                    mask:(NSUInteger)aMask;
{
    return NO;
}

/*
 exceptionHandler:shouldHandleException:mask:
 
 NSExceptionHandler delegate method where we can do something useful with the exception.  We log the stack trace, if it is an exception we're interested in.
 
 Written by DJS 2007-01, based on code from <http://www.cocoadev.com/index.pl?StackTraces>.
*/

- (BOOL)exceptionHandler:(NSExceptionHandler *)sender shouldHandleException:(NSException *)exception 
                    mask:(NSUInteger)aMask;
{
    if (![self shouldDisplayException:exception])
        return YES;
    
    NSString *stackTrace = [[exception userInfo] objectForKey:NSStackTraceKey];
    NSString *str = [NSString stringWithFormat:@"/usr/bin/atos -p %d %@ | tail -n +3 | head -n +%d | c++filt | cat -n",
        [[NSProcessInfo processInfo] processIdentifier],
        stackTrace,
        ([[stackTrace componentsSeparatedByString:@"  "] count] - 4)];
    FILE *file = popen([str UTF8String], "r");
    
    if (file)
    {
        char buffer[512];
        size_t length;
        
        NSLog(@"%@: %@\n", [exception name], [exception reason]);
        
//        fprintf(stderr, "An exception of type %s occured.\n%s\n", [[exception name] cString], [[exception reason] cString]);
//        fprintf(stderr, "Stack trace:\n");
        
        while ((length = fread(buffer, 1, sizeof(buffer), file)))
        {
            fwrite(buffer, 1, length, stderr);
        }
        
        pclose(file);
    }
    
    return YES;
}

@end

