//
//  DSSharedBaseDefines.h
//  Shared classes
//
//  Copyright 2002-2008 Dejal Systems, LLC. All rights reserved.
//


// Shortcuts:

#define PREFERENCES [NSUserDefaults standardUserDefaults]
#define NOTIFICATION [NSNotificationCenter defaultCenter]
#define INFODICT [[NSBundle mainBundle] infoDictionary]
#define SYSTEMVERSION [NSDictionary dictionaryWithContentsOfFile:@"/System/Library/CoreServices/SystemVersion.plist"]


// Intervals:

#define INTERVAL_NEVER          0
#define INTERVAL_SECOND         1
#define INTERVAL_MINUTE         60
#define INTERVAL_HOUR           (60 * 60)
#define INTERVAL_DAY            (60 * 60 * 24)
#define INTERVAL_WEEK           (60 * 60 * 24 * 7)
#define INTERVAL_MONTH          (60 * 60 * 24 * 365 / 12)
#define INTERVAL_QUARTER        (60 * 60 * 24 * 365 / 4)
#define INTERVAL_YEAR           (60 * 60 * 24 * 365)
#define INTERVAL_FOREVER        INT_MAX

#define UNITS_SECOND            0
#define UNITS_MINUTE            1
#define UNITS_HOUR              2
#define UNITS_DAY               3
#define UNITS_WEEK              4
#define UNITS_MONTH             5
#define UNITS_QUARTER           6
#define UNITS_YEAR              7
#define UNITS_NEVER             8
#define UNITS_FOREVER           9


// See also the DSIntervalData class.

// Given an amount and a units constant (e.g. 5, UNITS_HOUR), returns the corresponding interval (number of seconds), e.g. 18000.

static inline NSInteger IntervalForAmountUnits(NSInteger amount, NSInteger units)
{
    switch (units)
    {
        case UNITS_MINUTE:
            return amount * INTERVAL_MINUTE;
        case UNITS_HOUR:
            return amount * INTERVAL_HOUR;
        case UNITS_DAY:
            return amount * INTERVAL_DAY;
        case UNITS_WEEK:
            return amount * INTERVAL_WEEK;
        case UNITS_MONTH:
            return amount * INTERVAL_MONTH;
        case UNITS_YEAR:
            return amount * INTERVAL_YEAR;
        default:
            return amount * INTERVAL_SECOND;
    }
}

