//
//  DSSheetController.h
//  Shared classes
//
//  Created by David Sinclair on Fri Jan 09 2004.
//  Copyright (c) 2004 - 2006 Dejal Systems, LLC. All rights reserved.
//


@interface DSSheetController : NSWindowController
{
    NSString *_identifier;
    id _object;
    NSMutableDictionary *_userInfo;
    BOOL _isAltering;
    BOOL _isModal;
    id _delegate;
    SEL _didEndSelector;
    id _contextInfo;
    BOOL _canRecycle;
    
    IBOutlet NSImageView *iconImageView;
    IBOutlet NSTextField *promptStat;
    IBOutlet NSButton *okayButton;
    IBOutlet NSButton *cancelButton;
}

+ (id)controller;
+ (id)controllerForObject:(id)object;
+ (id)controllerForObject:(id)object modelessDelegate:(id)modelessDelegate didEndSelector:(SEL)didEndSelector contextInfo:(void *)contextInfo;

- (void)setup;
- (void)recycle;

+ (id)topmostController;
+ (id)findControllerForObject:(id)object;


- (id)beginSheetModalForWindow:(NSWindow *)docWindow modalDelegate:(id)modalDelegate didEndSelector:(SEL)didEndSelector contextInfo:(void *)contextInfo;
- (id)beginWindowForObject:(id)object modelessDelegate:(id)modelessDelegate didEndSelector:(SEL)didEndSelector contextInfo:(void *)contextInfo;

- (BOOL)prepare;
- (void)showWindow;
- (BOOL)maintainButtons;
- (BOOL)validate;
- (BOOL)shouldEndWithReturnCode:(NSInteger)returnCode;

- (void)endWithReturnCode:(NSInteger)returnCode;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TARGET/ACTION METHODS
// ----------------------------------------------------------------------------------------

- (IBAction)cancel:(id)sender;
- (IBAction)okay:(id)sender;
- (IBAction)help:(id)sender;
- (IBAction)hit:(id)sender;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TEXTFIELD DELEGATE METHODS
// ----------------------------------------------------------------------------------------

- (void)controlTextDidChange:(NSNotification *)notification;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark USER INTERFACE ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

+ (NSString *)nibName;
- (NSString *)helpBookPath;

- (NSImage *)icon;
- (void)setIcon:(NSImage *)value;

- (NSString *)prompt;
- (void)setPrompt:(NSString *)value;

- (NSString *)cancelTitle;
- (void)setCancelTitle:(NSString *)value;

- (NSString *)okayTitle;
- (void)setOkayTitle:(NSString *)value;

- (NSString *)windowTitle;
- (void)setWindowTitle:(NSString *)value;

- (void)setAddWindowTitle:(NSString *)addWindowTitle addPrompt:(NSString *)addPrompt
             addOkayTitle:(NSString *)addOkayTitle addIconNamed:(NSString *)addIconName
         alterWindowTitle:(NSString *)alterWindowTitle alterPrompt:(NSString *)alterPrompt
           alterOkayTitle:(NSString *)alterOkayTitle alterIconNamed:(NSString *)alterIconName;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark DATA ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSString *)identifier;
- (void)setIdentifier:(NSString *)value;

- (id)object;
- (void)setObject:(id)value;

- (NSMutableDictionary *)userInfo;
- (void)setUserInfo:(NSMutableDictionary *)value;

- (BOOL)isAltering;
- (void)setAltering:(BOOL)value;

- (BOOL)isModal;

- (id)delegate;
- (void)setDelegate:(id)value;

- (SEL)didEndSelector;
- (void)setDidEndSelector:(SEL)value;

- (id)contextInfo;
- (void)setContextInfo:(id)value;

- (BOOL)canRecycle;
- (void)setCanRecycle:(BOOL)value;

@property (retain,getter=userInfo) NSMutableDictionary *_userInfo;
@property (retain,getter=identifier) NSString *_identifier;
@property (getter=isModal) BOOL _isModal;
@property (getter=didEndSelector,setter=setDidEndSelector:) SEL _didEndSelector;
@property (retain,getter=object) id _object;
@property (getter=canRecycle,setter=setCanRecycle:) BOOL _canRecycle;
@property (retain,getter=contextInfo) id _contextInfo;
@property (getter=isAltering,setter=setAltering:) BOOL _isAltering;
@property (retain,getter=delegate) id _delegate;
@end
