//
//  DSWebController.h
//  Shared classes
//
//  Created by David Sinclair on Fri Oct 03 2003.
//  Copyright (c) 2003 - 2007 Dejal Systems, LLC. All rights reserved.
//

#import <WebKit/WebKit.h>


@interface DSWebController : NSObject
{
    WebView *_webView;
    NSTextField *_urlField;
    NSProgressIndicator *_progressIndicator;
    NSTextField *_statusStat;
    
    NSImage *_icon;
    NSString *_title;
    BOOL _isLoading;
    BOOL _isClearing;
}

+ (id)webControllerForWebView:(WebView *)webView urlField:(NSTextField *)urlField
            progressIndicator:(NSProgressIndicator *)progressIndicator statusStat:(NSTextField *)statusStat;

- (id)initForWebView:(WebView *)webView urlField:(NSTextField *)urlField
   progressIndicator:(NSProgressIndicator *)progressIndicator statusStat:(NSTextField *)statusStat;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark WEB FRAME LOAD DELEGATE METHODS
// ----------------------------------------------------------------------------------------

- (void)webView:(WebView *)sender didStartProvisionalLoadForFrame:(WebFrame *)frame;
// - (void)webView:(WebView *)sender didReceiveServerRedirectForProvisionalLoadForFrame:(WebFrame *)frame;
- (void)webView:(WebView *)sender didCommitLoadForFrame:(WebFrame *)frame;

- (void)webView:(WebView *)sender didFailProvisionalLoadWithError:(NSError *)error forFrame:(WebFrame *)frame;
- (void)webView:(WebView *)sender didFailLoadWithError:(NSError *)error forFrame:(WebFrame *)frame;

- (void)webView:(WebView *)sender didFinishLoadForFrame:(WebFrame *)frame;

- (void)webView:(WebView *)sender didReceiveIcon:(NSImage *)image forFrame:(WebFrame *)frame;
- (void)webView:(WebView *)sender didReceiveTitle:(NSString *)title forFrame:(WebFrame *)frame;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark WEB POLICY DELEGATE METHODS
// ----------------------------------------------------------------------------------------

- (void)webView:(WebView *)sender decidePolicyForNavigationAction:(NSDictionary *)actionInformation request:(NSURLRequest *)request frame:(WebFrame *)frame decisionListener:(id)listener;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark UTILITY METHODS
// ----------------------------------------------------------------------------------------

- (NSString *)source;

- (void)loadFromSource:(NSString *)source baseURL:(NSURL *)baseURL;

- (void)clearPage;
- (void)displayPageForError:(NSString *)error;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (WebView *)webView;
- (void)setWebView:(WebView *)value;

- (NSTextField *)urlField;
- (void)setURLField:(NSTextField *)value;

- (NSProgressIndicator *)progressIndicator;
- (void)setProgressIndicator:(NSProgressIndicator *)value;

- (NSTextField *)statusStat;
- (void)setStatusStat:(NSTextField *)value;


- (NSImage *)icon;
- (void)setIcon:(NSImage *)value;

- (NSString *)title;
- (void)setTitle:(NSString *)value;

- (BOOL)isLoading;
- (void)setIsLoading:(BOOL)value;

- (BOOL)isClearing;
- (void)setIsClearing:(BOOL)value;

- (BOOL)willDeferLinksToBrowser;
- (void)setDeferLinksToBrowser:(BOOL)value;

- (WebFrame *)mainFrame;
- (WebDataSource *)mainDataSource;

@end
