//
//  DSWebController.m
//  Shared classes
//
//  Created by David Sinclair on Fri Oct 03 2003.
//  Copyright (c) 2003 - 2007 Dejal Systems, LLC. All rights reserved.
//

#import "DSWebController.h"


@implementation DSWebController

/*
 webControllerForWebView:urlField:progressIndicator:statusStat:
 
 Convenience method to return a new autoreleased web controller.  All parameters other than the webView may be nil if not required.
 
 Written by DJS 2003-10.
*/

+ (id)webControllerForWebView:(WebView *)webView urlField:(NSTextField *)urlField
            progressIndicator:(NSProgressIndicator *)progressIndicator statusStat:(NSTextField *)statusStat
{
    return [[[self alloc] initForWebView:webView urlField:urlField
                       progressIndicator:progressIndicator statusStat:statusStat] autorelease];
}

/*
 initForWebView:urlField:progressIndicator:
 
 Designator initializer.  All parameters other than the webView may be nil if not required.
 
 Written by DJS 2003-10.
*/

- (id)initForWebView:(WebView *)webView urlField:(NSTextField *)urlField
   progressIndicator:(NSProgressIndicator *)progressIndicator statusStat:(NSTextField *)statusStat
{
    self = [self init];

    [self setWebView:webView];
    [self setURLField:urlField];
    [self setProgressIndicator:progressIndicator];
    [self setStatusStat:statusStat];

    [self setIcon:nil];
    [self setIsLoading:NO];
    [self setIsClearing:NO];

    [[self webView] setFrameLoadDelegate:self];
    
    // bonus retain so this object is never deallocated, as an extremely hacky workaround to crashing in WebKit, which I suspect may be related to still accessing this class for some reason:
//    [self retain];
    
    return self;
}

- (void)dealloc
{
    [[self webView] stopLoading:nil];
    
    [[self webView] setDownloadDelegate:nil];
    [[self webView] setFrameLoadDelegate:nil];
    [[self webView] setResourceLoadDelegate:nil];
    [[self webView] setPolicyDelegate:nil];
    [[self webView] setUIDelegate:nil];
    [[self webView] setHostWindow:nil];
    
    [self setWebView:nil];
    [self setURLField:nil];
    [self setProgressIndicator:nil];
    [self setStatusStat:nil];

    [self setIcon:nil];
    [self setIsLoading:NO];
    [self setIsClearing:NO];

    [super dealloc];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark WEB FRAME LOAD DELEGATE METHODS
// ----------------------------------------------------------------------------------------

- (void)webView:(WebView *)sender didStartProvisionalLoadForFrame:(WebFrame *)frame
{
    // Only report feedback for the main frame:
    if (frame == [sender mainFrame])
    {
        NSString *url = [[[[frame provisionalDataSource] request] URL] absoluteString];
        
        [self setIsLoading:YES];
        [self setIcon:nil];
        
        [[self progressIndicator] startAnimation:nil];
        [[self statusStat] setStringValue:[NSString stringWithFormat:NSLocalizedStringFromTable(@"Contacting <%@>...", @"DSShared", @"WebView status contacting server"), [[NSURL URLWithString:url] host]]];
        
//        NSLog(@"didStartProvisionalLoadForFrame; url = %@", url);		//  log
        
        if ([self urlField] && ![url isEqualToString:@"http://www.dejal.com/invalid"] && ![url isEqualToString:[[self urlField] stringValue]])
        {
            [[self urlField] setStringValue:url];
            [[NSNotificationCenter defaultCenter] postNotificationName:@"DSWebURLChangedNotification" object:self];
        }
    }
}

/*
- (void)webView:(WebView *)sender didReceiveServerRedirectForProvisionalLoadForFrame:(WebFrame *)frame
{
    // Only care about the main frame:
    if (frame == [sender mainFrame])
    {
        NSString *url = [[[[frame provisionalDataSource] request] URL] absoluteString];
        
//        NSLog(@"didReceiveServerRedirectForProvisionalLoadForFrame; url = %@", url);		//  log
    }
}
*/

- (void)webView:(WebView *)sender didCommitLoadForFrame:(WebFrame *)frame
{
    // Only report feedback for the main frame:
    if (frame == [sender mainFrame])
    {
        NSString *url = [[[[frame dataSource] request] URL] absoluteString];
        
        [[self statusStat] setStringValue:[NSString stringWithFormat:NSLocalizedStringFromTable(@"Loading <%@>...", @"DSShared", @"WebView status loading page"), url]];
        
        [[NSNotificationCenter defaultCenter] postNotificationName:@"DSWebStartedLoadingNotification" object:self];
    }
}

- (void)webView:(WebView *)sender didFailProvisionalLoadWithError:(NSError *)error forFrame:(WebFrame *)frame
{
    // Only report feedback for the main frame:
    if (frame == [sender mainFrame])
    {
//        NSLog(@"didFailProvisionalLoadWithError = %@ (%d)", [error localizedDescription], [error code]);		//  log
        
        if ([error code] == -999)
        {
            [self setIsLoading:NO];
            [self setIsClearing:NO];
            [[self progressIndicator] stopAnimation:nil];
            [[self statusStat] setStringValue:@""];
        }
        else
        {
            NSString *message = [NSLocalizedStringFromTable(@"Error contacting server: ", @"DSShared", @"WebView status contacting error") stringByAppendingString:[error localizedDescription]];
            
            [self displayPageForError:message];
        }
    }
}

- (void)webView:(WebView *)sender didFailLoadWithError:(NSError *)error forFrame:(WebFrame *)frame
{
    // Only report feedback for the main frame:
    if (frame == [sender mainFrame])
    {
//        NSLog(@"didFailLoadWithError = %@ (%d)", [error localizedDescription], [error code]);		//  log
        
        if ([error code] == -999)
        {
            [self setIsLoading:NO];
            [self setIsClearing:NO];
            [[self progressIndicator] stopAnimation:nil];
            [[self statusStat] setStringValue:@""];
        }
        else
        {
            NSString *message = [NSLocalizedStringFromTable(@"Error loading page: ", @"DSShared", @"WebView status load error") stringByAppendingString:[error localizedDescription]];
            
            [self displayPageForError:message];
        }
    }
}

- (void)webView:(WebView *)sender didFinishLoadForFrame:(WebFrame *)frame
{
    // Only report feedback for the main frame:
    if (frame == [sender mainFrame])
    {
        [self setIsLoading:NO];
        [self setIsClearing:NO];
        [[self progressIndicator] stopAnimation:nil];
        [[self statusStat] setStringValue:@""];
        
        [[NSNotificationCenter defaultCenter] postNotificationName:@"DSWebFinishedLoadingNotification" object:self];
    }
}

- (void)webView:(WebView *)sender didReceiveIcon:(NSImage *)image forFrame:(WebFrame *)frame
{
    // Only report feedback for the main frame:
    if (frame == [sender mainFrame] && ![[[[[frame dataSource] request] URL] absoluteString] isEqualToString:@"http://www.dejal.com/invalid"])
    {
        [self setIcon:image];
        
        [[NSNotificationCenter defaultCenter] postNotificationName:@"DSWebReceivedIconNotification"
                                                            object:self];
    }
}

- (void)webView:(WebView *)sender didReceiveTitle:(NSString *)title forFrame:(WebFrame *)frame
{
    if ([frame dataSource] == [self mainDataSource] && ![[[[[frame dataSource] request] URL] absoluteString] isEqualToString:@"http://www.dejal.com/invalid"])
    {
        [self setTitle:title];
        
        [[NSNotificationCenter defaultCenter] postNotificationName:@"DSWebReceivedTitleNotification"
                                                            object:self];
    }
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark WEB POLICY DELEGATE METHODS
// ----------------------------------------------------------------------------------------

- (void)webView:(WebView *)sender decidePolicyForNavigationAction:(NSDictionary *)actionInformation request:(NSURLRequest *)request frame:(WebFrame *)frame decisionListener:(id)listener
{
    int key = [[actionInformation objectForKey:WebActionNavigationTypeKey] intValue];
    
    switch (key)
    {
        case WebNavigationTypeLinkClicked:
            // Tell WebKit to ignore the link click:
            [listener ignore];
            
            // Open the URL in the user's web browser instead:
            [[NSWorkspace sharedWorkspace] openURL:[actionInformation objectForKey:WebActionOriginalURLKey]];
            break;
        case WebNavigationTypeOther:
            // Tell WebKit to use this action; required the load the page in the first place:
            [listener use];
            break;
        default:
            [listener ignore];
    }
} 


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark UTILITY METHODS
// ----------------------------------------------------------------------------------------

/*
 source
 
 Returns the source for the main frame, or nil if it isn't available.
 
 Written by DJS 2003-11.
*/

- (NSString *)source
{
    NSString *source = nil;
    WebFrame *mainFrame = [self mainFrame];
    id documentRepresentation = [[mainFrame dataSource] representation];
    
    if ([documentRepresentation canProvideDocumentSource])
        source = [documentRepresentation documentSource];
    
    return source;
}

- (void)loadFromSource:(NSString *)source baseURL:(NSURL *)baseURL
{
    WebFrame *mainFrame = [self mainFrame];
    NSString *urlFieldValue = [[self urlField] stringValue];
    
    if (!baseURL && [urlFieldValue length])
        baseURL = [NSURL URLWithString:urlFieldValue];
    
    // don't want this; sometimes we want a nil base url, e.g. for BlogAssist:
//    if (!baseURL)
//        baseURL = [NSURL URLWithString:@"file:///"];
    
    NSData *data = [source dataUsingEncoding:NSUTF8StringEncoding allowLossyConversion:YES];
    [mainFrame loadData:data MIMEType:@"text/html" textEncodingName:@"utf-8" baseURL:baseURL];
    
//    [mainFrame loadHTMLString:source baseURL:baseURL];
}

- (void)clearPage
{
    [[self webView] stopLoading:nil];
    [self setIsLoading:NO];
    [[self progressIndicator] stopAnimation:nil];
    [[self statusStat] setStringValue:@""];
    
//    NSLog(@"clearPage isClearing = %d", [self isClearing]);		//  log
    
    NSString *source = @"<html>\n<head>\n<title></title>\n</head>\n<body bgcolor=\"#ffffff\">\n</body>\n</html>\n";            //  hack
    
    [[self mainFrame] loadHTMLString:source baseURL:[NSURL URLWithString:@"http://www.dejal.com/invalid"]];
    
//    [self loadFromSource:source baseURL:nil];            //  hack

/*
    if ([self isClearing])
        [self setIsClearing:NO];
    else
    {
        [self setIsClearing:YES];
        
        NSString *source = @"<html>\n<head>\n<title></title>\n</head>\n<body bgcolor=\"#ffffff\">\n<p>Clear</p>\n</body>\n</html>\n";
        
//        [[self mainFrame] loadHTMLString:source baseURL:[NSURL URLWithString:@"file:///"]];
        
        [self loadFromSource:source baseURL:nil];
    }
*/
}

- (void)displayPageForError:(NSString *)error
{
    //  this doesn't work as yet!  The problem seems to be that this is called from the error delegate methods, and the WebView isn't in a mood to load a new page while handling an error from a previous load.
    
    [self setIsLoading:NO];
    [[self progressIndicator] stopAnimation:nil];
    [[self statusStat] setStringValue:error];
    
//    [[self mainFrame] stopLoading];
    
//    NSLog(@"displayPageForError = %@; isClearing = %d", error, [self isClearing]);		//  log
    
/*    if ([self isClearing])
        [self setIsClearing:NO];
    else
    {
        [self setIsClearing:YES];
*/        
//        NSString *source = [@"<html>\n<head>\n<title></title>\n</head>\n<body bgcolor=\"#ffffff\">\n<p><font color=\"#cc0000\" size=\"+2\">" stringByAppendingString:error];
//        source = [source stringByAppendingString:@"</font></p>\n</body>\n</html>\n"];
//        NSString *source = @"<html>\n<head>\n<title></title>\n</head>\n<body bgcolor=\"#ffffff\">\nOops.</body>\n</html>\n";            //  hack
        
//        [[self mainFrame] loadHTMLString:source baseURL:[NSURL URLWithString:@"http://www.dejal.com/invalid"]];
        
//        [self loadFromSource:source baseURL:nil];
//    }
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (WebView *)webView
{
    return _webView;
}

- (void)setWebView:(WebView *)value
{
    [value retain];
    [_webView release];
    _webView = value;
}

- (NSTextField *)urlField
{
    return _urlField;
}

- (void)setURLField:(NSTextField *)value
{
    [value retain];
    [_urlField autorelease];
    _urlField = value;
}

- (NSProgressIndicator *)progressIndicator
{
    return _progressIndicator;
}

- (void)setProgressIndicator:(NSProgressIndicator *)value
{
    [value retain];
    [_progressIndicator autorelease];
    _progressIndicator = value;
}

- (NSTextField *)statusStat
{
    return _statusStat;
}

- (void)setStatusStat:(NSTextField *)value
{
    [value retain];
    [_statusStat autorelease];
    _statusStat = value;
}


- (NSImage *)icon
{
    return _icon;
}

- (void)setIcon:(NSImage *)value
{
    [value retain];
    [_icon autorelease];
    _icon = value;
}

- (NSString *)title
{
    return _title;
}

- (void)setTitle:(NSString *)value
{
    [value retain];
    [_title autorelease];
    _title = value;
}

- (BOOL)isLoading
{
    return _isLoading;
}

- (void)setIsLoading:(BOOL)value
{
    _isLoading = value;
}

- (BOOL)isClearing
{
    return _isClearing;
}

- (void)setIsClearing:(BOOL)value
{
    _isClearing = value;
}

- (BOOL)willDeferLinksToBrowser
{
    return ([[self webView] policyDelegate] == self);
}

- (void)setDeferLinksToBrowser:(BOOL)value
{
    [[self webView] setPolicyDelegate:value ? self : nil];
}

- (WebFrame *)mainFrame
{
    return [[self webView] mainFrame];
}

- (WebDataSource *)mainDataSource
{
    return [[self mainFrame] dataSource];
}

@end
