//
//  DSDateData.m
//  Shared classes
//
//  Created by David Sinclair on 2005-11-06.
//  Copyright 2005-2008 Dejal Systems, LLC. All rights reserved.
//
//  See also DSIntervalData.
//

#import "DSDateData.h"
#import "DSSharedBaseDefines.h"
#import "DSStringCategories.h"


@implementation DSDateData

/*
 dateRelativeToNow:defaultValue:
 
 Given a date, returns a string with the interval between that date and now expressed with suitable units.  If the interval is over 10 years, it uses the default value.
 
 Written by DJS 2005-11.
*/

+ (NSString *)dateRelativeToNow:(NSDate *)date defaultValue:(NSString *)defaultValue;
{
    NSTimeInterval seconds = abs([date timeIntervalSinceNow]);
    
    return [self unitsForInterval:seconds defaultValue:defaultValue];
}

/*
 unitsForInterval:defaultValue:
 
 Given an interval (seconds), returns a string with the interval expressed with suitable units.  If the interval is over 10 years, it uses the default value.  (May want to extend it to support more years.)
 
 Written by DJS 2005-11.
*/

+ (NSString *)unitsForInterval:(NSTimeInterval)interval defaultValue:(NSString *)defaultValue;
{
    NSString *value;
    NSInteger seconds = interval;
    NSInteger minutes = abs(seconds / 60);   // The ancient dates are negative even after abs(), so do it again to be sure
    NSInteger hours = minutes / 60;
    NSInteger days = hours / 24;
    NSInteger weeks = days / 7;
    NSInteger months = days / 31;
    
    if (months > 120)
        value = defaultValue;
    else if (months > 0)
        value = [NSString stringWithFormat:@"%d %@",
            months, [NSString stringWithIntValue:months zero:nil
                                        singluar:NSLocalizedStringFromTable(@"month", @"DSShared", @"month singular")
                                          plural:NSLocalizedStringFromTable(@"months", @"DSShared", @"month plural")]];
    else if (weeks > 0)
        value = [NSString stringWithFormat:@"%d %@",
            weeks, [NSString stringWithIntValue:weeks zero:nil
                                       singluar:NSLocalizedStringFromTable(@"week", @"DSShared", @"week singular")
                                         plural:NSLocalizedStringFromTable(@"weeks", @"DSShared", @"week plural")]];
    else if (days > 0)
        value = [NSString stringWithFormat:@"%d %@",
            days, [NSString stringWithIntValue:days zero:nil
                                      singluar:NSLocalizedStringFromTable(@"day", @"DSShared", @"day singular")
                                        plural:NSLocalizedStringFromTable(@"days", @"DSShared", @"day plural")]];
    else if (hours > 0)
    {
        minutes = minutes % 60;
        
        value = [NSString stringWithFormat:@"%d %@ %d %@",
            hours, [NSString stringWithIntValue:hours zero:nil
                                       singluar:NSLocalizedStringFromTable(@"hour", @"DSShared", @"hour singular")
                                         plural:NSLocalizedStringFromTable(@"hours", @"DSShared", @"hour plural")],
            minutes, [NSString stringWithIntValue:minutes zero:nil
                                         singluar:NSLocalizedStringFromTable(@"min", @"DSShared", @"minute singular")
                                           plural:NSLocalizedStringFromTable(@"mins", @"DSShared", @"minute plural")]];
    }
    else if (minutes > 0)
    {
        seconds = seconds % 60;
        
        value = [NSString stringWithFormat:@"%d %@ %d %@",
            minutes, [NSString stringWithIntValue:minutes zero:nil
                                         singluar:NSLocalizedStringFromTable(@"min", @"DSShared", @"minute singular")
                                           plural:NSLocalizedStringFromTable(@"mins", @"DSShared", @"minute plural")],
            seconds, [NSString stringWithIntValue:seconds zero:nil
                                         singluar:NSLocalizedStringFromTable(@"sec", @"DSShared", @"second singular")
                                           plural:NSLocalizedStringFromTable(@"secs", @"DSShared", @"second plural")]];
    }
    else
    {
        value = [NSString stringWithFormat:@"%d %@",
            seconds, [NSString stringWithIntValue:seconds zero:nil
                                         singluar:NSLocalizedStringFromTable(@"second", @"DSShared", @"second singular")
                                           plural:NSLocalizedStringFromTable(@"seconds", @"DSShared", @"second plural")]];
    }
    
    return value;
}

@end
