//
//  DSFTPStreamData.h
//  Shared classes
//
//  Created by David Sinclair on 2005-11-13.
//  Copyright 2005 Dejal Systems, LLC. All rights reserved.
//


#import "DSStreamData.h"


// 500 KB buffer:

#define kMyBufferSize 500 * 1024


@interface DSAbstractFTPStreamData : DSStreamData
{
    NSDictionary *proxyDict;
    int fileSize;
    int totalBytesWritten;
    int leftOverByteCount;
    uint8_t buffer[kMyBufferSize];
}

- (void)setProxyForStream:(NSStream *)stream;
- (void)setUsername:(NSString *)username andPassword:(NSString *)password forStream:(NSStream *)stream;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface DSListingFTPStreamData : DSAbstractFTPStreamData
{
}


@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface DSDownloadFTPStreamData : DSAbstractFTPStreamData
{
}


@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface DSUploadFTPStreamData : DSAbstractFTPStreamData
{
}

- (void)uploadPath:(NSString *)path
            toHost:(NSString *)host
         directory:(NSString *)directory
          username:(NSString *)username
          password:(NSString *)password
       forDelegate:(id)delegate
     useSameThread:(BOOL)useSameThread;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface DSFTPData : NSObject
{
    DSUploadFTPStreamData *_streamData;
    NSMutableArray *_uploadInfo;
    NSArray *_cachedInfo;
    NSString *_host;
    NSString *_username;
    NSString *_password;
    int _index;
    BOOL _isWorking;
}

+ (id)ftpData;

- (void)uploadPath:(NSString *)path
            toHost:(NSString *)host
         directory:(NSString *)directory
          username:(NSString *)username
          password:(NSString *)password
    cachedInfoPath:(NSString *)cachedInfoPath;

- (void)stop;


- (NSArray *)cachedInfo;
- (void)setCachedInfo:(NSArray *)value;

- (NSString *)host;
- (void)setHost:(NSString *)value;

- (NSString *)username;
- (void)setUsername:(NSString *)value;

- (NSString *)password;
- (void)setPassword:(NSString *)value;

- (int)index;
- (void)resetIndex;
- (void)incrementIndex;

- (int)count;

- (BOOL)isMore;

- (BOOL)isWorking;
- (void)setWorking:(BOOL)value;

- (NSDictionary *)currentInfo;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface NSObject (DSUploadFTPStreamDataInformalProtocol)

/*
 streamData:uploadedBytes:
 
 Informs the delegate that this many bytes have been uploaded.
*/

- (void)streamData:(id)streamData uploadedBytes:(NSNumber *)bytes;

@end

