//
//  DSFTPStreamData.m
//  Shared classes
//
//  Created by David Sinclair on 2005-11-13.
//  Copyright 2005 Dejal Systems, LLC. All rights reserved.
//


#import "DSFTPStreamData.h"
#import "DSArrayCategories.h"
#import "DSDictionaryCategories.h"
#import "DSFileManagerCategories.h"
#import <CoreServices/CoreServices.h>
#import <CoreFoundation/CoreFoundation.h>
#import <SystemConfiguration/SystemConfiguration.h>

#import <sys/dirent.h>
#import <sys/stat.h>
#import <unistd.h>         // getopt
#import <string.h>         // strmode
#import <stdlib.h>


@implementation DSAbstractFTPStreamData

- (id)init;
{
    self = [super init];
    
    proxyDict = nil;
    fileSize = 0;
    totalBytesWritten = 0;
    leftOverByteCount = 0;
    
    return self;
}

- (void)dealloc;
{
    [proxyDict release];
    
    [super dealloc];
}

- (void)setProxyForStream:(NSStream *)stream;
{
    if (!stream)
        return;
    
    proxyDict = (NSDictionary *)SCDynamicStoreCopyProxies(NULL);
    
    if (!proxyDict)
        return;
    
    NSNumber *passiveMode = [proxyDict objectForKey:(NSString *)kSCPropNetProxiesFTPPassive];
    
    [stream setProperty:proxyDict forKey:(NSString *)kCFStreamPropertyFTPProxy /*NSStreamSOCKSProxyConfigurationKey*/];
    [stream setProperty:passiveMode forKey:(NSString *)kCFStreamPropertyFTPUsePassiveMode];
}

- (void)setUsername:(NSString *)username andPassword:(NSString *)password forStream:(NSStream *)stream;
{
    if (!stream || ![username length])
        return;
    
    [stream setProperty:username forKey:(NSString *)kCFStreamPropertyFTPUserName];
    [stream setProperty:password forKey:(NSString *)kCFStreamPropertyFTPPassword];
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation DSListingFTPStreamData

/*
 static Boolean
 MySimpleDirectoryListing(CFStringRef urlString, CFStringRef username, CFStringRef password)
 {
     CFReadStreamRef        readStream;
     CFStreamClientContext  context = { 0, NULL, NULL, NULL, NULL };
     CFURLRef               downloadURL;
     Boolean                success = true;
     MyStreamInfo           *streamInfo;
     
     assert(urlString != NULL);
     
     downloadURL = CFURLCreateWithString(kCFAllocatorDefault, urlString, NULL);
     assert(downloadURL != NULL);
     
     // Create an FTP read stream for downloading operation from an FTP URL.
     readStream = CFReadStreamCreateWithFTPURL(kCFAllocatorDefault, downloadURL);
     assert(readStream != NULL);
     CFRelease(downloadURL);
     
     // Initialize our MyStreamInfo structure, which we use to store some information about the stream.
     MySetupStreamInfo(&streamInfo, readStream, NULL);
     context.info = (void *)streamInfo;
     
     // CFReadStreamSetClient registers a callback to hear about interesting events that occur on a stream.
     success = CFReadStreamSetClient(readStream, kNetworkEvents, MyDirectoryListingCallBack, &context);
     if (success) {
         
         // Schedule a run loop on which the client can be notified about stream events.  The client callback will be triggered via the run loop.  It's the caller's responsibility to ensure that the run loop is running.
         CFReadStreamScheduleWithRunLoop(readStream, CFRunLoopGetCurrent(), kCFRunLoopCommonModes);
         
         MyCFStreamSetUsernamePassword(readStream, username, password);
         MyCFStreamSetFTPProxy(readStream, streamInfo);
         
         // CFReadStreamOpen will return success/failure.  Opening a stream causes it to reserve all the system resources it requires.  If the stream can open non-blocking, this will always return TRUE; listen to the run loop source to find out when the open completes and whether it was successful. success = CFReadStreamOpen(readStream);
         if (success == false) {
             fprintf(stderr, "CFReadStreamOpen failed\n");
             MyCleanupStreamInfo(streamInfo);
         }
     } else {
         fprintf(stderr, "CFReadStreamSetClient failed\n");
         MyCleanupStreamInfo(streamInfo);
     }
     
     return success;
 }

 static void
 MyDirectoryListingCallBack(CFReadStreamRef readStream, CFStreamEventType type, void * clientCallBackInfo)
 {
     MyStreamInfo     *info = (MyStreamInfo *)clientCallBackInfo;
     UInt8            *bufRemaining;
     CFIndex          bufSize = 0;
     CFIndex          totalBytesRead = 0, bytesRead;
     CFIndex          totalBytesConsumed, bytesConsumed;
     CFStreamError    error;
     CFDictionaryRef  parsedDict;
     
     assert(readStream != NULL);
     assert(info       != NULL);
     assert(info->readStream == readStream);
     
     switch (type) {
         
         case kCFStreamEventOpenCompleted:
             fprintf(stderr, "Open complete\n");
             break;
         case kCFStreamEventHasBytesAvailable:
             
             do {
                 int offset = info->leftOverByteCount;  // If previous loop had unparsed data.
                 
                 // CFReadStreamRead will return the number of bytes read, or -1 if an error occurs preventing any bytes from being read, or 0 if the stream's end was encountered.
                 bytesRead = CFReadStreamRead(info->readStream, info->buffer + offset, kMyBufferSize - offset);
                 if (bytesRead < 0) {
                     fprintf(stderr, "CFReadStreamRead returned %ld\n", bytesRead);
                     break;
                 } else if (bytesRead == 0) {
                     // Note that we break out of the loop instead of relying on CFReadStreamHasBytesAvailable to return 'false' because in some cases, CFReadStreamHasBytesAvailable will return 'true' even though CFReadStreamRead returned 0.
                     break;
                 }
                 
                 bufSize = bytesRead + offset;
                 totalBytesRead += bufSize;
                 
                 totalBytesConsumed = 0;
                 bytesConsumed = 0;
                 
                 do
                 {
                     bufRemaining = info->buffer + totalBytesConsumed;
                     
                     // CFFTPCreateParsedResourceListing parses a line of file or folder listing
                     of Unix format, and stores the extracted result in a CFDictionary.
                     bytesConsumed = CFFTPCreateParsedResourceListing(NULL, bufRemaining, bufSize, &parsedDict);
                     if (bytesConsumed > 0) {
                         
                         MyPrintDirectoryListing(parsedDict);
                         CFRelease(parsedDict);
                         
                         totalBytesConsumed += bytesConsumed;
                         bufSize -= bytesConsumed;
                         info->leftOverByteCount = bufSize;
                         
                     } else if (bytesConsumed == 0) {
                         // This should never happen because we supply a pretty large buffer.
                         info->leftOverByteCount = bufSize;
                         totalBytesRead -= info->leftOverByteCount;
                         memmove(info->buffer, bufRemaining, info->leftOverByteCount);
                     } else if (bytesConsumed == -1) {
                         fprintf(stderr, "CFFTPCreateParsedResourceListing parse failure\n");
                         goto exit;
                     }
                     
                 } while (bytesConsumed > 0);
                 
             } while (CFReadStreamHasBytesAvailable(info->readStream));
             
             fprintf(stderr, "Read %ld bytes\n", totalBytesRead);
             break;
         case kCFStreamEventErrorOccurred:
             error = CFReadStreamGetError(info->readStream);
             fprintf(stderr, "CFReadStreamGetError returned (%d, %ld)\n", error.domain, error.error);
             goto exit;
         case kCFStreamEventEndEncountered:
             fprintf(stderr, "Listing complete\n");
             goto exit;
         default:
             fprintf(stderr, "Received unexpected CFStream event (%d)", type);
             break;
     }
     return;
     
exit:
         MyCleanupStreamInfo(info);
     CFRunLoopStop(CFRunLoopGetCurrent());
     return;
 }

 static void
 MyPrintDirectoryListing(CFDictionaryRef dictionary)
 {
     CFDateRef             cfModDate;
     CFDateFormatterRef    format;
     CFLocaleRef           locale;
     CFDateFormatterStyle  style;
     CFNumberRef           cfType, cfMode, cfSize;
     CFStringRef           cfOwner, cfName, cfLink, cfGroup, cfDate;
     char                  owner[256], group[256], date[256], name[256];
     char                  permString[12], link[1024] = "\x0";
     SInt64                size;
     SInt32                mode, type;
     
     assert(dictionary != NULL);
     
     // You should not assume that the Directory Listing dictionary will contain all the possible keys.
         Most of the time it will, however, depending on the FTP server, some of the keys may be missing.
     
     cfType = CFDictionaryGetValue(dictionary, kCFFTPResourceType);
     if (cfType) {
         assert(CFGetTypeID(cfType) == CFNumberGetTypeID());
         CFNumberGetValue(cfType, kCFNumberSInt32Type, &type);
         
         cfMode = CFDictionaryGetValue(dictionary, kCFFTPResourceMode);
         if (cfMode) {
             assert(CFGetTypeID(cfMode) == CFNumberGetTypeID());
             CFNumberGetValue(cfMode, kCFNumberSInt32Type, &mode);
             
             // Converts inode status information into a symbolic string
             strmode(mode + DTTOIF(type), permString);
             
             fprintf(stderr, "%s ", permString);
         }
     }
     
     cfOwner = CFDictionaryGetValue(dictionary, kCFFTPResourceOwner);
     if (cfOwner) {
         assert(CFGetTypeID(cfOwner) == CFStringGetTypeID());
         CFStringGetCString(cfOwner, owner, 256, kCFStringEncodingASCII);
         fprintf(stderr, "%9s", owner);
     }
     
     cfGroup = CFDictionaryGetValue(dictionary, kCFFTPResourceGroup);
     if (cfGroup) {
         assert(CFGetTypeID(cfGroup) == CFStringGetTypeID());
         CFStringGetCString(cfGroup, group, 256, kCFStringEncodingASCII);
         fprintf(stderr, "%9s", group);
     }
     
     cfSize = CFDictionaryGetValue(dictionary, kCFFTPResourceSize);
     if (cfSize) {
         assert(CFGetTypeID(cfSize) == CFNumberGetTypeID());
         CFNumberGetValue(cfSize, kCFNumberSInt64Type, &size);
         fprintf(stderr, "%9lld ", size);
     }
     
     cfModDate = CFDictionaryGetValue(dictionary, kCFFTPResourceModDate);
     if (cfModDate) {
         assert(CFGetTypeID(cfModDate) == CFDateGetTypeID());
         locale = CFLocaleCopyCurrent();
         assert(locale != NULL);
         
         style = kCFDateFormatterShortStyle;
         format = CFDateFormatterCreate(kCFAllocatorDefault, locale, style, style);
         assert(format != NULL);
         CFRelease(locale);
         
         cfDate = CFDateFormatterCreateStringWithDate(kCFAllocatorDefault, format, cfModDate);
         assert(cfDate != NULL);
         CFRelease(format);
         
         CFStringGetCString(cfDate, date, 256, kCFStringEncodingUTF8);
         fprintf(stderr, "%18s ", date);
     }
     
     // Note that this sample assumes UTF-8 since that's what the Mac OS X FTP server returns, however, some servers may use a different encoding.
     cfName = CFDictionaryGetValue(dictionary, kCFFTPResourceName);
     if (cfName) {
         assert(CFGetTypeID(cfName) == CFStringGetTypeID());
         CFStringGetCString(cfName, name, 256, kCFStringEncodingUTF8);
         fprintf(stderr, "%s", name);
         
         cfLink = CFDictionaryGetValue(dictionary, kCFFTPResourceLink);
         if (cfLink) {
             assert(CFGetTypeID(cfLink) == CFStringGetTypeID());
             CFStringGetCString(cfLink, link, 1024, kCFStringEncodingUTF8);
             if (strlen(link) > 0) fprintf(stderr, " -> %s", link);
         }
     }
     
     fprintf(stderr, "\n");
 }
*/

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation DSDownloadFTPStreamData

/*
static Boolean
MySimpleDownload(CFStringRef urlString, CFURLRef destinationFolder, CFStringRef username, CFStringRef password)
{
    CFReadStreamRef        readStream;
    CFWriteStreamRef       writeStream;
    CFStreamClientContext  context = { 0, NULL, NULL, NULL, NULL };
    CFURLRef               downloadPath, downloadURL;
    CFStringRef            fileName;
    Boolean                dirPath, success = true;
    MyStreamInfo           *streamInfo;
    
    assert(urlString != NULL);
    assert(destinationFolder != NULL);
    
    // Returns true if the CFURL path represents a directory.
    dirPath = CFURLHasDirectoryPath(destinationFolder);
    if (!dirPath) {
        fprintf(stderr, "Download destination must be a directory.\n");
        return false;
    }
    
    // Create a CFURL from the urlString.
    downloadURL = CFURLCreateWithString(kCFAllocatorDefault, urlString, NULL);
    assert(downloadURL != NULL);
    
    // Copy the end of the file path and use it as the file name.
    fileName = CFURLCopyLastPathComponent(downloadURL);
    assert(fileName != NULL);
    
    // Create the downloadPath by taking the destination folder and appending the file name.
    downloadPath = CFURLCreateCopyAppendingPathComponent(kCFAllocatorDefault, destinationFolder, fileName, false);
    assert(downloadPath != NULL);
    CFRelease(fileName);
    
    // Create a CFWriteStream for the file being downloaded.
    writeStream = CFWriteStreamCreateWithFile(kCFAllocatorDefault, downloadPath);
    assert(writeStream != NULL);
    CFRelease(downloadPath);
    
    // CFReadStreamCreateWithFTPURL creates an FTP read stream for downloading from an FTP URL.
    readStream = CFReadStreamCreateWithFTPURL(kCFAllocatorDefault, downloadURL);
    assert(readStream != NULL);
    CFRelease(downloadURL);
    
    // Initialize our MyStreamInfo structure, which we use to store some information about the stream.
    MySetupStreamInfo(&streamInfo, readStream, writeStream);
    context.info = (void *)streamInfo;
    
    // CFWriteStreamOpen will return success/failure.  Opening a stream causes it to reserve all the system resources it requires.  If the stream can open non-blocking, this will always return TRUE; listen to the run loop source to find out when the open completes and whether it was successful.
    success = CFWriteStreamOpen(writeStream);
    if (success) {
        
        // CFReadStreamSetClient registers a callback to hear about interesting events that occur on a stream.
        success = CFReadStreamSetClient(readStream, kNetworkEvents, MyDownloadCallBack, &context);
        if (success) {
            
            // Schedule a run loop on which the client can be notified about stream events.  The client callback will be triggered via the run loop.  It's the caller's responsibility to ensure that the run loop is running.
            CFReadStreamScheduleWithRunLoop(readStream, CFRunLoopGetCurrent(), kCFRunLoopCommonModes);
            
            MyCFStreamSetUsernamePassword(readStream, username, password);
            MyCFStreamSetFTPProxy(readStream, streamInfo);
            
            // Setting the kCFStreamPropertyFTPFetchResourceInfo property will instruct the FTP stream to fetch the file size before downloading the file.  Note that fetching the file size adds some time to the length of the download.  Fetching the file size allows you to potentially provide a progress dialog during the download operation. You will retrieve the actual file size after your CFReadStream Callback gets called with a kCFStreamEventOpenCompleted event.
            CFReadStreamSetProperty(readStream, kCFStreamPropertyFTPFetchResourceInfo, kCFBooleanTrue);
            
            // CFReadStreamOpen will return success/failure.  Opening a stream causes it to reserve all the system resources it requires.  If the stream can open non-blocking, this will always return TRUE; listen to the run loop source to find out when the open completes and whether it was successful.
            success = CFReadStreamOpen(readStream);
            if (success == false) {
                fprintf(stderr, "CFReadStreamOpen failed\n");
                MyCleanupStreamInfo(streamInfo);
            }
        } else {
            fprintf(stderr, "CFReadStreamSetClient failed\n");
            MyCleanupStreamInfo(streamInfo);
        }
    } else {
        fprintf(stderr, "CFWriteStreamOpen failed\n");
        MyCleanupStreamInfo(streamInfo);
    }
    
    return success;
}

 static void
 MyDownloadCallBack(CFReadStreamRef readStream, CFStreamEventType type, void * clientCallBackInfo)
 {
     MyStreamInfo      *info = (MyStreamInfo *)clientCallBackInfo;
     CFIndex           bytesRead = 0, bytesWritten = 0;
     CFStreamError     error;
     CFNumberRef       cfSize;
     SInt64            size;
     float             progress;
     
     assert(readStream != NULL);
     assert(info       != NULL);
     assert(info->readStream == readStream);
     
     switch (type) {
         
         case kCFStreamEventOpenCompleted:
             // Retrieve the file size from the CFReadStream:
             cfSize = CFReadStreamCopyProperty(info->readStream, kCFStreamPropertyFTPResourceSize);
             
             fprintf(stderr, "Open complete\n");
             
             if (cfSize) {
                 if (CFNumberGetValue(cfSize, kCFNumberSInt64Type, &size)) {
                     fprintf(stderr, "File size is %lld\n", size);
                     info->fileSize = size;
                 }
                 CFRelease(cfSize);
             } else {
                 fprintf(stderr, "File size is unknown\n");
                 info->fileSize = 0;
             }
                 break;
         case kCFStreamEventHasBytesAvailable:
             
             do {
                 // CFReadStreamRead will return the number of bytes read, or -1 if an error occurs preventing any bytes from being read, or 0 if the stream's end was encountered. 
                 bytesRead = CFReadStreamRead(info->readStream, info->buffer, kMyBufferSize);
                 if (bytesRead < 0) {
                     fprintf(stderr, "CFReadStreamRead returned %ld\n", bytesRead);
                     break;
                 } else if (bytesRead == 0) {
                     // Note that we break out of the loop instead of relying on CFReadStreamHasBytesAvailable
                     to return 'false' because in some cases, CFReadStreamHasBytesAvailable will return 'true' even
                     though CFReadStreamRead returned 0.
                     break;
                 }
                 
                 // Just in case we call CFWriteStreamWrite and it returns without writing all the data, we loop until all the data is written successfully.  Since we're writing to the "local" file system, it's unlikely that CFWriteStreamWrite will return before writing all the data, but it would be bad if we simply exited the callback because we'd be losing some of the data that we downloaded.
                 bytesWritten = 0;
                 while (bytesWritten < bytesRead) {
                     CFIndex result;
                     
                     result = CFWriteStreamWrite(info->writeStream, info->buffer + bytesWritten, bytesRead - bytesWritten);
                     if (result <= 0) {
                         fprintf(stderr, "CFWriteStreamWrite returned %ld\n", result);
                         goto exit;
                     }
                     bytesWritten += result;
                 }
                 info->totalBytesWritten += bytesWritten;
                 
             } while (CFReadStreamHasBytesAvailable(info->readStream));
             
             if (info->fileSize > 0) {
                 progress = 100*((float)info->totalBytesWritten/(float)info->fileSize);
                 fprintf(stderr, "%.0f%%\n", progress);
             }
                 break;
         case kCFStreamEventErrorOccurred:
             error = CFReadStreamGetError(info->readStream);
             fprintf(stderr, "CFReadStreamGetError returned (%d, %ld)\n", error.domain, error.error);
             goto exit;
         case kCFStreamEventEndEncountered:
             fprintf(stderr, "Download complete\n");
             goto exit;
         default:
             fprintf(stderr, "Received unexpected CFStream event (%d)", type);
             break;
     }
     return;
     
exit:    
         MyCleanupStreamInfo(info);
     CFRunLoopStop(CFRunLoopGetCurrent());
     return;
 }
*/

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation DSUploadFTPStreamData

/*
 uploadPath:toHost:directory:username:password:forDelegate:useSameThread:
 
 Connects to a FTP host and initiates an upload of the specified file or directory to the specified location.  The delegate is informed of the progress.  The delegate is not retained.  If the delegate implements -streamDataConnecting, it is invoked.  If useSameThread is NO, the upload is done in a new thread, otherwise it is done in the same thread.  N.B.: If the specified path is a folder, only the folder itself is uploaded, not its contents.  To upload a folder and contents, use DSFTPData instead.  This class is mostly useful for uploading individual files.
 
 Written by DJS 2005-11.
 Changed by DJS 2006-01 to add useSameThread.
*/

- (void)uploadPath:(NSString *)path
            toHost:(NSString *)host
         directory:(NSString *)directory
          username:(NSString *)username
          password:(NSString *)password
       forDelegate:(id)delegate
     useSameThread:(BOOL)useSameThread;
{
    // Must set the delegate first, since the -informDelegateUsingSignature:withObject: method uses it:
    [self setDelegate:delegate];
    
    if (![host length])
    {
        [self informDelegateOfError:@"Can't connect as no host was specified" terminate:NO];
        
        return;
    }
    
    NSDictionary *parameters = [NSDictionary dictionaryWithObjectsAndKeys:
        path, @"Path",
        host, @"Host",
        directory, @"Directory",
        username, @"Username",
        password, @"Password",
        [NSNumber numberWithBool:useSameThread], @"UseSameThread",
        nil];
    
    // Establish the connection:
    [self connectWithParameters:parameters];
}

/*
 setStreamsWithParameters:
 
 Given the parameters dictionary of -connectToHost:onPort:forDelegate:, connects and sets the input and output streams.  Executes on a sub-thread.
 
 Written by DJS 2005-11.
*/

- (void)setStreamsWithParameters:(NSDictionary *)parameters;
{
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSString *path = [parameters objectForKey:@"Path"];
    NSString *filename = [path lastPathComponent];
    NSString *host = [parameters objectForKey:@"Host"];
    NSString *directory = [parameters objectForKey:@"Directory"];
    NSString *username = [parameters objectForKey:@"Username"];
    NSString *password = [parameters objectForKey:@"Password"];
    NSURL *sourceURL = [NSURL fileURLWithPath:path];
    
    if ([directory isEqualToString:@"/"])
        directory = @"";
    
    NSString *destination = [NSString stringWithFormat:@"ftp://%@/%@/%@", host, directory, filename];
    
    if ([fileManager isDirectoryAtPath:path])
        destination = [destination stringByAppendingString:@"/"];
    
    NSURL *destinationURL = [NSURL URLWithString:destination];
    NSInputStream *input = nil;
    NSOutputStream *output = nil;
    
    if (!sourceURL || !destinationURL)
        return;
    
    // Create a CFReadStream from the local file being uploaded:
    input = (NSInputStream *)CFReadStreamCreateWithFile(kCFAllocatorDefault, (CFURLRef)sourceURL);
    
    if (input == NULL)
        return;
    
    // Create an FTP write stream for uploading operation to a FTP URL. If the URL specifies a directory, the open will be followed by a close event/state and the directory will have been created. Intermediary directory structure is not created:
    output = (NSOutputStream *)CFWriteStreamCreateWithFTPURL(kCFAllocatorDefault, (CFURLRef)destinationURL);
    
    if (output == NULL)
        return;
    
    [self setUsername:username andPassword:password forStream:output];
    [self setProxyForStream:output];
    
    [self setInputStream:input];
    [self setOutputStream:output];
}

/*
 handleHasSpaceAvailableForStream:
 
 Invoked by -stream:handleEvent: when there is a space available event.
 
 Written by DJS 2005-11.
*/

- (void)handleHasSpaceAvailableForStream:(NSStream *)stream;
{
    int totalBytesRead = 0;
    int bytesRead = 0;
    int bytesWritten = 0;
    
    if (stream != [self outputStream])
        return;
    
    do
    {
        // The first thing we do is check to see if there's some leftover data that we read in a previous callback, which we were unable to upload for whatever reason:
        if (leftOverByteCount > 0)
        {
            bytesRead = leftOverByteCount;
        }
        else
        {
            // -[NSInputStream read:maxLength:] will return the number of bytes read, or -1 if an error occurs preventing any bytes from being read, or 0 if the stream's end was encountered:
            bytesRead = [[self inputStream] read:buffer maxLength:kMyBufferSize];
            
            if (bytesRead < 0)
                [self informDelegateOfError:[NSString stringWithFormat:@"Error while reading: %d", bytesRead] terminate:YES];
            
            totalBytesRead += bytesRead;
        }
        
        // -[NSOutputStream write:maxLength:] returns the number of bytes successfully written, -1 if an error has occurred, or 0 if the stream has been filled to capacity (for fixed-length streams).  If the stream is not full, this call will block until at least one byte is written:
        bytesWritten = [[self outputStream] write:buffer maxLength:bytesRead];
        
        if (bytesWritten > 0)
        {
            totalBytesWritten += bytesWritten;
            
            // If we couldn't upload all the data that we read, we temporarily store the data in our ivars until more space is available:
            if (bytesWritten < bytesRead)
            {
                leftOverByteCount = bytesRead - bytesWritten;
                memmove(buffer, buffer + bytesWritten, leftOverByteCount);
            }
            else
            {
                leftOverByteCount = 0;
            }
        }
        else
        {
            if (bytesWritten < 0)
                [self informDelegateOfError:[NSString stringWithFormat:@"Error while uploading: %d", bytesWritten] terminate:NO];
            
            break;
        }
        
    }
    while ([[self outputStream] hasSpaceAvailable]);
    
    [self informDelegateUsingSignature:@"streamData:uploadedBytes:"
                            withObject:[NSNumber numberWithInt:totalBytesWritten]];
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface DSFTPData (Private)

- (void)recursivelyScanLocalPath:(NSString *)localPath forRemoteDirectory:(NSString *)remoteDirectory;
- (void)filterUploadInfoUsingCachedInfo;

- (void)uploadNextPath;

- (void)streamDataEnded:(id)streamData;

- (NSMutableArray *)uploadInfo;
- (void)setUploadInfo:(NSMutableArray *)value;

- (DSUploadFTPStreamData *)streamData;
- (void)setStreamData:(DSUploadFTPStreamData *)value;

@end


@implementation DSFTPData

/*
 ftpData
 
 Convenience method to create a DSFTPData instance.
 
 Written by DJS 2005-11.
*/

+ (id)ftpData;
{
    return [[[self alloc] init] autorelease];
}

- (void)dealloc;
{
    [self setStreamData:nil];
    [self setUploadInfo:nil];
    [self setCachedInfo:nil];
    [self setHost:nil];
    [self setUsername:nil];
    [self setPassword:nil];
    
    [super dealloc];
}


/*
 uploadPath:toHost:directory:username:password:cachedInfoPath:
 
 Connects to a FTP host and initiates an upload of the specified file or directory to the specified location.  If a directory, its contents are recursively enumerated and uploaded too.  If a cachedInfoPath is specified, information is read from and written to that file to avoid re-uploading previously uploaded items; otherwise everything is uploaded every time.  You should make sure that the DSFTPData instance is retained until the upload is finished.  It posts a "DSFTPDataProgressNotification" notification after each file uploaded, so simple progress can be displayed, if desired.  Note that the upload is performed in the current thread; add a parameter for useSameThread if different behavior is needed in the future (-[DSUploadFTPStreamData uploadPath::::] supports the useSameThread parameter).
 
 Written by DJS 2005-11.
*/

- (void)uploadPath:(NSString *)path
            toHost:(NSString *)host
         directory:(NSString *)directory
          username:(NSString *)username
          password:(NSString *)password
    cachedInfoPath:(NSString *)cachedInfoPath;
{
    [self resetIndex];
    [self setWorking:YES];
    
    [self setUploadInfo:[NSMutableArray array]];
    [self setHost:host];
    [self setUsername:username];
    [self setPassword:password];
    
    if ([cachedInfoPath length])
        [self setCachedInfo:[NSArray arrayWithContentsOfFile:cachedInfoPath]];
    
    [self recursivelyScanLocalPath:path forRemoteDirectory:directory];
    
    [self filterUploadInfoUsingCachedInfo];
    
    if ([cachedInfoPath length])
        [[self cachedInfo] writeToFile:cachedInfoPath atomically:NO];
    
    if ([self count])
        [self uploadNextPath];
    else
        [self streamDataEnded:nil];
}

/*
 stop
 
 Stops the upload immediately.  (May result in a partial file on the remote server, though probably not.)
 
 Written by DJS 2005-12.
*/

- (void)stop;
{
    [self setWorking:NO];
    [[self streamData] disconnect];
}


/*
 cachedInfo
 
 Returns an array of dictionaries describing the known folders and files, as found during the recursive scan.  Callers can call this after -uploadPath:::: to get and store the info to speed up future uploads.
 
 Written by DJS 2005-12.
*/

- (NSArray *)cachedInfo;
{
    return _cachedInfo;
}

/*
 setCachedInfo:
 
 Sets an array of dictionaries, as previously returned by -cachedInfo, above.  Callers can call this before -uploadPath:::: to cause the recursive scan to only include folders that haven't been seen before and files that have a changed modification date.
 
 Written by DJS 2005-12.
*/

- (void)setCachedInfo:(NSArray *)value;
{
    [value retain];
    [_cachedInfo autorelease];
    _cachedInfo = value;
}

- (NSString *)host;
{
    return _host;
}

- (void)setHost:(NSString *)value;
{
    [value retain];
    [_host autorelease];
    _host = value;
}

- (NSString *)username;
{
    return _username;
}

- (void)setUsername:(NSString *)value;
{
    [value retain];
    [_username autorelease];
    _username = value;
}

- (NSString *)password;
{
    return _password;
}

- (void)setPassword:(NSString *)value;
{
    [value retain];
    [_password autorelease];
    _password = value;
}

- (int)index;
{
    return _index;
}

- (void)resetIndex;
{
    _index = 0;
}

- (void)incrementIndex;
{
    _index++;
}

- (int)count;
{
    return [[self uploadInfo] count];
}

- (BOOL)isMore;
{
    return [self isWorking] && ([self index] + 1) < [self count];
}

- (BOOL)isWorking;
{
    return _isWorking;
}

- (void)setWorking:(BOOL)value;
{
    _isWorking = value;
}

/*
 currentInfo
 
 Returns a dictionary with information about the current operation, including the local path, remote directory, and status information etc.  If the upload hasn't started yet, or has completed, nil is returned.
 
 Written by DJS 2005-12.
*/

- (NSDictionary *)currentInfo;
{
    if ([self index] >= [self count])
        return nil;
    
    NSDictionary *dict = [[self uploadInfo] objectAtIndex:[self index]];
    NSString *path = [dict emptyStringOrObjectForKey:@"LocalPath"];
    NSString *directory = [dict emptyStringOrObjectForKey:@"RemoteDirectory"];
    int nth = [self index] + 1;
    int total = [self count];
    int percent = (nth * 100) / total;
    BOOL isMore = [self isMore];
    
    NSDictionary *info = [NSDictionary dictionaryWithObjectsAndKeys:
        path, @"LocalPath",
        directory, @"RemoteDirectory",
        [NSNumber numberWithInt:nth], @"Nth",       // Don't change these keys, as they are used in Simon's reports
        [NSNumber numberWithInt:total], @"Total",
        [NSNumber numberWithInt:percent], @"Percent",
        [NSNumber numberWithBool:isMore], @"IsMore",
        nil];
    
    return info;
}

@end


@implementation DSFTPData (Private)

/*
 recursivelyScanLocalPath:forRemoteDirectory:
 
 Adds entries to the upload info for each local file or folder and the corresponding remote directory it should go to.  A recursive method, private to DSFTPData.
 
 Written by DJS 2005-12.
*/

- (void)recursivelyScanLocalPath:(NSString *)localPath forRemoteDirectory:(NSString *)remoteDirectory;
{
    NSFileManager *fileManager = [NSFileManager defaultManager];
    BOOL isDirectory = [fileManager isDirectoryAtPath:localPath];
    NSMutableDictionary *dict = [NSMutableDictionary dictionary];
    
    [dict setObject:localPath forKey:@"LocalPath"];
    [dict setObject:remoteDirectory forKey:@"RemoteDirectory"];
    [dict setObject:[NSNumber numberWithBool:isDirectory] forKey:@"IsDirectory"];
    
    // We only care about the modification date for files, not folders:
    if (!isDirectory)
        [dict setObject:[fileManager fileModificationDateAtPath:localPath] forKey:@"ModificationDate"];
    
    // Add information about the item to the upload info:
    [[self uploadInfo] addObject:dict];
    
    // If it's a file, we're done here:
    if (!isDirectory)
        return;
    
    // Recursively scan the folder:
    NSString *tempDirectory = [remoteDirectory stringByAppendingPathComponent:[localPath lastPathComponent]];
    NSArray *contents = [fileManager directoryContentsAtPath:localPath];
    NSEnumerator *enumerator = [contents objectEnumerator];
    NSString *filename;
    
    while ((filename = [enumerator nextObject]))
    {
        NSString *tempPath = [localPath stringByAppendingPathComponent:filename];
        
        [self recursivelyScanLocalPath:tempPath forRemoteDirectory:tempDirectory];
    }
}

/*
 filterUploadInfoUsingCachedInfo
 
 Given cached info as set by -setCachedInfo:, filters out folders that exist in both, and files that haven't been modified.  After this, -uploadInfo contains only the items that should be uploaded, and -cachedInfo contains all currently known items (i.e. what was in -uploadInfo).
 
 Written by DJS 2005-12.
*/

- (void)filterUploadInfoUsingCachedInfo;
{
    NSMutableArray *result = nil;
    
    if ([self cachedInfo])
    {
        NSEnumerator *enumerator = [[self uploadInfo] objectEnumerator];
        NSDictionary *dict;
        
        result = [NSMutableArray array];
        
        while ((dict = [enumerator nextObject]))
        {
            if (![[self cachedInfo] containsObjectEquivalentTo:dict])
                [result addObject:dict];
        }
    }
    
    [self setCachedInfo:[self uploadInfo]];
    
    // If there was previously some cached info, change the upload info to the filtered array:
    if (result)
        [self setUploadInfo:result];
}

/*
 uploadNextPath
 
 Uploads the next file or folder to the remote server.  Private to DSFTPData; called automatically after each is completed.
 
 Written by DJS 2005-12.
 Changed by DJS 2006-01 to add the useSameThread parameter to the -uploadPath:::: call.
*/

- (void)uploadNextPath;
{
    [self setStreamData:[DSUploadFTPStreamData streamData]];
    NSDictionary *info = [self currentInfo];
    NSString *path = [info objectForKey:@"LocalPath"];
    NSString *directory = [info objectForKey:@"RemoteDirectory"];
    NSString *host = [self host];
    NSString *username = [self username];
    NSString *password = [self password];
    
//    NSLog(@"Starting uploading %@", path);		//  log
    
    // Initiate the upload:
    [[self streamData] uploadPath:path toHost:host directory:directory username:username password:password forDelegate:self useSameThread:YES];
}

/*
 streamDataEnded:
 
 Delegate message from DSUploadFTPStreamData to let us know that an upload has finished, so we can send a "DSFTPDataProgressNotification" notification on the progress.
 
 Written by DJS 2005-11.
*/

- (void)streamDataEnded:(id)streamData;
{
    
    // NOTE: if I add a DSListingFTPStreamData scan before the uploads, this method will need to recognize the difference, since it would get called for that too.  A bool ivar would suffice.
    
//    NSLog(@"...Finished uploading");		//  log
    
    [[NSNotificationCenter defaultCenter] postNotificationName:@"DSFTPDataProgressNotification"
                                                        object:self userInfo:[self currentInfo]];
    
    // Note whether there's more before incrementing the index, otherwise the last item would be skipped:
    BOOL isMore = [self isMore];
    
    // Increment the index after we're completely done with this item:
    [self incrementIndex];
    
    if (isMore)
        [self uploadNextPath];
    else
        [self setWorking:NO];
}

/*
- (void)streamData:(id)streamData sentDelegateMessage:(NSDictionary *)parameters;
{
    NSLog(@"streamData %@ sent %@", streamData, [parameters description]);		//  log
}
*/

- (NSMutableArray *)uploadInfo;
{
    return _uploadInfo;
}

- (void)setUploadInfo:(NSMutableArray *)value;
{
    [value retain];
    [_uploadInfo autorelease];
    _uploadInfo = value;
}

- (DSUploadFTPStreamData *)streamData;
{
    return _streamData;
}

- (void)setStreamData:(DSUploadFTPStreamData *)value;
{
    [value retain];
    [_streamData autorelease];
    _streamData = value;
}

@end

