//
//  DSSpeechData.m
//  Shared
//
//  Created by David Sinclair on Sat Nov 30 2002.
//

#import "DSSpeechData.h"
#import "DSSharedDefines.h"


void MySpeechDoneCallback(SpeechChannel chan,SInt32 refCon);
void MySpeechWordCallback (SpeechChannel chan, SInt32 refCon, UInt32 wordPos, 
    UInt16 wordLen);


@interface DSSpeechData (Private)

- (void)setCallbacks;
- (NSPort *)port;
- (void)setReserved1:(unsigned int)r;
- (void)setReserved2:(unsigned int)r;
- (BOOL)usesPort;
- (void)handleMessage:(unsigned)msgid;

@end


@implementation DSSpeechData

/*"Returns the voice names in the same order as expected by setVoice:."*/

+ (NSArray *)voiceNames
{
    NSMutableArray *voices = [NSMutableArray arrayWithCapacity:0];
    short voiceCount;
    OSErr error = noErr;
    int voiceIndex;
    
    error = CountVoices(&voiceCount);
    
    if (error != noErr)
        return voices;

    //NSLog(@"hello : %d", voiceCount);

    for (voiceIndex = 0; voiceIndex < voiceCount; voiceIndex++)
    {
        VoiceSpec voiceSpec;
        VoiceDescription voiceDescription;
        
        error = GetIndVoice(voiceIndex + 1, &voiceSpec);
        if (error != noErr)
            return voices;

        error = GetVoiceDescription(&voiceSpec, &voiceDescription, sizeof(voiceDescription));

        if (error == noErr)
        {
            NSString *voiceName = [NSString stringWithPascalString:voiceDescription.name];
            
//            NSString *voiceName = [[[NSString alloc] initWithCString:
//                &(voiceDescription.name[1]) length:voiceDescription.name[0]] autorelease];

            //NSLog(voiceName);
            
            [voices addObject:voiceName];
        }
        else
            return voices;
    }
    
    return voices;
}

- (id)init
{
    NSRunLoop *loop = [NSRunLoop currentRunLoop];
    self = [super init];
    
    // we have 2 options here : we use a port or we don't.
    // using a port means delegate message are invoked from the main 
    // thread (runloop in which this object is created), otherwise, those message 
    // are asynchronous.
    
    if(loop != nil) 
    {
        _port = [[NSPort port] retain];
        // we use a port so that the speech manager callbacks can talk to the main thread.
        // That way, we can safely access interface elements from the delegate methods
        
        [_port setDelegate:self];
        [loop addPort:_port forMode:NSDefaultRunLoopMode];
        _usePort = YES;
    }
    else
        _usePort = NO;
    
    NewSpeechChannel(NULL, &_speechChannel); // NULL voice is default voice

    [self setCallbacks];
    
    return self;
}

- (void)dealloc
{
    [self stopSpeaking];
    
    if(_speechChannel != NULL)
    {
        DisposeSpeechChannel(_speechChannel);
    }
    
    if (_port)
    {
        [[NSRunLoop currentRunLoop] removePort:_port forMode:NSDefaultRunLoopMode];
        
        [_port release];
    }
    
    [super dealloc];
}


- (float)rate
{
    Fixed value = 0;

    if (_speechChannel != NULL)
    {
        GetSpeechInfo (_speechChannel, soRate, &value);

        value = value / 65536.0;
    }
    
    return value;
}

/*"Sets the pitch. Pitch is given in Hertz and should be comprised between 80 and 500, depending on the voice. Note that extreme value can make you app crash..."*/

- (void)setRate:(float)rate
{
    Fixed theNewValue = rate * 65536.0;

    if (_speechChannel != NULL)
        SetSpeechInfo (_speechChannel, soRate, &theNewValue);
}

- (float)pitch
{
    Fixed value = 0;

    if (_speechChannel != NULL)
    {
        GetSpeechPitch (_speechChannel, &value);

        value = value / 65536.0;
    }

    return value;
}

- (void)setPitch:(float)pitch
{
    Fixed theNewValue = pitch * 65536.0;

    if (_speechChannel != NULL)
        SetSpeechPitch (_speechChannel, theNewValue);
}

/*
- (void)setPitch:(float)pitch
{
    int fixedPitch;
    
    pitch = (pitch - 90.0) / (300.0 - 90.0) * (65.0 - 30.0) + 30.0;
    
    // I don't know what Apple means with pitch between 30 and 65, so I convert that range to [90, 300]. I did not test frequencies correspond, though.
    
    fixedPitch = (int)pitch;
    
    fixedPitch = fixedPitch << 16; // fixed point
    
    if (_speechChannel != NULL)
    {
        SetSpeechPitch (_speechChannel, fixedPitch);
    }
}
*/

- (float)inflection
{
    Fixed value = 0;

    if (_speechChannel != NULL)
    {
        GetSpeechInfo (_speechChannel, soPitchMod, &value);

        value = value / 65536.0;
    }
    
    return value;
}

- (void)setInflection:(float)inflection
{
    Fixed theNewValue = inflection * 65536.0;

    if (_speechChannel != NULL)
        SetSpeechInfo (_speechChannel, soPitchMod, &theNewValue);
}

- (float)volume
{
    Fixed value = 0;

    if (_speechChannel != NULL)
    {
        GetSpeechInfo (_speechChannel, soVolume, &value);

        value = value / 65536.0;
    }

    return value;
}

- (void)setVolume:(float)volume
{
    Fixed theNewValue = volume * 65536.0;

    if (_speechChannel != NULL)
        SetSpeechInfo (_speechChannel, soVolume, &theNewValue);
}

- (int)voice
{
    return 0;		//  hack; not implemented yet
}

- (void)setVoice:(int)index
{
    if (index < 0)
        return;
    
    VoiceSpec voice;
    OSErr error = noErr;
    
    if (_speechChannel != NULL)
    {
        DisposeSpeechChannel(_speechChannel);
        _speechChannel = NULL;
    }
    
    error = GetIndVoice(index + 1, &voice);

    if (error == noErr)
    {
        NewSpeechChannel(&voice, &_speechChannel);
        [self setCallbacks];
    }
}

- (void)setVoiceFromName:(NSString *)name
{
    NSArray *voices = [DSSpeechData voiceNames];

    int index = [voices indexOfObject:name];

    if (index != NSNotFound)
        [self setVoice:index];
}

- (void)reset
{
    if (_speechChannel != NULL)
        SetSpeechInfo (_speechChannel, soReset, NULL);
}

- (void)speakText:(NSString *)text
{
    if (_speechChannel != NULL && text != nil)
    {
        //finished = NO;
        SpeakText(_speechChannel, [text cString], [text length]);
        //while(!finished) ;
        //sleep(2);
    }
}

- (void)_speakTextTimer:(NSTimer *)timer;
{
    NSDictionary *userInfo = [timer userInfo];
    NSString *text = [userInfo objectForKey:@"String"];
    NSTimeInterval interval = [userInfo floatForKey:@"Interval"];
    
    [self speakText:text afterSilentInterval:interval];
}

- (void)speakText:(NSString *)text afterSilentInterval:(NSTimeInterval)interval;
{
    if (!SpeechBusySystemWide())
        [self speakText:text];
    else
    {
        NSDictionary *info = [NSDictionary dictionaryWithObjectsAndKeys:
            text, @"String",
            [NSNumber numberWithFloat:interval], @"Interval",
            nil];
        
        [NSTimer scheduledTimerWithTimeInterval:interval target:self selector:@selector(_speakTextTimer:) userInfo:info repeats:NO];
    }
}

- (void)stopSpeaking
{
    if (_speechChannel != NULL)
    {
        StopSpeech(_speechChannel);

        if ([_delegate respondsToSelector:@selector(didFinishSpeaking:)])
        {
            [_delegate didFinishSpeaking:self];
        }
    }
}

- (void)setDelegate:(id)delegate
{
    _delegate = delegate;
}

- (id)delegate
{
    return _delegate;
}

//--- Private ---

- (void)setCallbacks
{
    if (_speechChannel != NULL)
    {
        SetSpeechInfo(_speechChannel, soSpeechDoneCallBack, &MySpeechDoneCallback);
        SetSpeechInfo(_speechChannel, soWordCallBack, &MySpeechWordCallback);
        SetSpeechInfo(_speechChannel, soRefCon, (const void*)self);
    }
}

- (void)setReserved1:(unsigned int)r
{
    _reserved1 = r;
}

- (void)setReserved2:(unsigned int)r
{
    _reserved2 = r;
}

- (NSPort *)port
{
    return _port;
}

- (BOOL)usesPort
{
    return _usePort;
}

- (void)handleMessage:(unsigned)msgid
{
    if (msgid == 5)
    {
        if ([_delegate respondsToSelector:@selector(willSpeakWord:at:length:)])
        {
            if (_reserved1 >= 0 && _reserved2 >= 0)
                [_delegate willSpeakWord:self at:_reserved1 length:_reserved2];
            else
                [_delegate willSpeakWord:self at:0 length:0];
        }
    }
    else if (msgid == 8)
    {
        if ([_delegate respondsToSelector:@selector(didFinishSpeaking:)])
        {
            [_delegate didFinishSpeaking:self];
        }
    }
}

//--- NSPort delegate ---

- (void)handlePortMessage:(NSPortMessage *)portMessage
{
    int msg = [portMessage msgid];
    
    [self handleMessage:msg];
}

@synthesize _reserved2;
@synthesize _port;
@synthesize _reserved1;
@synthesize _usePort;
@synthesize _speechChannel;
@end


void MySpeechDoneCallback(SpeechChannel chan,SInt32 refCon)
{
    DSSpeechData *speaker = (DSSpeechData *)refCon;
    unsigned msg = 8;
    //NSLog(@"Speech Done");
    
    if ([speaker isKindOfClass:[DSSpeechData class]])
    {
        if ([speaker usesPort])
        {
            NSPortMessage *message = [[NSPortMessage alloc] initWithSendPort:[speaker port]
                receivePort:[speaker port] components:nil];
        
            [message setMsgid:msg];
            [message sendBeforeDate:nil];
            [message release];
        }
        else
        {
            // short-circuit port
            [speaker handleMessage:msg];
        }
    } 
}

void MySpeechWordCallback(SpeechChannel chan, SInt32 refCon, UInt32 wordPos,UInt16 wordLen)
{
    DSSpeechData *speaker = (DSSpeechData *)refCon;
    unsigned msg = 5;
    
    //NSLog(@"Word Done");
    
    if ([speaker isKindOfClass:[DSSpeechData class]])
    {
        [speaker setReserved1:wordPos];
        [speaker setReserved2:wordLen];
        
        if ([speaker usesPort])
        {
            NSPortMessage *message = [[NSPortMessage alloc] initWithSendPort:[speaker port]
                receivePort:[speaker port] components:nil];
        
            [message setMsgid:msg];
            [message sendBeforeDate:nil];
            [message release];
        }
        else 
        {
            // short-circuit port
            [speaker handleMessage:msg];
        }
    } 
}

