//
//  DSStreamData.h
//  Shared classes
//
//  Created by David Sinclair on Thu Jul 01 2004.
//  Copyright (c) 2004 - 2007 Dejal Systems, LLC. All rights reserved.
//

#import "DSDelegateData.h"


// Note: this is an abstract class.  See DSSocketStreamData and DSFTPStreamData for concrete subclasses.


@interface DSStreamData : DSDelegateData
{
    NSRunLoop *_runLoop;
    NSInputStream *_inputStream;
    NSOutputStream *_outputStream;
    int _connectionCount;
    BOOL _needToStop;
}

+ (id)streamData;

- (void)disconnect;

- (void)sendString:(NSString *)string;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSRunLoop *)runLoop;
- (void)setRunLoop:(NSRunLoop *)value;

- (NSInputStream *)inputStream;
- (void)setInputStream:(NSInputStream *)value;

- (NSOutputStream *)outputStream;
- (void)setOutputStream:(NSOutputStream *)value;

- (BOOL)isConnected;
- (int)connectionCount;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark OVERRIDE METHODS (for subclasses)
// ----------------------------------------------------------------------------------------

- (void)setStreamsWithParameters:(NSDictionary *)parameters;

- (void)handleNoEventForStream:(NSStream *)stream;
- (void)handleOpenCompletedForStream:(NSStream *)stream;
- (void)handleHasBytesAvailableForStream:(NSStream *)stream;
- (void)handleHasSpaceAvailableForStream:(NSStream *)stream;
- (BOOL)handleErrorOccurred:(NSError *)error forStream:(NSStream *)stream;
- (BOOL)handleEndEncounteredForStream:(NSStream *)stream;
- (void)handleOtherEvent:(NSStreamEvent)streamEvent forStream:(NSStream *)stream;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark PRIVATE METHODS (only for subclasses)
// ----------------------------------------------------------------------------------------

- (void)connectWithParameters:(NSDictionary *)parameters;
- (void)threadedConnectWithParameters:(NSDictionary *)parameters;

//- (void)threadedDisconnect;

- (void)stream:(NSStream *)stream handleEvent:(NSStreamEvent)streamEvent;

- (void)receiveStringForStream:(NSStream *)stream;

- (void)run;
- (void)terminate;

- (void)informDelegateOfError:(id)error terminate:(BOOL)shouldTerminate;

- (int)resetConnectionCount;
- (int)incrementConnectionCount;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface NSObject (DSStreamDataInformalProtocol)

/*
 streamDataShouldUseSameThread:
 
 Asks the delegate whether to use same thread for this connection, or a new one (the default).  This will be the first message received.
*/

- (BOOL)streamDataShouldUseSameThread:(id)streamData;

/*
 streamDataConnecting:

 Informs the delegate that a connection is being attempted.
*/

- (void)streamDataConnecting:(id)streamData;

/*
 streamData:connectedToStream:
 
 Informs the delegate that a connection has been successfully established to the specified stream; normally you'd just implement -streamDataConnected:, below.
*/

- (void)streamData:(id)streamData connectedToStream:(NSStream *)stream;

/*
 streamDataConnected:
 
 Informs the delegate that a connection has been successfully established (with both input and output streams).
*/

- (void)streamDataConnected:(id)streamData;

/*
 streamDataDisconnected:
 
 Informs the delegate that the connection has been terminated by the user.
*/

- (void)streamDataDisconnected:(id)streamData;

/*
 streamDataConnectionTerminated:
 
 Informs the delegate that the connection has been terminated by the remote host.
*/

- (void)streamDataConnectionTerminated:(id)streamData;

/*
 streamData:errorOccurred:
 
 Informs the delegate that an error occurred; the error from the remote host is provided.  Note that the error may be either a NSError or a NSString; passing -description to either will get a string.
*/

- (void)streamData:(id)streamData errorOccurred:(id)error;

/*
 streamDataEnded:
 
 Informs the delegate that a connection has been concluded (for any reason); this will be the last message received.
*/

- (void)streamDataEnded:(id)streamData;

/*
 streamData:receivedString:
 
 Informs the delegate that the specified string was received.  It can log it or do whatever else it wants with it.  This is only used if the subclass calls -receiveStringForStream: from -handleHasBytesAvailableForStream:.
*/

- (void)streamData:(id)streamData receivedString:(NSString *)string;

/*
 streamData:sentString:
 
 Informs the delegate that the specified string was sent.
*/

- (void)streamData:(id)streamData sentString:(NSString *)string;

@end
