//
//  DSTaskData.h
//  Shared classes
//
//  Created by David Sinclair on 2006-02-12.
//  Copyright 2006 Dejal Systems, LLC. All rights reserved.
//


#import "DSDelegateData.h"


@interface DSTaskData : DSDelegateData
{
    NSMutableString *_output;
    NSString *_error;
    NSTask *_task;
    NSFileHandle *_outputFileHandle;
    NSFileHandle *_errorFileHandle;
}

+ (id)launchedTaskWithLaunchPath:(NSString *)path arguments:(NSArray *)arguments delegate:(id)delegate;

- (id)initWithLaunchPath:(NSString *)path arguments:(NSArray *)arguments delegate:(id)delegate;

- (void)launch;
- (void)stop;
- (BOOL)isRunning;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSString *)output;
- (NSString *)error;

- (NSTask *)task;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface NSObject (DSTaskDataInformalProtocol)

/*
 taskDataLaunched:
 
 Informs the delegate that the task has successfully launched.
*/

- (void)taskDataLaunched:(id)taskData;

/*
 taskDataFailedLaunch:
 
 Informs the delegate that the task could not be launched.
*/

- (void)taskDataFailedLaunch:(id)taskData;

/*
 taskDataEnded:
 
 Informs the delegate that the task has been terminated or otherwise finished; this will be the last message received.
*/

- (void)taskDataEnded:(id)taskData;

/*
 taskData:receivedString:
 
 Informs the delegate that the specified string was received.  It can log it or do whatever else it wants with it.  Note that this may be partial output; call -output to get the complete output.
*/

- (void)taskData:(id)taskData receivedString:(NSString *)string;

/*
 taskData:errorOccurred:
 
 Informs the delegate that an error occurred; the error from the remote host is provided.  Note that the error may be either a NSError or a NSString; passing -description to either will get a string.  (Currently it is always a string, but that may change.)
*/

- (void)taskData:(id)taskData errorOccurred:(id)error;

@end
