#import "DSGraphTableSheetController.h"
#import "DSTableViewCategories.h"
#import "DSGraphController.h"


@implementation DSGraphTableSheetController

+ (id)showGraphTableSheetWithTitle:(NSString *)title forController:(id)tableController
                          ofWindow:(NSWindow *)tableWindow withTableView:(NSTableView *)tableView
                         countPref:(NSString *)countPref groupPref:(NSString *)groupPref
{
    self = [[[self class] alloc] initGraphTableSheetWithTitle:title forController:tableController
                                                     ofWindow:tableWindow withTableView:tableView
                                                    countPref:countPref groupPref:groupPref];
    return self;
}

- (id)initGraphTableSheetWithTitle:(NSString *)title forController:(id)tableController
                          ofWindow:(NSWindow *)tableWindow withTableView:(NSTableView *)tableView
                         countPref:(NSString *)countPref groupPref:(NSString *)groupPref
{
    self = [self init];
    
    [NSBundle loadNibNamed:@"DSGraphTableSheet" owner:self];

    if (![self isWindowLoaded])
        [self window];

    [self setTableController:tableController];
    [self setTableView:tableView];
    [self setCountPref:countPref];
    [self setGroupPref:groupPref];

    [titleStat setStringValue:title];

    NSEnumerator *enumerator = [[tableView tableColumns] objectEnumerator];
    NSTableColumn *column;

    [countPopup removeAllItems];
    [groupPopup removeAllItems];
    
    while ((column = [enumerator nextObject]))
    {
        NSString *title = [[column headerCell] stringValue];

        [countPopup addItemWithTitle:title];
        [groupPopup addItemWithTitle:title];
    }

    [[groupPopup menu] addItem:[NSMenuItem separatorItem]];    
    [groupPopup addItemWithTitle:@"None"];

    [countPopup selectItemWithTitle:[[NSUserDefaults standardUserDefaults] objectForKey:countPref]];
    [groupPopup selectItemWithTitle:[[NSUserDefaults standardUserDefaults] objectForKey:groupPref]];

    [self showAsSheetOnWindow:tableWindow];
    
    return self;
}

- (void)dealloc
{
    [_lines release];
    [_tableView release];

    [super dealloc];
}

- (void)showAsSheetOnWindow:(NSWindow *)tableWindow
{
    [NSApp beginSheet:[self window] modalForWindow:tableWindow modalDelegate:nil
       didEndSelector:nil contextInfo:nil];
}


- (IBAction)cancelButtonHit:(id)sender
{
    [NSApp endSheet:[self window]];
    [[self window] orderOut:nil];
    [self autorelease];
}

- (IBAction)graphButtonHit:(id)sender
{
    BOOL sane = YES;

    if ([self tableController] && [self tableView])
    {
        DSGraphController *graphController = [DSGraphController graph];
        NSEnumerator *enumerator = [[self tableView] multipleSelectedOrAllRowsEnumerator];
        NSNumber *rowNum;
        NSTableColumn *countColumn = nil;
        NSTableColumn *groupColumn = nil;
        NSArray *tableColumns = [[self tableView] tableColumns];
        int countColumnIndex = [countPopup indexOfSelectedItem];
        int groupColumnIndex = [groupPopup indexOfSelectedItem];
        
        if (countColumnIndex < [tableColumns count])
            countColumn = [tableColumns objectAtIndex:countColumnIndex];
        
        if (groupColumnIndex < [tableColumns count])
            groupColumn = [tableColumns objectAtIndex:groupColumnIndex];

        while (sane && (rowNum = [enumerator nextObject]))
        {
            int row = [rowNum intValue];
            id groupValue = nil;
            id countValue = nil;

            if (countColumnIndex < [tableColumns count] && countColumn)
            {
                countValue = [[self tableController] tableView:[self tableView]
                                     objectValueForTableColumn:countColumn row:row];

                if (!countValue)
                    countValue = @"unknown";
            }

            if (groupColumnIndex < [tableColumns count] && groupColumn)
            {
                groupValue = [[self tableController] tableView:[self tableView]
                                     objectValueForTableColumn:groupColumn row:row];

                if (!groupValue)
                    groupValue = @"unknown";
            }

            sane = [graphController addGroupObject:groupValue withCountObject:countValue];
        }

        if (sane)
        {
            [graphController setTitle:[titleStat stringValue] withCountName:[countPopup titleOfSelectedItem]
                            groupName:[groupPopup titleOfSelectedItem]];

            [graphController displayWindow];
        }
        else
        {
            NSRunCriticalAlertPanel([titleStat stringValue],
                                    @"There are too many different values to create the graph.  Please select fewer lines, or diffrent columns.",
                                    @"OK", nil, nil);
        }        
    }

    if (sane)
    {
        [[NSUserDefaults standardUserDefaults] setObject:[countPopup titleOfSelectedItem] forKey:[self countPref]];
        [[NSUserDefaults standardUserDefaults] setObject:[groupPopup titleOfSelectedItem] forKey:[self groupPref]];
        [[NSUserDefaults standardUserDefaults] synchronize];

        [NSApp endSheet:[self window]];
        [[self window] orderOut:nil];
        [self autorelease];
    }
}

- (IBAction)countPopupHit:(id)sender
{
    //    if ([unitsPopup indexOfSelectedItem] == FREQ_INDEFINITE)
}

- (IBAction)groupPopupHit:(id)sender
{
}


- (id)tableController
{
    return _tableController;
}

- (void)setTableController:(id)value
{
    [value retain];
    [_tableController release];
    _tableController = value;
}

- (NSTableView *)tableView
{
    return _tableView;
}

- (void)setTableView:(NSTableView *)value
{
    [value retain];
    [_tableView release];
    _tableView = value;
}

- (NSMutableArray *)lines
{
    return _lines;
}

- (void)setLines:(NSMutableArray *)value
{
    [value retain];
    [_lines release];
    _lines = value;
}

- (NSString *)countPref
{
    return _countPref;
}

- (void)setCountPref:(NSString *)value
{
    [value retain];
    [_countPref release];
    _countPref = value;
}

- (NSString *)groupPref
{
    return _groupPref;
}

- (void)setGroupPref:(NSString *)value
{
    [value retain];
    [_groupPref release];
    _groupPref = value;
}

@end
