//
//  DSPopupHandler.h
//  Shared classes
//
//  Created by David Sinclair on Mon Aug 04 2003.
//  Copyright (c) 2003 - 2004 Dejal Systems, LLC. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "DSViewsListView.h"


#define FIRST_UNUSED_MODE -1


/*
 DSPopupHandler
 
 This class is an abstract superclass for a DSViewsListView handler.  It should be used if there are multiple container views in one Nib file.  If there is only one container view in one Nib file, use DSPopupSingleHandler instead.  If there are multiple container views in multiple Nib files, use DSPopupClusterHandler instead.  Do not change this class in a way that will break existing subclasses; any changes should probably go in a subclass instead.
*/

@interface DSPopupHandler : NSObject <DSViewsListHandlerProtocol>
{
    id _viewsListView;
    id _userInfo;
    int _mode;

    NSString *_changeNotificationName;
}

- (id)initForViewsListView:(id)viewsListView withSettings:(NSDictionary *)settings userInfo:(id)userInfo;

- (NSDictionary *)settings;
- (NSMutableDictionary *)mutableSettings;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark CONFIGURATION METHODS
// ----------------------------------------------------------------------------------------

- (BOOL)shouldAddAfter;
- (BOOL)modesAreUnique;
- (int)defaultMode;

+ (NSString *)defaultChangeNotificationName;
- (NSString *)defaultChangeNotificationName;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark UTILITY METHODS
// ----------------------------------------------------------------------------------------

- (void)selectDefaultItem;

- (void)notifyOfChange;
- (void)viewsListViewHandlersDidChange:(NSNotification *)note;
- (void)modeDidChange;
- (void)containerViewDidChange;
- (void)enableRemoveButtons:(BOOL)enable;

- (NSMutableArray *)handlersForMode:(int)mode includingSelf:(BOOL)inclSelf;

- (NSCountedSet *)usedModesIncludingSelf:(BOOL)inclSelf;
- (NSMutableArray *)unusedModes;
- (int)firstUnusedMode;

- (BOOL)validateMenuItem:(NSMenuItem *)menuItem;

+ (void)appendValue:(NSDictionary *)settings toString:(NSMutableString *)output;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark VIEWS LIST HANDLER PROTOCOL METHODS
// ----------------------------------------------------------------------------------------

- (NSView *)containerView;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TARGET/ACTION METHODS
// ----------------------------------------------------------------------------------------

- (IBAction)changeMode:(id)sender;
- (IBAction)add:(id)sender;
- (IBAction)remove:(id)sender;
- (IBAction)hit:(id)sender;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSWindow *)window;

- (id)viewsListView;
- (void)setViewsListView:(id)value;

- (id)userInfo;
- (void)setUserInfo:(id)value;

- (int)mode;
- (void)setMode:(int)value;

- (NSString *)changeNotificationName;
- (void)setChangeNotificationName:(NSString *)value;

@end
