//
//  DSChooserIntern.h
//  Shared classes
//
//  Created by David Sinclair on Tues Mar 02 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//
//  An "intern" is a component used by the DSAssistantController class -- the label was chosen as a description of a subordinate assistant.
//

#import "DSInternProtocol.h"


@interface DSChooserIntern : NSObject <DSInternProtocol>
{
    NSString *_name;
    NSArray *_items;
    
    IBOutlet NSView *containerView;
    IBOutlet NSTextView *descriptionTextView;
    IBOutlet NSOutlineView *listOutlineView;
    IBOutlet NSSplitView *splitter;
}

// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark HELPER METHODS
// ----------------------------------------------------------------------------------------

+ (NSMutableDictionary *)settingsWithName:(NSString *)name;
+ (void)addLineToSettings:(NSMutableDictionary *)settings basedOnDictionary:(NSMutableDictionary *)dict withName:(NSString *)name identifier:(NSString *)identifier class:(Class)class description:(NSString *)description;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark INTERN PROTOCOL METHODS
// ----------------------------------------------------------------------------------------

- (id)initWithSettings:(NSDictionary *)settings;

- (NSString *)name;
- (NSView *)containerView;
- (BOOL)willFinish;
- (BOOL)canProceed;
- (Class)nextInternClass;
- (NSString *)operation;
- (NSDictionary *)settings;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TARGET/ACTION METHODS
// ----------------------------------------------------------------------------------------

- (IBAction)outlineViewHit:(id)sender;
- (IBAction)doubleClicked:(id)sender;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark NOTIFICATION METHODS
// ----------------------------------------------------------------------------------------

- (void)outlineViewSelectionDidChange:(NSNotification *)aNotification;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark SPLIT VIEW DELEGATE METHODS
// ----------------------------------------------------------------------------------------

- (float)splitView:(NSSplitView *)sender constrainMaxCoordinate:(float)proposedMax ofSubviewAt:(int)offset;
- (float)splitView:(NSSplitView *)sender constrainMinCoordinate:(float)proposedMin ofSubviewAt:(int)offset;

- (BOOL)splitView:(NSSplitView *)sender canCollapseSubview:(NSView *)subview;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark OUTLINE VIEW DATA SOURCE METHODS
// ----------------------------------------------------------------------------------------

- (id)outlineView:(NSOutlineView *)olv child:(int)index ofItem:(id)item;
- (BOOL)outlineView:(NSOutlineView *)olv isItemExpandable:(id)item;
- (int)outlineView:(NSOutlineView *)olv numberOfChildrenOfItem:(id)item;
- (id)outlineView:(NSOutlineView *)olv objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item;

- (void)expandAllItems;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark OUTLINE VIEW DELEGATE METHODS
// ----------------------------------------------------------------------------------------

- (void)outlineView:(NSOutlineView *)olv willDisplayCell:(NSCell *)cell
     forTableColumn:(NSTableColumn *)tableColumn item:(id)item;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (void)setName:(NSString *)value;

- (NSArray *)items;
- (void)setItems:(NSArray *)value;

- (NSString *)internNibName;

@end
