//
//  DSPluginProtocol.h
//  Shared classes
//
//  Created by David Sinclair on Thu Jun 19 2003.
//  Copyright (c) 2003 - 2007 Dejal Systems, LLC. All rights reserved.
//


@protocol DSPluginProtocol

/*
 initializeClass:
 
 The plugin bundle is remembered, for convenient future access.  Returns YES if it is okay to use this plugin, or NO if it should be ignored (e.g. if it requires some feature not available on this machine).
*/

+ (BOOL)initializeClass:(NSBundle *)theBundle;

/*
 terminateClass
 
 Called once the plugin won't be used anymore, this will unload it (not supported by Cocoa at present).
*/

+ (void)terminateClass;

/*
 identifier
 
 Returns a unique identifier for this plugin class.  Must be unique, and must never change, so it can be referenced by saved data.
*/

+ (NSString *)identifier;

/*
 name
 
 Returns the localized display name of this plugin.
*/

+ (NSString *)name;

/*
 settings
 
 Returns a mutable dictionary of settings for use by the plugin.
*/

- (NSMutableDictionary *)settings;

/*
 setSettings:
 
 Set the settings ivar.  Normally set via +pluginWithSettings: instead.
*/

- (void)setSettings:(NSMutableDictionary *)value;

@end


@interface NSObject (DSPluginInformalProtocol)

/*
 pluginWithSettings:
 
 Returns a new instance of this plugin class, using the specified settings (which are plugin-specific).  Plugins must implement either this or +pluginForType:withSettings:, below.  Use this variation if a plugin class only supports one type of operation.
*/

+ (id)pluginWithSettings:(NSMutableDictionary *)settings;

/*
 pluginForType:withSettings:
 
 Returns a new instance of this plugin class, using the specified settings (which are plugin-specific).  Plugins must implement either this or +pluginWithSettings:, above.  Use this variation if a plugin class supports multiple types of plugin operations.
*/

+ (id)pluginForType:(NSString *)type withSettings:(NSMutableDictionary *)settings;

@end

