#import "DSProgressCellController.h"
#import "DSProgressCell.h"


@implementation DSProgressCellController

+ (id)controller
{
    return [[[self alloc] init] autorelease];
}

- (id)init
{
    if ((self = [super init]))
    {
        if (![NSBundle loadNibNamed:@"DSProgressCell" owner:self])
        {
            [self release];
            self = nil;
        }
    }
    
    // Shouldn't need to do this, but they seem to be vanishing sometimes, and this object will normally hang around for the entire life of the app anyway:
    [progressView retain];
    [progressIndicator retain];
    
    [self setAnimating:NO];
    
    return self;
}

- (void)dealloc
{
    // Need to release the view manually, as this is not part of a window controller, so top-level Nib objects aren't automatically released:
    [progressView release];
    
    // And do extra releases to balance the extra retains above:
    [progressIndicator release];
    [progressView release];
    
    [super dealloc];
}

- (NSView *)progressView
{
    // This is a hack to mask a bug somewhere, where this or progressIndicator is mysteriously getting released:
    while ([progressView retainCount] < 3)
        [progressView retain];
    
    return progressView;
}

- (NSProgressIndicator *)progressIndicator
{
    // This is a hack to mask a bug somewhere, where this or progressView is mysteriously getting released:
    while ([progressIndicator retainCount] < 3)
        [progressIndicator retain];
    
    return progressIndicator;
}

- (BOOL)isAnimating
{
    return _isAnimating;
}

- (void)setAnimating:(BOOL)value
{
    _isAnimating = value;
}

- (void)toggleAnimation:(BOOL)value
{
    if (value)
        [self startAnimation];
    else
        [self stopAnimation];
}

- (void)startAnimation
{
    if (![self isAnimating])
    {
        [[self progressIndicator] startAnimation:nil];
        
        [self setAnimating:YES];
    }
}

- (void)stopAnimation
{
    if ([self isAnimating])
    {
        [[self progressIndicator] stopAnimation:nil];
        
        [self setAnimating:NO];
        
        [[self progressView] removeFromSuperviewWithoutNeedingDisplay];
    }
}

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@implementation NSTableView (DSProgressExtensions)

/*
 reloadDataAndProgress
 
 Call this instead of -reloadData when the table includes progress cells and rows are being removed, and progress indicators may be animating, so the progress subviews are correctly reset too.  Safe to call for non-progress tables.  No need to call if the animation is guaranteed to be stopped before removing the row.
 
 Written by DJS 2004-12.
*/

- (void)reloadDataAndProgress
{
    while ([[self subviews] count])
        [[[self subviews] lastObject] removeFromSuperviewWithoutNeedingDisplay];
    
    [self reloadData];
}

/*
 setProgressController:forCell:
 
 Call this from -tableView:willDisplayCell:forTableColumn:row: for the progress column, to adjust the progress cell to use the indicated controller.
 
 Written by DJS 2004-12.
*/

- (void)setProgressController:(DSProgressCellController *)progressController forCell:(id)cell
{
    if ([cell isKindOfClass:[DSProgressCell class]])
        [(DSProgressCell *)cell setProgressController:progressController];
}

@end
