//
//  DSProgressData.h
//  Shared classes
//
//  Created by David Sinclair on Wed Jun 11 2003.
//  Copyright (c) 2003 - 2005 Dejal Systems, LLC. All rights reserved.
//

#import <Foundation/Foundation.h>


typedef enum
{
    ProgressModeFinished,
    ProgressModeNormal,
    ProgressModeCollapsed,
    ProgressModeAlert,
    ProgressModeConfirmation
} ProgressModeType;


@interface DSProgressData : NSObject
{
    NSMutableArray *_items;
    NSMutableDictionary *_currentItem;

    NSString *_windowTitle;
    NSString *_operation;
    NSString *_sizeAction;
    NSString *_sourceName;
    NSString *_destinationName;
    NSString *_button1Title;
    NSString *_button2Title;
    NSString *_button3Title;
    NSString *_message;

    ProgressModeType _mode;
    int _countNth;
    int _sizeNth;
    int _sizeTotal;

    BOOL _stopped;
}

- (BOOL)modeIsNormalOrCollapsed;

- (void)setModeToDefaultAlways:(BOOL)always;


- (void)startOverWithWindowTitle:(NSString *)windowTitle
                       operation:(NSString *)operation
                      sizeAction:(NSString *)sizeAction
                      sourceName:(NSString *)sourceName
                 destinationName:(NSString *)destinationName
                     buttonTitle:(NSString *)buttonTitle;

- (void)reset;

- (BOOL)donePreparing;

- (BOOL)preparingObject:(id)object withName:(NSString *)name size:(int)size;
- (BOOL)preparingObject:(id)object withName:(NSString *)name size:(int)size parentPath:(NSString *)parentPath level:(int)level;

- (void)changeOperation:(NSString *)operation sizeAction:(NSString *)sizeAction
            buttonTitle:(NSString *)buttonTitle;

- (id)nextObject;

- (void)changeToAlertWithMessage:(NSString *)message;
- (void)changeToAlertWithMessage:(NSString *)message buttonTitle:(NSString *)buttonTitle;


- (NSString *)windowTitle;
- (void)setWindowTitle:(NSString *)value;

- (NSString *)operation;
- (void)setOperation:(NSString *)value;

- (NSString *)sizeAction;
- (void)setSizeAction:(NSString *)value;

- (NSString *)sourceName;
- (void)setSourceName:(NSString *)value;

- (NSString *)destinationName;
- (void)setDestinationName:(NSString *)value;

- (NSString *)button1Title;
- (void)setButton1Title:(NSString *)value;

- (NSString *)button2Title;
- (void)setButton2Title:(NSString *)value;

- (NSString *)button3Title;
- (void)setButton3Title:(NSString *)value;

- (NSString *)message;
- (void)setMessage:(NSString *)value;

- (ProgressModeType)mode;
- (void)setMode:(ProgressModeType)value;

- (int)countRemaining;
- (void)setCountRemaining:(int)value;

- (int)countNth;
- (void)setCountNth:(int)value;

- (int)countTotal;

- (int)sizeNth;
- (void)setSizeNth:(int)value;

- (int)sizeTotal;
- (void)setSizeTotal:(int)value;

- (BOOL)stopped;
- (void)setStopped:(BOOL)value;


- (NSString *)currentName;
- (void)setCurrentName:(NSString *)value;

- (int)currentSize;
- (void)setCurrentSize:(int)value;

- (NSString *)currentPath;

- (NSString *)currentParentPath;
- (void)setCurrentParentPath:(NSString *)parentPath;

- (int)currentLevel;
- (void)setCurrentLevel:(int)value;

- (id)currentObject;
- (void)setCurrentObject:(id)value;

@end
