#import "DSProgressViewController.h"
#import "DSSharedDefines.h"


@implementation DSProgressViewController

- (id)init
{
    self = [super init];

    [PREFERENCES registerDefaults:[NSDictionary dictionaryWithObjectsAndKeys:
        [NSNumber numberWithInt:1], PROGRESS_FINDERLIKE_ESTIMATES,
        [NSNumber numberWithInt:0], PROGRESS_START_COLLAPSED,
        nil]];

    [self setProgressData:[[[DSProgressData alloc] init] autorelease]];
    
    [NSBundle loadNibNamed:@"DSProgressView" owner:self];

    return self;
}

- (void)dealloc
{
    [self setProgressData:nil];
    [self setContainerView:nil];
    [self setStartDate:nil];
    
    [super dealloc];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark VIEW ADJUSTMENT METHODS
// ----------------------------------------------------------------------------------------

- (NSString *)_buildOperationString
{
    DSProgressData *progressData = [self progressData];
    NSString *operation = [progressData operation];

    if (![operation length])
        operation = NSLocalizedStringFromTable(@"Preparing", @"DSShared", @"default progress operation");

    if ([progressData countTotal] == 1 && [[progressData currentName] length])
        operation = [NSString stringWithFormat:NSLocalizedStringFromTable(@"%@ \"%@\"", @"DSShared", @"single progress operation"), operation, [progressData currentName]];
    else if ([progressData countRemaining])
        operation = [NSString stringWithFormat:NSLocalizedStringFromTable(@"%@ %d %@", @"DSShared", @"remaining progress operation"), operation, [progressData countRemaining], [NSString stringWithIntValue:[progressData countRemaining] zero:nil singluar:NSLocalizedStringFromTable(@"item", @"DSShared", @"item progress operation") plural:NSLocalizedStringFromTable(@"items", @"DSShared", @"items progress operation")]];

    if ([[progressData sourceName] length])
        operation = [NSString stringWithFormat:NSLocalizedStringFromTable(@"%@ from \"%@\"", @"DSShared", @"from progress operation"), operation, [progressData sourceName]];

    if ([[progressData destinationName] length])
        operation = [NSString stringWithFormat:NSLocalizedStringFromTable(@"%@ to \"%@\"", @"DSShared", @"to progress operation"), operation, [progressData destinationName]];

    return operation;
}

- (void)_adjustDisclosureButton:(NSButton *)disclosureButton viewIsChanging:(BOOL)viewIsChanging
{
    NSString *operation = [self _buildOperationString];

    if (viewIsChanging)
        [[disclosureButton cell] setHighlightsBy:NSPushInCellMask];

    if (![[disclosureButton title] isEqualToString:operation])
        [disclosureButton setTitle:operation];
}

- (void)_adjustStopButton:(NSButton *)stopButton dimStopIfStopped:(BOOL)dimStop
{
    DSProgressData *progressData = [self progressData];

    if (![[stopButton title] isEqualToString:[progressData button1Title]])
        [stopButton setTitle:[progressData button1Title]];

    if (dimStop && ([stopButton isEnabled] == [progressData stopped]))
        [stopButton setEnabled:![progressData stopped]];
}

- (void)_adjustItemStat:(NSTextField *)itemStat
{
    DSProgressData *progressData = [self progressData];
    NSString *itemPrefix;
    NSString *itemString = @"";

    // Include the countTotal in the item prefix if we're still counting:
    if (![progressData donePreparing])
        itemPrefix = [NSString stringWithFormat:NSLocalizedStringFromTable(@"Item %d", @"DSShared", @"Item scanning prefix for progress"), [progressData countTotal]];
    else
        itemPrefix = NSLocalizedStringFromTable(@"Item", @"DSShared", @"Item actioning prefix for progress");

    // Include the name of the current item, if any:
    if ([progressData currentName])
        itemString = [NSString stringWithFormat:NSLocalizedStringFromTable(@"%@: \"%@\"", @"DSShared", @"Item progress"), itemPrefix, [progressData currentName]];
    else if ([progressData countTotal])
        itemString = itemPrefix;

    if (![[itemStat stringValue] isEqualToString:itemString])
        [itemStat setStringValue:itemString];
}

- (void)_adjustSizeStat:(NSTextField *)sizeStat
{
    DSProgressData *progressData = [self progressData];
    NSString *sizeString = @"";

    // Only include the size details if we're done preparing:
    if ([progressData donePreparing])
    {
        NSString *action = [NSLocalizedStringFromTable(@"Completed", @"DSShared", @"default action progress") or:[progressData sizeAction]];
        NSString *nth = [NSString stringAsBytesWithInt:[progressData sizeNth]];
        NSString *total = [NSString stringAsBytesWithInt:[progressData sizeTotal]];

        sizeString = [NSString stringWithFormat:NSLocalizedStringFromTable(@"%@: %@ of %@", @"DSShared", @"Progress sizes, e.g. 'Copied: 123.4 MB of 500.0 MB'"), action, nth, total];
    }

    if (![[sizeStat stringValue] isEqualToString:sizeString])
        [sizeStat setStringValue:sizeString];
}

- (void)_adjustTimeStat:(NSTextField *)timeStat
{
    DSProgressData *progressData = [self progressData];
    NSString *timeString = @"";

    // Only use the time display when we're done preparing:
    if ([progressData donePreparing])
    {
        // If we haven't started the time estimation, do so now:
        if (![self startDate])
            [self setStartDate:[NSDate date]];

        int seconds = [[NSDate date] timeIntervalSinceDate:[self startDate]];
        int minutes = 0;
        float nthFloat = [progressData sizeNth];
        float totalFloat = [progressData sizeTotal];
        float percentage = (nthFloat / totalFloat) * 100.0;
        
        // Wait for things to settle down, and for something to actually be done:
        if (seconds < 5 || percentage < 1.0)
            timeString = NSLocalizedStringFromTable(@"Estimating time remaining...", @"DSShared", @"Estimating time progress");
        else
        {
            // Calculate remaining seconds based on how much has been done and how long it took to do that:
            seconds = (seconds * (100.0 - percentage)) / percentage;
            minutes = seconds / 60;

            if ([PREFERENCES integerForKey:PROGRESS_FINDERLIKE_ESTIMATES])
            {
                if (minutes >= 120)
                    timeString = [NSString stringWithFormat:NSLocalizedStringFromTable(@"About %d hours", @"DSShared", @">= 2 hours remaining progress"), minutes / 60];
                else if (minutes >= 50)
                    timeString = NSLocalizedStringFromTable(@"About an hour", @"DSShared", @">= 50 minutes remaining progress");
                else if (seconds >= 120)
                    timeString = [NSString stringWithFormat:NSLocalizedStringFromTable(@"About %d minutes", @"DSShared", @">= 120 seconds remaining progress"), seconds / 60];
                else if (seconds >= 50)
                    timeString = NSLocalizedStringFromTable(@"About a minute", @"DSShared", @">= 50 seconds remaining progress");
                else if (seconds >= 20)
                    timeString = [NSString stringWithFormat:NSLocalizedStringFromTable(@"Less than a minute", @"DSShared", @">= 20 seconds remaining progress"), seconds / 60];
                else if (seconds >= 7)
                    timeString = [NSString stringWithFormat:NSLocalizedStringFromTable(@"About 10 seconds", @"DSShared", @">= 7 seconds remaining progress"), seconds / 60];
                else
                    timeString = [NSString stringWithFormat:NSLocalizedStringFromTable(@"About 5 seconds", @"DSShared", @">= 0 seconds remaining progress"), seconds / 60];
            }
            else
            {
                seconds = minutes % 60;

                if (minutes >= 120)
                {
                    int hours = minutes / 60;
                    minutes = minutes % 60;
                    
                    timeString = [NSString stringWithFormat:NSLocalizedStringFromTable(@"%d hours %d min", @"DSShared", @"Hours and minutes remaining progress"), hours, minutes];
                }
                else if (minutes > 0)
                    timeString = [NSString stringWithFormat:NSLocalizedStringFromTable(@"%d min %d sec", @"DSShared", @"Minutes and seconds remaining progress"), minutes, seconds];
                else if (seconds != 1)
                    timeString = [NSString stringWithFormat:NSLocalizedStringFromTable(@"%d seconds", @"DSShared", @"Seconds (plural) remaining progress"), seconds];
                else
                    timeString = [NSString stringWithFormat:NSLocalizedStringFromTable(@"%d second", @"DSShared", @"Second (singular) remaining progress"), seconds];
            }
            
            timeString = [NSLocalizedStringFromTable(@"Time remaining: ", @"DSShared", @"Time remaining prefix for progress") stringByAppendingString:timeString];
        }
    }
    
    if (![[timeStat stringValue] isEqualToString:timeString])
        [timeStat setStringValue:timeString];
}

- (void)_adjustProgressBar:(NSProgressIndicator *)progressBar
{
    DSProgressData *progressData = [self progressData];

    // Determinate if we're done preparing, otherwise indeterminate:
    if ([progressData donePreparing])
    {
        [progressBar setIndeterminate:NO];
        [progressBar setDoubleValue:[progressData sizeNth]];
        [progressBar setMaxValue:[progressData sizeTotal]];
    }
    else
    {
        [progressBar setIndeterminate:YES];
        [progressBar animate:nil];
    }    
}

/*
 adjustContentsViewChanged

 Updates the appropriate container view from the ivars.  Returns YES if the container view was swapped for another one (i.e. the mode changed).  N.B.: This is the *ONLY* place where the view is altered; this must be called from the main thread, but other methods are okay to be called from a worker thread, since they don't affect the user interface.

 Written 2003-06.
*/

- (BOOL)adjustContentsViewChanged
{
    DSProgressData *progressData = [self progressData];
    NSView *oldView = [self containerView];
    NSView *newView = nil;
    ProgressModeType mode = [progressData mode];

/*
    [self setCurrentName:[[NSCalendarDate calendarDate] description]];            // hack
    
    if (!([[NSCalendarDate calendarDate] secondOfMinute] % 10))   	          // hack
        [self setCountRemaining:[self countRemaining] - 1];  	   	          // hack

    [self setSizeNth:[self sizeNth] + [[NSCalendarDate calendarDate] minuteOfHour] * [[NSCalendarDate calendarDate] secondOfMinute] * 10];					            // hack
*/

    if (mode == ProgressModeFinished)
    {
        newView = nil;
    }
    else if (mode == ProgressModeAlert)
    {
        newView = alertContainerView;

        NSString *operation = [self _buildOperationString];

        [self _adjustStopButton:alertOKButton dimStopIfStopped:NO];

        if (![[alertOperationStat stringValue] isEqualToString:operation])
            [alertOperationStat setStringValue:operation];

        [alertMessageStat setStringValue:[NSLocalizedStringFromTable(@"Sorry, an unexpected error occurred.  The operation can not continue.", @"DSShared", @"Missing error message for progress alert") or:[progressData message]]];

        [self _adjustProgressBar:alertProgressBar];
    }
/*	not yet implemented:
    else if (mode == ProgressModeConfirmation)
    {
        
    }
*/
    else if (mode == ProgressModeCollapsed)
    {
        newView = collapsedContainerView;

        [self _adjustDisclosureButton:collapsedDisclosureButton viewIsChanging:oldView != newView];
        [self _adjustStopButton:collapsedStopButton dimStopIfStopped:YES];
        [self _adjustProgressBar:collapsedProgressBar];

        // If we're done preparing, display the time remaining, otherwise display the current item:
        if ([progressData donePreparing])
            [self _adjustTimeStat:collapsedTimeStat];
        else
            [self _adjustItemStat:collapsedTimeStat];
    }
    else
    {
        newView = normalContainerView;

        [self _adjustDisclosureButton:normalDisclosureButton viewIsChanging:oldView != newView];
        [self _adjustStopButton:normalStopButton dimStopIfStopped:YES];
        [self _adjustItemStat:normalCurrentItemStat];
        [self _adjustSizeStat:normalSizeStat];
        [self _adjustTimeStat:normalTimeStat];
        [self _adjustProgressBar:normalProgressBar];
    }

    [self setContainerView:newView];
    
    return oldView != newView;
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TARGET/ACTION METHODS
// ----------------------------------------------------------------------------------------

/*
 alertOKButtonHit:

 Action method for the OK button in the Alert view.  Simply notes that the view should be closed.  The client method should have no more interest in this view; it should release it after changing to an alert.
 
 Written 2003-06.
*/

- (IBAction)alertOKButtonHit:(id)sender
{
/*
    [self setMode:ProgressModeNormal];            // hack
    [self setMessage:@""];          		  // hack
    [self setOperation:@"Decrypting"];
    [self setButton1Title:@"Finish"];		  // hack
    [self setCurrentName:[[NSCalendarDate calendarDate] descriptionWithCalendarFormat:@"%H:%M:%S"]]; // hack
    [self setCountRemaining:100];
    [self setCountTotal:100];
    [self setSizeNth:0];
    [self setSizeTotal:1234567];    	          // hack
*/
    
    [[self progressData] setMode:ProgressModeFinished];
}

/*
 collapsedDisclosureButtonHit:

 Action method for the disclosure button in the Collapsed view.  Toggles to Normal (expanded) mode.

 Written 2003-06.
 */

- (IBAction)collapsedDisclosureButtonHit:(id)sender
{
    [[self progressData] setMode:ProgressModeNormal];

    [PREFERENCES setInteger:0 forKey:PROGRESS_START_COLLAPSED];
}

/*
 collapsedStopButtonHit:

 Action method for the Stop button in the Collapsed view.  See -normalStopButtonHit: for more information.

 Written 2003-06.
 */

- (IBAction)collapsedStopButtonHit:(id)sender
{
/*
    [self setMode:ProgressModeAlert];				// hack
    [self setMessage:@"This is a sample error message."];	// hack
    [self setButton1Title:@"Go Normal"];    		        // hack
*/
    
    [[self progressData] setStopped:YES]; 
}

/*
 normalDisclosureButtonHit:
 
 Action method for the disclosure button in the Normal view.  Toggles to Collapsed mode.
 
 Written 2003-06.
*/

- (IBAction)normalDisclosureButtonHit:(id)sender
{
    [[self progressData] setMode:ProgressModeCollapsed];

    [PREFERENCES setInteger:1 forKey:PROGRESS_START_COLLAPSED];
}

/*
 normalStopButtonHit:
 
 Action method for the Stop button in the Normal view.  It notes that the user clicked the stop button; the client method should poll the stopped accessor and release the progress data (which automatically changes the mode to ProgressModeFinished) when they see that it has stopped (they can continue and even change modes if desired, e.g. to clean up).  The stop button is dimmed; they can cancel the stop if desired, too.
 
 Written 2003-06.
*/

- (IBAction)normalStopButtonHit:(id)sender
{
//    [self setMode:ProgressModeFinished];            // hack

    [[self progressData] setStopped:YES]; 
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (DSProgressData *)progressData
{
    return _progressData;
}

- (void)setProgressData:(DSProgressData *)value
{
    [value retain];
    [_progressData autorelease];
    _progressData = value;
}

- (NSView *)containerView
{
    return _containerView;
}

- (void)setContainerView:(NSView *)value
{
    [value retain];
    [_containerView autorelease];
    _containerView = value;
}

- (NSDate *)startDate
{
    return _startDate;
}

- (void)setStartDate:(NSDate *)value
{
    [value retain];
    [_startDate autorelease];
    _startDate = value;
}

@end
