#import "DSSplashWindow.h"

@implementation DSSplashWindow

// In Interface Builder we set DSSplashWindow to be the class for our window, so our own initializer is called here.
- (id)initWithContentRect:(NSRect)contentRect styleMask:(unsigned int)aStyle backing:(NSBackingStoreType)bufferingType defer:(BOOL)flag
{
    //Call NSWindow's version of this function, but pass in the all-important value of NSBorderlessWindowMask
    //for the styleMask so that the window doesn't have a title bar
    NSWindow *result = [super initWithContentRect:contentRect styleMask:NSBorderlessWindowMask backing:NSBackingStoreBuffered defer:NO];
    //Set the background color to clear so that (along with the setOpaque call below) we can see through the parts
    //of the window that we're not drawing into
    [result setBackgroundColor:[NSColor clearColor]];
    // Make it float above other windows in the app; unfortunaely it floats above other windows in all apps... the only way to avoid that seems to be to make it normal window level... which means it is obsured by the monitor window when it shows.  Arrg!  So now I'm just not showing the monitor window till after removing the splash.  That's fine:
//    [result setLevel:NSFloatingWindowLevel];
    // Center it on the screen:
    [result center];
    //Let's start with full transparency for all drawing into the window
    [result setAlphaValue:0.0];
    //but let's turn off opaqueness so that we can see through the parts of the window that we're not drawing into
    [result setOpaque:NO];
    //and while we're at it, make sure the window has a shadow, which will automatically be the shape of our custom content.  (For Simon, don't want a shadow.)
    [result setHasShadow:NO];
    
    return result;
}

/*
// Custom windows that use the NSBorderlessWindowMask can't become key by default.  Therefore, controls in such windows
// won't ever be enabled by default.  Thus, we override this method to change that.
- (BOOL) canBecomeKeyWindow
{
    return YES;
}

// Once the user starts dragging the mouse, we move the window with it. We do this because the window has no title
// bar for the user to drag (so we have to implement dragging ourselves)
- (void)mouseDragged:(NSEvent *)theEvent
{
    NSPoint currentLocation;
    NSPoint newOrigin;
    NSRect  screenFrame = [[NSScreen mainScreen] frame];
    NSRect  windowFrame = [self frame];


    //grab the current global mouse location; we could just as easily get the mouse location
    //in the same way as we do in -mouseDown:
    currentLocation = [self convertBaseToScreen:[self mouseLocationOutsideOfEventStream]];
    newOrigin.x = currentLocation.x - initialLocation.x;
    newOrigin.y = currentLocation.y - initialLocation.y;

    // Don't let window get dragged up under the menu bar
    if( (newOrigin.y+windowFrame.size.height) > (screenFrame.origin.y+screenFrame.size.height) ){
        newOrigin.y=screenFrame.origin.y + (screenFrame.size.height-windowFrame.size.height);
    }

    //go ahead and move the window to the new location
    [self setFrameOrigin:newOrigin];
}

// We start tracking the a drag operation here when the user first clicks the mouse,
// to establish the initial location.
- (void)mouseDown:(NSEvent *)theEvent
{
    NSRect  windowFrame = [self frame];

    //grab the mouse location in global coordinates
    initialLocation = [self convertBaseToScreen:[theEvent locationInWindow]];
    initialLocation.x -= windowFrame.origin.x;
    initialLocation.y -= windowFrame.origin.y;
}
*/

@end
