//
//  DSSplitSupervisor.h
//  Shared classes
//
//  Created by David Sinclair on Mon May 24 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//
//  A "supervisor" is a special kind of controller that manages a particular view object.
//  This is the supervisor for a NSSplitView.
//

#import "DSSupervisor.h"


@interface DSSplitSupervisor : DSSupervisor
{
    IBOutlet NSSplitView *splitView;
    IBOutlet NSBox *firstBox;
    IBOutlet NSBox *secondBox;
}

- (void)setupWithSupervisors:(NSArray *)supervisorArray;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark CONFIGURATION METHODS
// ----------------------------------------------------------------------------------------

- (NSString *)preferenceForSplitPosition;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark UTILITY METHODS
// ----------------------------------------------------------------------------------------

- (id)supervisorForIdentifier:(NSString *)identifier;

- (void)resizeSplitterViewFirstFixed:(BOOL)firstIsFixed;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark NSSPLITVIEW DELEGATE METHODS
// ----------------------------------------------------------------------------------------

- (float)splitView:(NSSplitView *)sender constrainMaxCoordinate:(float)proposedMax ofSubviewAt:(int)offset;
- (float)splitView:(NSSplitView *)sender constrainMinCoordinate:(float)proposedMin ofSubviewAt:(int)offset;

- (void)splitViewDidResizeSubviews:(NSNotification *)aNotification;

// - (void)splitView:(NSSplitView *)sender resizeSubviewsWithOldSize:(NSSize)oldSize;

- (BOOL)splitView:(NSSplitView *)sender canCollapseSubview:(NSView *)subview;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSView *)firstSubview;
- (NSView *)secondSubview;

- (id)firstSupervisor;
- (id)secondSupervisor;

@end
