//
//  DSSupervisor.h
//  Shared classes
//
//  Created by David Sinclair on Wed May 12 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//
//  A "supervisor" is a special kind of controller that manages a particular view object.
//  This is the supervisor superclass, but it can be instantiated as a concrete class, e.g. for custom views.
//


@interface DSSupervisor : NSObject
{
    NSString *_identifier;
    id _representedObject;
    NSMutableDictionary *_userInfo;
    
    IBOutlet NSView *containerView;
    IBOutlet NSWindow *window;
}

- (void)setup;
- (void)reload;
- (void)prepare;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark NOTIFICATION METHODS
// ----------------------------------------------------------------------------------------

- (void)postNotificationName:(NSString *)name;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark VALIDATION METHODS
// ----------------------------------------------------------------------------------------

- (BOOL)validateToolbarItem:(NSToolbarItem *)toolbarItem;
- (BOOL)validateMenuItem:(NSMenuItem *)anItem;
- (BOOL)validateAction:(SEL)action;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark DATA ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSView *)containerView;

- (NSString *)identifier;
- (void)setIdentifier:(NSString *)value;

- (id)representedObject;
- (void)setRepresentedObject:(id)value;

- (NSMutableDictionary *)userInfo;
- (void)setUserInfo:(NSMutableDictionary *)value;

@end
