//
//  DSTableSupervisor.h
//  Shared classes
//
//  Created by David Sinclair on Fri May 07 2004.
//  Copyright (c) 2004 Dejal Systems, LLC. All rights reserved.
//
//  A "supervisor" is a special kind of controller that manages a particular view object.
//  This is the supervisor for a NSTableView.
//

#import "DSSupervisor.h"


@interface DSTableSupervisor : DSSupervisor
{
    int _selectionCount;
    
    IBOutlet NSTableView *theTableView;
}

- (id)valueWithRowObject:(id)rowObject forIdentifier:(NSString *)identifier;
- (void)setValue:(id)value withRowObject:(id)rowObject forIdentifier:(NSString *)identifier;

- (id)objectForRow:(int)row;
- (int)rowForObject:(id)object;

- (id)selectedObject;
- (NSMutableArray *)selectedOrAllObjects:(BOOL)flag;
- (void)selectObjects:(NSArray *)selected byExtendingSelection:(BOOL)flag;

- (void)sort;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark CONFIGURATION METHODS
// ----------------------------------------------------------------------------------------

- (BOOL)isSortable;

- (NSArray *)acceptsDragPasteboardTypes;
- (NSArray *)dragWithPasteboardTypes;

- (NSString *)internalDraggingPasteboardType;

- (BOOL)supportsDragDelete;

- (BOOL)shouldWaitForDoubleClick;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TABLE VIEW DATA SOURCE METHODS
// ----------------------------------------------------------------------------------------

- (int)numberOfRowsInTableView:(NSTableView *)tableView;
- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row;
- (void)tableView:(NSTableView *)tableView setObjectValue:(id)object forTableColumn:(NSTableColumn *)tableColumn row:(int)row;

- (void)tableView:(NSTableView *)tableView sortDescriptorsDidChange:(NSArray *)oldDescriptors;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TABLE VIEW DRAGGING DATA SOURCE METHODS
// ----------------------------------------------------------------------------------------

- (BOOL)tableView:(NSTableView *)tableView writeRows:(NSArray *)rows toPasteboard:(NSPasteboard *)pboard;

- (NSDragOperation)tableView:(NSTableView *)tableView validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)operation;

- (BOOL)tableView:(NSTableView *)tableView acceptDrop:(id <NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)operation;

- (BOOL)tableView:(NSTableView *)tableView deleteWithPasteboard:(NSPasteboard *)pboard;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TABLE VIEW DELEGATE METHODS
// ----------------------------------------------------------------------------------------

// - (void)tableView:(NSTableView *)tableView didClickTableColumn:(NSTableColumn *)tableColumn;

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification;
- (void)tableViewColumnDidResize:(NSNotification *)aNotification;

  
// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark TARGET / ACTION METHODS
// ----------------------------------------------------------------------------------------

- (IBAction)tableViewHit:(id)sender;
- (void)singleClicked:(id)sender;
- (IBAction)doubleClicked:(id)sender;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSTableView *)currentTableView;


- (int)selectionCount;
- (int)beginSelection;
- (int)endSelection;

@end
