//
//  FilePathField.m
//  Shared classes
//
//  Created by Alec Bartsch on Sat Jan 11 2003.
//  Copyright (c) 2003 Alec Bartsch. All rights reserved.
//

#import "FilePathField.h"


@implementation FilePathField

- (id)init
{
    if ((self = [super init]))
    {
        NSArray *pbTypes = [NSArray arrayWithObjects: NSFilenamesPboardType, nil];
        [self registerForDraggedTypes: pbTypes];
        [self setAcceptsFiles: YES];
        [self setAcceptsDirectories: YES];
    }
    return self;
}

- (id)initWithCoder: (NSCoder *)decoder
{
    [self setAcceptsFiles: YES];
    [self setAcceptsDirectories: YES];
    return [super initWithCoder: decoder];
}

- (void)awakeFromNib
{
    NSArray *pbTypes = [NSArray arrayWithObjects: NSFilenamesPboardType, nil];
    [self registerForDraggedTypes: pbTypes];
}

- (void)setAcceptsFiles: (BOOL)flag
{
    _acceptsFiles = flag;
}

- (BOOL)acceptsFiles
{
    return _acceptsFiles;
}

- (void)setAcceptsDirectories: (BOOL)flag
{
    _acceptsDirectories = flag;
}

- (BOOL)acceptsDirectories
{
    return _acceptsDirectories;
}

- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
{
    NSPasteboard *pboard = [sender draggingPasteboard];
    NSDragOperation sourceDragMask = [sender draggingSourceOperationMask];

    if ( [[pboard types] containsObject: NSFilenamesPboardType] )
    {
        NSString *filename = [[pboard propertyListForType: NSFilenamesPboardType] objectAtIndex: 0];
        BOOL isDirectory;
        [[NSFileManager defaultManager] fileExistsAtPath: filename isDirectory: &isDirectory];

        if ((isDirectory && _acceptsDirectories) || (!isDirectory && _acceptsFiles))
        {
            if ([self currentEditor])
            {
                [self abortEditing];
            }
            else
            {
                [[self cell] setHighlighted: YES];
                return sourceDragMask;
            }
        }
    }
    return NSDragOperationNone;
}

- (void)draggingExited:(id <NSDraggingInfo>)sender
{
    [[self cell] setHighlighted: NO];
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
    NSPasteboard *pboard = [sender draggingPasteboard];
    [[self cell] setHighlighted: NO];
    [self setStringValue: [[pboard propertyListForType: NSFilenamesPboardType] objectAtIndex: 0]];
    [self sendAction: [self action] to: [self target]];
    return YES;
}

@end
