//
//  DSBackgroundView.m
//  Shared classes
//
//  Created by David Sinclair on 2005-08-27.
//  Copyright 2005 - 2006 Dejal Systems, LLC. All rights reserved.
//

#import "DSBackgroundView.h"


@implementation DSBackgroundView

- (id)initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
    
    if (self)
    {
        [self setBackgroundColor:nil];
        [self setBorderColor:nil];
        [self setDrawsBackground:YES];
        [self setDrawsBorder:YES];
    }
    
    return self;
}

- (void)dealloc
{
    [self setBackgroundColor:nil];
    [self setBorderColor:nil];
    
    [super dealloc];
}

- (BOOL)isOpaque
{
    return ([self drawsBackground] && [[self backgroundColor] alphaComponent] == 1.0);
}

- (void)drawRect:(NSRect)rect
{
    if ([self drawsBackground])
    {
        [[self backgroundColor] set];
        NSRectFillUsingOperation(rect, NSCompositeSourceOver);
    }
    
    if ([self drawsBorder])
    {
        [[self borderColor] set];
        NSFrameRectWithWidthUsingOperation([self bounds], 1.0, NSCompositeSourceOver);
    }
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSORS METHODS
// ----------------------------------------------------------------------------------------

- (NSColor *)backgroundColor
{
    if (!_backgroundColor)
        [self setBackgroundColor:[NSColor colorWithCalibratedRed:0.906 green:0.929 blue:0.965 alpha:1.0]];
    
    return _backgroundColor;
}

- (void)setBackgroundColor:(NSColor *)value
{
    [value retain];
    [_backgroundColor autorelease];
    _backgroundColor = value;
}

- (NSColor *)borderColor
{
    if (!_borderColor)
        [self setBorderColor:[NSColor lightGrayColor]];
    
    return _borderColor;
}

- (void)setBorderColor:(NSColor *)value
{
    [value retain];
    [_borderColor autorelease];
    _borderColor = value;
}

- (BOOL)drawsBackground
{
    return _drawsBackground;
}

- (void)setDrawsBackground:(BOOL)value
{
    _drawsBackground = value;
}

- (BOOL)drawsBorder
{
    return _drawsBorder;
}

- (void)setDrawsBorder:(BOOL)value
{
    _drawsBorder = value;
}

@synthesize _borderColor;
@synthesize _backgroundColor;

@end
