//
//  DSDateControl.m
//  Shared classes
//
//  Created by David Sinclair on Fri Nov 28 2003.
//  Copyright (c) 2003 Dejal Systems, LLC. All rights reserved.
//

#import "DSDateControl.h"


#define DSDATECONTROL_LEFT_OFFSET		3.5
#define DSDATECONTROL_INTERCELL_SPACE           4
#define DSDATECONTROL_RIGHT_OFFSET		3
#define DSDATECONTROL_TOP_OFFSET		3
#define DSDATECONTROL_BOTTOM_OFFSET		9

int _DSDateControlMax[WBTIMECONTROL_CELL_COUNT] = {9999,12,31};


@implementation DSDateControl

- (id)initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
    
    if (self)
    {
        int i;
        NSRect tCellFrame;
        
        // Override the default cell values:
        [(NSTextFieldCell *)cells[[self indexForDateComponent:DSDATECONTROL_YEAR_ID]] setStringValue:[NSString stringWithFormat:@"%04d",[currentDate yearOfCommonEra]]];
        [(NSTextFieldCell *)cells[[self indexForDateComponent:DSDATECONTROL_MONTH_ID]] setStringValue:[NSString stringWithFormat:@"%02d",[currentDate monthOfYear]]];
        [(NSTextFieldCell *)cells[[self indexForDateComponent:DSDATECONTROL_DAY_ID]] setStringValue:[NSString stringWithFormat:@"%02d",[currentDate dayOfMonth]]];
        
        // Override the 2 colon cells:
        for (i = 0; i < (WBTIMECONTROL_CELL_COUNT - 1); i++)
            [(NSTextFieldCell *)colonCells[i] setStringValue:@"-"];
        
        // Override the Cells' frame:
        tCellFrame.origin.x = DSDATECONTROL_LEFT_OFFSET;
        tCellFrame.origin.y = DSDATECONTROL_BOTTOM_OFFSET;
        
        tCellFrame.size.width = (NSWidth(frame) - 2 * DSDATECONTROL_INTERCELL_SPACE - DSDATECONTROL_RIGHT_OFFSET - DSDATECONTROL_LEFT_OFFSET) / 4;
        tCellFrame.size.height = NSHeight(frame) - DSDATECONTROL_TOP_OFFSET - DSDATECONTROL_BOTTOM_OFFSET;
        int baseWidth = tCellFrame.size.width;
        int yearIndex = [self indexForDateComponent:DSDATECONTROL_YEAR_ID];
        
        for (i = 0; i < WBTIMECONTROL_CELL_COUNT; i++)
        {
            if (i == yearIndex)
                tCellFrame.size.width = baseWidth * 2;
            
            rects[i] = tCellFrame;
            
            tCellFrame.origin.x = NSMaxX(tCellFrame) + DSDATECONTROL_INTERCELL_SPACE;
            
            if (i == yearIndex)
                tCellFrame.size.width = baseWidth;
        }
        
        // Override the Colons' frame:
        for (i = 0; i < (WBTIMECONTROL_CELL_COUNT - 1); i++)
        {
            colonRects[i] = tCellFrame;
            
            colonRects[i].origin.x = NSMaxX(rects[i]) - 3;
            colonRects[i].size.width = NSMinX(rects[i + 1]) - NSMinX(colonRects[i]) - 1;
        }
    }
    
    return self;
}

- (void)setDate:(id)aDate
{
    if (![aDate isKindOfClass:[NSCalendarDate class]])
        aDate = [NSCalendarDate dateWithTimeIntervalSinceReferenceDate:[aDate timeIntervalSinceReferenceDate]];
    
    if ([currentDate isEqualToDate:aDate] == NO)
    {
        if (currentDate != aDate)
        {
            [currentDate release];
        	currentDate = [aDate copy];
        }
    
        [self editOff];
        
        [cells[[self indexForDateComponent:DSDATECONTROL_YEAR_ID]] setStringValue:[NSString stringWithFormat:@"%04d", [self year]]];
        [cells[[self indexForDateComponent:DSDATECONTROL_MONTH_ID]] setStringValue:[NSString stringWithFormat:@"%02d", [self month]]];
        [cells[[self indexForDateComponent:DSDATECONTROL_DAY_ID]] setStringValue:[NSString stringWithFormat:@"%02d", [self day]]];
    }
}

- (void)setHour:(int)aHour
{
    // Overridden to do nothing, since it isn't relevant for the date control subclass.
}

- (void)setMinute:(int)aMinute
{
    // Overridden to do nothing, since it isn't relevant for the date control subclass.
}

- (void) setSecond:(int)aSecond
{
    // Overridden to do nothing, since it isn't relevant for the date control subclass.
}

- (void)setYear:(int)aYear
{
    int tYear = [self year];
    
    if (aYear > 0 && aYear <= _DSDateControlMax[[self indexForDateComponent:DSDATECONTROL_YEAR_ID]])
    {
        NSCalendarDate *tDate;
        
        tDate = [currentDate dateByAddingYears:aYear - tYear
                                        months:0
                                          days:0
                                         hours:0
                                       minutes:0
                                       seconds:0];
        
        [currentDate release];
        
        currentDate = [tDate retain];
    
        stepperMidValue = aYear;
        
        if (isUsingFieldEditor == YES && selected == [self indexForDateComponent:DSDATECONTROL_YEAR_ID])
        {
            [[self currentEditor] setString:[NSString stringWithFormat:@"%04d",aYear]];
                
            [self editOff];
        }
        else
        {
            [cells[[self indexForDateComponent:DSDATECONTROL_YEAR_ID]] setStringValue:[NSString stringWithFormat:@"%04d",aYear]];
        }
    }
}

- (int)year
{
    return [currentDate yearOfCommonEra];
}

- (void)setMonth:(int)aMonth
{
    int tMonth = [self month];
        
    if (aMonth > 0 && aMonth <= _DSDateControlMax[[self indexForDateComponent:DSDATECONTROL_MONTH_ID]])
    {
        NSCalendarDate * tDate;
        
        tDate=[currentDate dateByAddingYears:0
                                      months:aMonth - tMonth
                                        days:0
                                       hours:0
                                     minutes:0
                                     seconds:0];
        
        [currentDate release];
        
        currentDate = [tDate retain];
    
        stepperMidValue = aMonth;
        
        if (isUsingFieldEditor == YES && selected == [self indexForDateComponent:DSDATECONTROL_MONTH_ID])
        {
            [[self currentEditor] setString:[NSString stringWithFormat:@"%02d",aMonth]];
                
            [self editOff];
        }
        else
        {
            [cells[[self indexForDateComponent:DSDATECONTROL_MONTH_ID]] setStringValue:[NSString stringWithFormat:@"%02d",aMonth]];
        }
    }
}

- (int)month
{
    return [currentDate monthOfYear];
}

- (void)setDay:(int)aDay
{
    int tDay = [self day];
    
    if (aDay > 0 && aDay <= _DSDateControlMax[[self indexForDateComponent:DSDATECONTROL_DAY_ID]])
    {
        NSCalendarDate * tDate;
        
        tDate=[currentDate dateByAddingYears:0
                                      months:0
                                        days:aDay - tDay
                                       hours:0
                                     minutes:0
                                     seconds:0];
        
        [currentDate release];
        
        currentDate=[tDate retain];
        
        stepperMidValue = aDay;
        
        if (isUsingFieldEditor == YES && selected == [self indexForDateComponent:DSDATECONTROL_DAY_ID])
        {
            [[self currentEditor] setString:[NSString stringWithFormat:@"%02d",aDay]];
                
            [self editOff];
        }
        else
        {
            [cells[[self indexForDateComponent:DSDATECONTROL_DAY_ID]] setStringValue:[NSString stringWithFormat:@"%02d",aDay]];
        }
    }
}

- (int)day
{
    return [currentDate dayOfMonth];
}

- (void)setSelected:(int)aSelected
{
    selected = aSelected;
    
    if (stepper != nil)
    {
        [stepper setMinValue:1];
        [stepper setMaxValue:_DSDateControlMax[selected]];
        
        if (selected == [self indexForDateComponent:DSDATECONTROL_YEAR_ID])
            [stepper setIntValue:[self year]];
        else if (selected == [self indexForDateComponent:DSDATECONTROL_MONTH_ID])
            [stepper setIntValue:[self month]];
        else if (selected == [self indexForDateComponent:DSDATECONTROL_DAY_ID])
            [stepper setIntValue:[self day]];
        
        stepperMidValue = [stepper doubleValue];
    }
    
    [self setNeedsDisplay];
}

- (void)awakeFromNib
{
    [self setSelected:0];
}

- (BOOL)acceptNewValueInSelectedCell:(id)sender
{
    NSString *string;
    int tValue;
	
    string = [[[[self currentEditor] string] copy] autorelease];

    tValue = [string intValue];
    
    if (tValue > 0 && tValue <= _DSDateControlMax[selected])
    {
        if (selected == [self indexForDateComponent:DSDATECONTROL_YEAR_ID])
            [cells[selected] setStringValue:[NSString stringWithFormat:@"%04d",tValue]];
        else
            [cells[selected] setStringValue:[NSString stringWithFormat:@"%02d",tValue]];
                
        // Set the new date:
        [self hitStepperWithValue:tValue];
        
        return YES;
    }
    
    return NO;
}

- (BOOL)textView:(NSTextView *)textView shouldChangeTextInRange:(NSRange)affectedCharRange replacementString:(NSString *)replacementString
{
    int i, tLength, maxLen;
    unichar tUniChar;
    
    tLength = [replacementString length];
    
    if (selected == [self indexForDateComponent:DSDATECONTROL_YEAR_ID])
        maxLen = 4;
    else
        maxLen = 2;
    
    if (affectedCharRange.location >= maxLen || affectedCharRange.length > maxLen)
        return NO;
    
    for (i = 0; i < tLength; i++)
    {
        tUniChar = [replacementString characterAtIndex:i];
        
        if (tUniChar < '0' || tUniChar > '9')
            return NO;
    }
    
    if (stepper)
    {
        NSMutableString *tString;
        int tValue;
        
        tString = [[[textView string] mutableCopy] autorelease];
        
        [tString replaceCharactersInRange:affectedCharRange withString:replacementString];
        
        tValue = [tString intValue];
        
        if (tValue > _DSDateControlMax[selected])
            [stepper setDoubleValue:stepperMidValue];
        else
            [stepper setIntValue:tValue];
    }
    
    return YES;
}


- (void)hitStepperWithValue:(int)value
{
    if (selected == [self indexForDateComponent:DSDATECONTROL_YEAR_ID])
        [self setYear:value];
    else if (selected == [self indexForDateComponent:DSDATECONTROL_MONTH_ID])
        [self setMonth:value];
    else if (selected == [self indexForDateComponent:DSDATECONTROL_DAY_ID])
        [self setDay:value];
}

- (IBAction)takeIntValueFrom:(id)sender
{
    [self hitStepperWithValue:[sender intValue]];
}


- (int)indexForDateComponent:(int)component
{
    // ¶¶ to be implemented: for now, just returns the date components in ISO order; later make it respect the user's prefs.
    
//    NSString *format = [[NSUserDefaults standardUserDefaults] stringForKey:NSShortDateFormatString];
    
//    NSLog(@"format = %@", format);		// ¶¶ log
    
    return component;
}

@end
