//
//  DSEditableBrowserCell.m
//  Shared classes
//
//  Created by David Sinclair on Thu Nov 20 2003.
//  Copyright (c) 2003 - 2004 Dejal Systems, LLC. All rights reserved.
//

#import "DSEditableBrowserCell.h"


@interface NSObject (DSBrowserDelegate)

- (NSMenu *)browser:(NSBrowser *)browser menuForRow:(int)row column:(int)column;

@end


@implementation DSEditableBrowserCell

- (void)editWithFrame:(NSRect)aRect inView:(NSView *)controlView editor:(NSText *)textObj delegate:(id)anObject event:(NSEvent *)theEvent
{
    NSRect textFrame, imageFrame;
    NSDivideRect(aRect, &imageFrame, &textFrame, 3 + [[self image] size].width, NSMinXEdge);
    
    [textObj setTextColor:[NSColor blackColor]];    // ¶¶ doesn't seem to work!
    
    [super editWithFrame:textFrame inView:controlView editor:textObj delegate:anObject event:theEvent];
}

- (void)selectWithFrame:(NSRect)aRect inView:(NSView *)controlView editor:(NSText *)textObj delegate:(id)anObject start:(int)selStart length:(int)selLength
{
    NSRect textFrame, imageFrame;
    NSDivideRect(aRect, &imageFrame, &textFrame, 3 + [[self image] size].width, NSMinXEdge);
    
    [textObj setTextColor:[NSColor blackColor]];    // ¶¶ doesn't seem to work!
    
    [super selectWithFrame:textFrame inView:controlView editor:textObj delegate:anObject start:selStart length:selLength];
}

/*
 menuForEvent:inRect:ofView:
 
 Override of a NSCell method, to allow a contextual menu for an individual cell.  You should implement -browser:menuForRow:column: in your browser delegate.  If that isn't implemented, this returns the default menu.
 
 Written by DJS 2004-06.
*/

- (NSMenu *)menuForEvent:(NSEvent *)anEvent inRect:(NSRect)cellFrame ofView:(NSMatrix *)matrix
{
    id browser = matrix;
    id delegate = nil;
    
    while (![browser isKindOfClass:[NSBrowser class]] && (browser = [browser superview]))
        ;
    
    if ([browser respondsToSelector:@selector(delegate)])
        delegate = [browser delegate];
    
    if (delegate)
    {
        int column = [browser columnOfMatrix:matrix];
        int row = -1;
        int ignored = -1;
        
        if ([matrix getRow:&row column:&ignored ofCell:self] && [delegate respondsToSelector:@selector(browser:menuForRow:column:)])
            return [delegate browser:browser menuForRow:row column:column];
    }
    
    return [super menuForEvent:anEvent inRect:cellFrame ofView:matrix];
}

@end
