//
//  DSIntervalPicker.h
//  Shared classes
//
//  Created by David Sinclair on 2008-07-11.
//  Copyright 2008 Dejal Systems, LLC. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "DSIntervalData.h"


typedef enum _DSIntervalPickerRangeFiltering
{
    DSIntervalAnyRange = 0,
    DSIntervalFirstLessThanSecond = 1,
    DSIntervalFirstLessOrEqualToSecond = 2,
    DSIntervalFirstGreaterOrEqualToSecond = 3,
    DSIntervalFirstGreaterThanSecond = 4
} DSIntervalPickerRangeFiltering;


@interface DSIntervalPickerCell : NSActionCell
{
    // All ivars are private; these are used internally:
    NSTextFieldCell *_firstAmountCell;
    NSTextFieldCell *_rangeSeparatorCell;
    NSTextFieldCell *_secondAmountCell;
    NSTextFieldCell *_unitsCell;
    NSStepperCell *_stepperCell;
    NSTextFieldCell *_selectedCell;
    NSMutableArray *_includedUnitsMapping;
    NSRect _firstAmountFrame;
    NSRect _rangeSeparatorFrame;
    NSRect _secondAmountFrame;
    NSRect _unitsFrame;
    NSRect _stepperFrame;
    BOOL _shouldAppendEdit;
    BOOL _autoPopMenu;
    
    // These ivars have public properties:
    DSIntervalData *_interval;
    DSIntervalPickerRangeFiltering _rangeFiltering;
    NSInteger _minimumAmount;
    NSInteger _maximumAmount;
    BOOL _includeSeconds;
    BOOL _includeMinutes;
    BOOL _includeHours;
    BOOL _includeDays;
    BOOL _includeWeeks;
    BOOL _includeMonths;
    BOOL _includeYears;
    BOOL _includeForever;
    BOOL _includeNever;
}

@property(copy) DSIntervalData *interval;
@property NSInteger firstAmount;
@property NSInteger secondAmount;
@property NSInteger amount;
@property NSInteger units;
@property(readonly) NSTimeInterval firstTimeInterval;
@property(readonly) NSTimeInterval secondTimeInterval;
@property(readonly) NSTimeInterval amountTimeInterval;
@property BOOL usingRange;
@property DSIntervalPickerRangeFiltering rangeFiltering;
@property NSInteger minimumAmount;
@property NSInteger maximumAmount;
@property BOOL includeSeconds;
@property BOOL includeMinutes;
@property BOOL includeHours;
@property BOOL includeDays;
@property BOOL includeWeeks;
@property BOOL includeMonths;
@property BOOL includeYears;
@property BOOL includeForever;
@property BOOL includeNever;

- (void)setupForShortTerm;
- (void)setupForLongTerm;
- (void)setupForAllUnits;

@end


// ----------------------------------------------------------------------------------------
#pragma mark -
// ----------------------------------------------------------------------------------------


@interface DSIntervalPicker : NSControl
{
}

@property(readonly) DSIntervalPickerCell *pickerCell;
@property(copy) DSIntervalData *interval;
@property NSInteger firstAmount;
@property NSInteger secondAmount;
@property NSInteger amount;
@property NSInteger units;
@property(readonly) NSTimeInterval firstTimeInterval;
@property(readonly) NSTimeInterval secondTimeInterval;
@property(readonly) NSTimeInterval amountTimeInterval;
@property BOOL usingRange;
@property DSIntervalPickerRangeFiltering rangeFiltering;
@property NSInteger minimumAmount;
@property NSInteger maximumAmount;
@property BOOL includeSeconds;
@property BOOL includeMinutes;
@property BOOL includeHours;
@property BOOL includeDays;
@property BOOL includeWeeks;
@property BOOL includeMonths;
@property BOOL includeYears;
@property BOOL includeForever;
@property BOOL includeNever;

- (void)setupForShortTerm;
- (void)setupForLongTerm;
- (void)setupForAllUnits;

@end

