//
//  DSOutlineView.h
//  Shared classes
//
//  Created by David Sinclair on 2006-04-09.
//  Copyright 2006 Dejal Systems, LLC. All rights reserved.
//
//  Based on sample code from <http://www.cocoadev.com/index.pl?MailStyleGradientSelection>.
//


// NOTE: may want to merge the DSTableView enhancements into here, perhaps sharing the implementation via a private class to avoid code duplication.


@interface DSOutlineView : NSOutlineView
{
    NSIndexSet *_DSDraggedRows;
    
    BOOL _DSUsesGradientSelection;
    BOOL _DSSelectionGradientIsContiguous;
    BOOL _DSUsesDisabledGradientSelectionOnly;
    BOOL _DSHasBreakBetweenGradientSelectedRows;
}

/* Useful for delegate when deciding how to colour text */
- (NSIndexSet *)draggedRows;

// Gradient selection methods
/* Sets whether the outline view should use gradient selection bars. */
- (void)setUsesGradientSelection:(BOOL)flag;
- (BOOL)usesGradientSelection;

/* Sets whether gradient selections should be contiguous across multiple
    rows. (iTunes and Mail don't have this, but I think it looks better.) */
- (void)setSelectionGradientIsContiguous:(BOOL)flag;
- (BOOL)selectionGradientIsContiguous;

/* Sets whether the selection should always look disabled (grey), even
    when the outline view has the focus (like in Mail) */
- (void)setUsesDisabledGradientSelectionOnly:(BOOL)flag;
- (BOOL)usesDisabledGradientSelectionOnly;

/* Sets whether selected rows have a pixel gap between them that is the
    background colour rather than the selection colour */
- (void)setHasBreakBetweenGradientSelectedRows:(BOOL)flag;
- (BOOL)hasBreakBetweenGradientSelectedRows;

@property (getter=selectionGradientIsContiguous,setter=setSelectionGradientIsContiguous:) BOOL _DSSelectionGradientIsContiguous;
@property (getter=hasBreakBetweenGradientSelectedRows,setter=setHasBreakBetweenGradientSelectedRows:) BOOL _DSHasBreakBetweenGradientSelectedRows;
@property (getter=usesDisabledGradientSelectionOnly,setter=setUsesDisabledGradientSelectionOnly:) BOOL _DSUsesDisabledGradientSelectionOnly;
@property (retain,readonly,getter=draggedRows) NSIndexSet *_DSDraggedRows;
@property (getter=usesGradientSelection,setter=setUsesGradientSelection:) BOOL _DSUsesGradientSelection;
@end

