//
//  DSTableView.m
//  Shared classes
//
//  Created by David Sinclair on Tue Nov 11 2003.
//  Copyright (c) 2003 - 2007 Dejal Systems, LLC. All rights reserved.
//


// NOTE: may want to merge the DSOutlineView enhancements into here, perhaps sharing the implementation via a private class to avoid code duplication.

// NOTE: This class used to implement source list drawing and column hiding.  This code has been deleted, as these are implemented in 10.5 and later.  See -[NSTableColumn setHidden:] for hiding.


#import "DSTableView.h"
#import "DSTableViewCategories.h"


@implementation DSTableView

- (void)dealloc;
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    
    [self setDragPasteboard:nil];
    
    [super dealloc];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark DELETE KEY METHODS
// ----------------------------------------------------------------------------------------

- (void)keyDown:(NSEvent *)theEvent;
{
//    NSLog(@"keyDown: %@, %d", theEvent, [[theEvent characters] length]);
    
    if ([[theEvent characters] isEqualToString:[NSString stringWithFormat:@"%C", NSDeleteCharacter]] &&
        [self selectedRow] >= 0 && [[self delegate] respondsToSelector:@selector(tableView:canDeleteRowIndexes:)] &&
        [[self delegate] respondsToSelector:@selector(tableView:deleteRowIndexes:)] &&
        [[self delegate] tableView:self canDeleteRowIndexes:[self selectedRowIndexes]])
        [[self delegate] tableView:self deleteRowIndexes:[self selectedRowIndexes]];
    else
        [super keyDown:theEvent];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark CUSTOM DRAG METHODS
// ----------------------------------------------------------------------------------------

- (NSImage *)dragImageForRowsWithIndexes:(NSIndexSet *)dragRows
                            tableColumns:(NSArray *)tableColumns
                                   event:(NSEvent *)dragEvent
                                  offset:(NSPointPointer)dragImageOffset;
{
    if ([[self delegate] respondsToSelector:@selector(tableView:dragImageForRowsWithIndexes:tableColumns:event:offset:)])
        return [[self delegate] tableView:self dragImageForRowsWithIndexes:dragRows tableColumns:tableColumns event:dragEvent offset:dragImageOffset];
    else
        return [super dragImageForRowsWithIndexes:dragRows tableColumns:tableColumns event:dragEvent offset:dragImageOffset];
}

// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark DRAG DELETE METHODS
// ----------------------------------------------------------------------------------------

/*
 draggedImage:endedAt:operation:
 
 This method is overridden in this subclass to handle dragging out of the table as a delete operation.  The "poof" animation effect is used if available, and the table's data source is sent -tableView:deleteWithPasteboard:, if implemented, to actually remove the dragged rows.
 
 Written by DJS 2003-11.
 Changed by DJS 2005-10 to integrate into a generic table subclass.
*/

- (void)draggedImage:(NSImage *)anImage endedAt:(NSPoint)aPoint operation:(NSDragOperation)operation;
{
    // If not using the drag delete functionality, just call the superclass:
    if (![self usesDragDelete])
        [super draggedImage:anImage endedAt:aPoint operation:operation];
    
    NSPoint mousePoint = [NSEvent mouseLocation];
    NSPoint winPoint = [[self window] convertScreenToBase:mousePoint];
    NSPoint viewPoint = [self convertPoint:winPoint fromView:nil];
    
    // If the drag session ended outside the view:
    if (!NSMouseInRect(viewPoint, [self visibleRect], [self isFlipped]) && [[self dataSource] respondsToSelector: @selector(tableView:deleteWithPasteboard:)])
    {
        NSShowAnimationEffect(NSAnimationEffectDisappearingItemDefault, mousePoint, NSZeroSize, nil, nil, nil);
        
        [[self dataSource] performSelector:@selector(tableView:deleteWithPasteboard:) withObject:self withObject:[self dragPasteboard]];
    }
}

/*
 dragImage:at:offset:event:pasteboard:source:slideBack:
 
 This method is overridden in this subclass to note the pasteboard used for the drag, and to avoid the default slide-back animation when the drag ends up outside the table.
 
 Written by DJS 2003-11.
 Changed by DJS 2005-10 to integrate into a generic table subclass.
*/

- (void)dragImage:(NSImage *)anImage at:(NSPoint)imageLoc offset:(NSSize)mouseOffset event:(NSEvent *)theEvent pasteboard:(NSPasteboard *)pboard source:(id)sourceObject slideBack:(BOOL)slideBack;
{
    [self setDragPasteboard:pboard];
    
    [super dragImage:anImage at:imageLoc offset:mouseOffset event:theEvent pasteboard:pboard source:sourceObject slideBack:[self usesDragDelete] ? NO : slideBack];
}


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark BACKGROUND COLOR METHODS
// ----------------------------------------------------------------------------------------

/*
 isOpaque
 
 Returns YES if the background color is opaque (i.e. no transparency), otherwise NO.  If NO is returned, the other custom background drawing behavior kicks in.
 
 Written by DJS 2006-06.
*/

- (BOOL)isOpaque;
{
    return ([[self backgroundColor] alphaComponent] == 1.0);
}

/*
 setBackgroundColor:
 
 Sets the receiver’s background color to a given color.  If the background isn't opaque, the enclosing scroll view's background drawing is turned off so it doesn't interfere with our custom drawing.
 
 Written by DJS 2006-06.
*/

- (void)setBackgroundColor:(NSColor *)aColor;
{
    [super setBackgroundColor:aColor];
    [[self enclosingScrollView] setDrawsBackground:[self isOpaque]];
}

@synthesize _DSDragPasteboard, _DSUsesDragDelete;

@end

