//
//  DSViewsListView.h
//  Shared classes
//
//  Created by David Sinclair on Fri Jun 20 2003.
//  Copyright (c) 2003 - 2005 Dejal Systems, LLC. All rights reserved.
//

#import <AppKit/AppKit.h>


@class DSViewsListView;

// Handlers passed to -addHandler: etc must conform to this protocol

@protocol DSViewsListHandlerProtocol

- (NSView *)containerView;

- (id)initForViewsListView:(DSViewsListView *)viewsListView withSettings:(id)settings userInfo:(id)userInfo;
- (id)settings;

@end


@interface DSViewsListView : NSView
{
    NSMutableArray *_viewHandlersArray;
    NSSplitView *_splitView;
    NSScrollView *_scrollView;
    NSView *_oldNextKeyView;
    int _batchCount;
    float _minimumHeight;
    BOOL _isWithinTopSubview;
    BOOL _addToBottom;
    BOOL _drawsBackground;
    BOOL _drawsBorder;
    BOOL _animate;
}

// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark CONTENT MANAGEMENT METHODS
// ----------------------------------------------------------------------------------------

- (void)addHandlersWithClass:(Class)handlerClass
               settingsArray:(NSArray *)settingsArray
             defaultSettings:(id)defaultSettings
                    userInfo:(id)userInfo;
- (NSArray *)handlerSettings;

- (void)replaceView:(NSView *)oldView withView:(NSView *)newView;

- (void)replaceHandler:(id)oldHandler withHandler:(id)newHandler;

- (void)addHandler:(id)handler;
- (void)insertHandler:(id)newHandler beforeHandler:(id)oldHandler;
- (void)insertHandler:(id)newHandler afterHandler:(id)oldHandler;

- (void)removeHandler:(id)handler;
- (void)removeAllHandlers;

- (int)count;

- (id)firstHandler;
- (id)lastHandler;

- (NSEnumerator *)handlerEnumerator;
- (NSEnumerator *)reverseHandlerEnumerator;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark BATCH MODE METHODS
// ----------------------------------------------------------------------------------------

- (int)batchCount;

- (int)beginBatch;
- (int)endBatch;


// ----------------------------------------------------------------------------------------
#pragma mark -
#pragma mark ACCESSOR METHODS
// ----------------------------------------------------------------------------------------

- (NSSplitView *)splitView;
- (BOOL)isWithinTopSubview;
- (void)setResizesViaSplitView:(NSSplitView *)splitView isWithinTopSubview:(BOOL)isWithinTopSubview;

- (NSScrollView *)scrollView;
- (void)setResizesViaScrollView:(NSScrollView *)scrollView;

- (NSView *)oldNextKeyView;
- (void)setOldNextKeyView:(NSView *)value;

- (BOOL)willAddToBottom;
- (void)setAddToBottom:(BOOL)value;

- (BOOL)drawsBackground;
- (void)setDrawsBackground:(BOOL)value;

- (BOOL)drawsBorder;
- (void)setDrawsBorder:(BOOL)value;

- (BOOL)willAnimate;
- (void)setAnimate:(BOOL)value;

- (float)minimumHeight;
- (void)setMinimumHeight:(float)value;

@property (retain,getter=oldNextKeyView) NSView *_oldNextKeyView;
@property (getter=minimumHeight,setter=setMinimumHeight:) float _minimumHeight;
@property (getter=willAnimate,setter=setAnimate:) BOOL _animate;
@property (getter=drawsBorder,setter=setDrawsBorder:) BOOL _drawsBorder;
@property (getter=isWithinTopSubview) BOOL _isWithinTopSubview;
@property (retain,getter=viewHandlersArray) NSMutableArray *_viewHandlersArray;
@property (retain,getter=splitView) NSSplitView *_splitView;
@property (retain,getter=scrollView) NSScrollView *_scrollView;
@property (getter=drawsBackground,setter=setDrawsBackground:) BOOL _drawsBackground;
@property (getter=batchCount) int _batchCount;
@property (getter=willAddToBottom,setter=setAddToBottom:) BOOL _addToBottom;
@end
