#import "MainController.h"
#import "MainDefines.h"
#import "NarratorDocument.h"


@implementation MainController

/*
 preferenceClasses
 
 Returns an array of class names (as strings) for the preference pages.
 
 Written by DJS 2007-10.
*/

- (NSArray *)preferenceClasses
{
    return [NSArray arrayWithObjects:@"PrefsGeneral", @"PrefsDictionary", @"PrefsExport", @"DSPrefsUpdates", nil];
}

- (void)addReplacement:(NSString *)replace with:(NSString *)with toArray:(NSMutableArray *)array;
{
    NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:
                          replace, @"replace",
                          with, @"with",
                          nil];
    
    [array addObject:dict];
}

/*
 defaultPreferences
 
 Returns a mutable dictionary containing the default preference settings.
 
 Written by DJS 2007-10.
*/

- (NSMutableDictionary *)defaultPreferences
{
    NSMutableDictionary *dict = [super defaultPreferences];
    
    [dict setBool:YES forKey:CHECK_SPELLING];
    [dict setBool:YES forKey:CHECK_GRAMMAR];
    [dict setBool:YES forKey:SHOW_RULER];
    [dict setBool:YES forKey:SMART_COPY_PASTE];
    [dict setBool:YES forKey:SMART_QUOTES];
    [dict setBool:YES forKey:SMART_LINKS];
    [dict setObject:[NSData dataWithObject:[NSFont userFontOfSize:13.0]] forKey:DOCUMENT_FONT];
    
    [dict setBool:YES forKey:SHOULD_CREATE_NEW_DOCUMENT];
    [dict setBool:YES forKey:SHOULD_REOPEN_DOCUMENTS];
//    [dict setBool:YES forKey:SHOULD_AUTOSAVE];
//    [dict setFloat:5.0 forKey:AUTOSAVE_INTERVAL];
    [dict setBool:NO forKey:SAVE_SELECTION];
    [dict setBool:NO forKey:READ_FROM_INSERTION_POINT];
    [dict setBool:NO forKey:AUTO_READ];
    [dict setBool:YES forKey:CONFIRM_DELETE_CHAPTER];
    [dict setBool:YES forKey:CONFIRM_DELETE_CHARACTER];
    
    NSMutableArray *array = [NSMutableArray array];
    
    [self addReplacement:@"Dejal" with:@"Deejill" toArray:array];
    [self addReplacement:@"LOL" with:@"ha ha" toArray:array];
    [self addReplacement:@"BTW" with:@"by the way" toArray:array];
    [self addReplacement:@"IMHO" with:@"in my humble opinion" toArray:array];
    
    [dict setObject:array forKey:REPLACEMENT_WORDS];
    
    [dict setObject:[NSKeyedArchiver archivedDataWithRootObject:[NSURL fileURLWithPath:[@"~/Music/Narrator/" stringByStandardizingPath]]]  forKey:EXPORT_URL];
    [dict setObject:@"Narratives" forKey:EXPORT_PLAYLIST];
    [dict setObject:@"Dejal Narrator" forKey:EXPORT_ARTIST];
//    [dict setObject:@"Narratives" forKey:EXPORT_ALBUM];
    [dict setObject:@"Spoken Word" forKey:EXPORT_GENRE];
    [dict setBool:YES forKey:EXPORT_IS_BOOKMARKABLE];
    
    return dict;
}

/*
 upgradePreferencesFromVersion:
 
 If any of the preferences are out of date or need resetting (e.g. when adding a column to a table), do so here.  The parameter is the version number from the preference file; if this is less than the current version number, do the upgrade.  Return the current version number.  Note: for things like tables, it is often better to just change the autosave name (e.g. by appending the app version number), so the old pref isn't used, but is retained for backward compatibility.
 
 Written by DJS 2007-10.
*/

- (NSInteger)upgradePreferencesFromVersion:(NSInteger)preferencesVersion
{
    /*      ¶¶ commented out as not needed yet
    if (preferencesVersion < 1)
    [PREFERENCES restoreFactorySettingsForKey:xxx];
    
    return 1;
    */
    
    return 0;            // ¶¶ hack
}

/*
 setupAssistantSettings
 
 Returns the settings for the Setup Assistant intern, including the DSInternClass object (a Class type) with the class of the intern.  Return nil if no Setup Assistant is desired.
 
 Written by DJS 2007-10.
*/

- (NSDictionary *)setupAssistantSettings;
{
    return nil;
    
    // return [NSDictionary dictionaryWithObject:[SetupIntern class] forKey:@"DSInternClass"];
}

/*
 loadInitialApplicationData
 
 Loads the application's data while the splash is still visible (if applicable).  Don't display any UI yet (other than temporary loading UI).  Returns YES if the data was loaded, or NO if there is no data yet, i.e. the Setup Assistant should be displayed.
 
 Written by DJS 2007-10.
*/

- (BOOL)loadInitialApplicationData;
{
    // Register self as the Services provider:
    [NSApp setServicesProvider:self];
    NSUpdateDynamicServices();
    
//    if ([PREFERENCES boolForKey:SHOULD_AUTOSAVE])
//        [[NSDocumentController sharedDocumentController] setAutosavingDelay:[PREFERENCES floatForKey:AUTOSAVE_INTERVAL]];
    
    return YES;
}

/*
 displayInitialApplicationViews
 
 Displays any standard application windows or other UI.  Returns YES if the windows were successfully displayed, or NO if they couldn't be displayed for some reason, in which case the application terminates.
 
 Written by DJS 2008-01.
*/

- (BOOL)displayInitialApplicationViews;
{
    [self showWelcomeDocument:nil];
    
    return YES;
}

/*
 applicationShouldOpenUntitledFile:
 
 Called to determine if an untitled document should be opened.  We say NO if we're still starting up, as we might need to display the license assistant etc.  Otherwise we say YES unless there are other documents to re-open, or we want to display the welcome document.
 
 Written by DJS 2008-01.
*/

- (BOOL)applicationShouldOpenUntitledFile:(NSApplication *)sender;
{
    // If called by the OS and we're still starting, defer the decision till we reach -displayInitialApplicationViews:
    if (sender && ![self fullyStarted])
        return NO;
    
    NSString *dataPath = [@"~/Library/Application Support/Dejal/SnowyCoreDataDemo/" stringByStandardizingPath];
    
    if (![[NSFileManager defaultManager] fileExistsAtPath:dataPath])
    {
        // Create it:
        [dataPath validatedDirectoryPath];
        
        [self showWelcomeDocument:nil];
        
        return NO;
    }
    else if ([PREFERENCES boolForKey:SHOULD_REOPEN_DOCUMENTS])
    {
        NSArray *reopen = [PREFERENCES objectForKey:REOPEN_DOCUMENTS];
        
        if ([reopen count])
        {
            for (NSString *path in reopen)
            {
                NSURL *url = [NSURL fileURLWithPath:[path stringByStandardizingPath]];
                
                [[NSDocumentController sharedDocumentController] openDocumentWithContentsOfURL:url display:YES error:nil];
            }
            
            return NO;
        }
    }
    
    return [PREFERENCES boolForKey:SHOULD_CREATE_NEW_DOCUMENT] && ![[[NSDocumentController sharedDocumentController] documents] count];
}

- (void)finishQuittingStuff
{
    if ([PREFERENCES boolForKey:SHOULD_REOPEN_DOCUMENTS])
    {
        NSMutableArray *reopen = [NSMutableArray array];
        
        for (NSDocument *document in [[NSDocumentController sharedDocumentController] documents])
        {
            NSString *path = [[[document fileURL] path] stringByAbbreviatingWithTildeInPath];
            
            if ([path length])
                [reopen addObject:path];
        }
        
        [PREFERENCES setObject:reopen forKey:REOPEN_DOCUMENTS];
    }
}


- (NSString *)productSpecificStats
{
    return @"";
}

/*
 availableLicensesArray
 
 Returns an array of dictionaries with information on the available licenses for the licensing features.
 
 Written by DJS 2007-10.
*/

- (NSArray *)availableLicensesArray;
{
    NSMutableArray *array = [NSMutableArray array];
    
    return array;
}


/*
 validateMenuItem:
 
 Validates the application menus that have the main controller as a target, plus the Dock menu.
 
 Written by DJS 2007-10.
*/

/*
- (BOOL)validateMenuItem:(NSMenuItem *)menuItem;
{
    SEL action = [menuItem action];
    BOOL activate = [super validateMenuItem:menuItem];
    
    if (action == @selector(showEditor:))
        activate &= [self fullyStarted];
    
    return activate;
}    
*/


/*
 showWelcomeDocument
 
 Shows the welcome document.
 
 Written by DJS 2007-12.
*/

- (IBAction)showWelcomeDocument:(id)sender;
{
    NSString *path = [[NSBundle mainBundle] pathForResource:NSLocalizedString(@"Welcome to Narrator!", @"Filename of the welcome document") ofType:@"narrative"];
    NSURL *url = [NSURL fileURLWithPath:path];
    
    [[NSDocumentController sharedDocumentController] openDocumentWithContentsOfURL:url display:YES error:nil];
}

/*
 helpNarratorVoiceTalent:
 
 Goes to the Narrator Voice Talent web page.
 
 Written by DJS 2008-02.
*/

- (IBAction)helpNarratorVoiceTalent:(id)sender;
{
    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:URL_NARRATOR_VOICE_TALENT]];
}


/*
 newDocumentFromService:userData:error:
 
 Handler for the Services menu command.  Opens a new document with the selected text.
 
 Written by DJS 2007-12.
*/

- (void)newDocumentFromService:(NSPasteboard *)pboard userData:(NSString *)userData error:(NSString **)error;
{
    NSArray *types = [pboard types];
    NSAttributedString *text = nil;
    NarratorDocument *doc = nil;
    
    if ([types containsObject:NSRTFDPboardType] || [types containsObject:NSRTFPboardType] || [types containsObject:NSStringPboardType])
    {
        if ([types containsObject:NSRTFDPboardType])
            text = [NSAttributedString attributedStringWithRTFD:[pboard dataForType:NSRTFDPboardType]];
        else if ([types containsObject:NSRTFPboardType])
            text = [NSAttributedString attributedStringWithRTF:[pboard dataForType:NSRTFPboardType]];
        else
            text = [NSAttributedString attributedStringWithString:[pboard stringForType:NSStringPboardType]];
        
        if (text)
        {
            // Open a new untitled document:
            doc = [[NSDocumentController sharedDocumentController] openUntitledDocumentAndDisplay:YES error:nil];
            
            [doc replaceCurrentChapterTextWithAttributedString:text];
            
            [doc updateChangeCount:NSChangeDone];
            [NSApp activateIgnoringOtherApps:YES];
        }
    }
}

@end

