//
//  Version1Characters.m
//  Narrator
//
//  Created by David Sinclair on Wed May 15 2002.
//  Copyright (c) 2002 Dejal Systems, LLC. All rights reserved.
//

#import "Version1Characters.h"
#import "MainDefines.h"

static unsigned int kCharacterVersion = 0;
static unsigned int kCastVersion = 1;

@implementation Character

- (id)init
{
    [super init];

    [Character setVersion:kCharacterVersion];
    
    name = @"";
    voice = @"";
    rate = [[NSNumber numberWithDouble:163.2] retain];
    pitch = [[NSNumber numberWithDouble:40.0] retain];
    inflection = [[NSNumber numberWithDouble:45.0] retain];
    volume = [[NSNumber numberWithDouble:1.0] retain];
    marker = [[NSColor blueColor] retain];
    
    return self;
}

- (void)dealloc
{
    [name release];
    [voice release];
    [rate release];
    [pitch release];
    [inflection release];
    [volume release];
    [marker release];

    [super dealloc];
}


+ (int)version
{
    return kCharacterVersion;
}


- (NSString *)description
{
    return [NSString stringWithFormat:@"Character: %@; voice: %@; marker: %@", name, voice, marker];
}

- (NSString *)name
{
    return name;
}

- (void)setName:(NSString *)value
{
    [name autorelease];
    name = [value copy];
}


- (NSString *)voice
{
    return voice;
}

- (void)setVoice:(NSString *)value
{
    [voice autorelease];
    voice = [value copy];
}


- (NSNumber *)rate
{
    return rate;
}

- (void)setRate:(NSNumber *)value
{
    [rate autorelease];
    rate = [value copy];
}


- (NSNumber *)pitch
{
    return pitch;
}

- (void)setPitch:(NSNumber *)value
{

//    NSLog(@"Pitch: %@, retainCount: %@; value: %@, retainCount: %@", pitch, [pitch retainCount], value, [value retainCount]);	// ¶¶ hack

    [pitch autorelease];
    pitch = [value copy];
//    [value retain];
//    [pitch release];
//    pitch = value;

//    NSLog(@"Pitch: %@, retainCount: %@; value: %@, retainCount: %@", pitch, [pitch retainCount], value, [value retainCount]);	// ¶¶ hack

}

- (NSNumber *)inflection
{
    return inflection;
}

- (void)setInflection:(NSNumber *)value
{
    [inflection autorelease];
    inflection = [value copy];
}


- (NSNumber *)volume
{
    return volume;
}

- (void)setVolume:(NSNumber *)value
{
    [volume autorelease];
    volume = [value copy];
}

- (NSColor *)marker
{
    return marker;
}

- (void)setMarker:(NSColor *)value
{
    [marker autorelease];
    marker = [value copy];
}


- (void)encodeWithCoder:(NSCoder *)coder
{
    [coder encodeObject:[self name]];
    [coder encodeObject:[self voice]];
    [coder encodeObject:[self rate]];
    [coder encodeObject:[self pitch]];
    [coder encodeObject:[self inflection]];
    [coder encodeObject:[self volume]];
    [coder encodeObject:[self marker]];
}

- (id)initWithCoder:(NSCoder *)coder
{
    unsigned int vers;
    
    [super init];
    
    vers = [coder versionForClassName: @"Character"];
    
    [self setName:[coder decodeObject]];
    [self setVoice:[coder decodeObject]];
    [self setRate:[coder decodeObject]];
    [self setPitch:[coder decodeObject]];
    [self setInflection:[coder decodeObject]];
    [self setVolume:[coder decodeObject]];
    [self setMarker:[coder decodeObject]];

    if (vers > 1)
    {
        // Decode new stuff in version 2
    }
    
    return self;
}

@end

@implementation Cast

- (id)init
{
    self = [super init];
    
    [Cast setVersion:kCastVersion];
    
    [self setNativeText:[[NSAttributedString alloc] init]];
    [self setCharacters:[NSMutableArray array]];
    [self setConfig:[NSMutableDictionary dictionaryWithObjectsAndKeys:
        [NSNumber numberWithBool:[[PREFERENCES objectForKey:SHOW_RULER] intValue]],SHOW_RULER,
        [NSNumber numberWithBool:[[PREFERENCES objectForKey:SAVE_SELECTION] intValue]],SAVE_SELECTION,
        nil]];

//    NSLog(@"version = %d",[Cast version]);			// ¶¶ hack
    
    return self;
}

- (void)dealloc
{
    [nativeText release];
    [characters release];
    [config release];
    [super dealloc];
}

+ (int)version
{
    return kCastVersion;
}


- (NSAttributedString *)nativeText
{
    return nativeText;
}

- (void)setNativeText:(NSAttributedString *)value
{
    [nativeText autorelease];
    nativeText = [value retain];
}

- (NSMutableArray *)characters
{
    return characters;
}

- (void)setCharacters:(NSMutableArray *)value
{
    [characters autorelease];
    characters = [value retain];
}

- (NSMutableDictionary *)config
{
    return config;
}

- (void)setConfig:(NSMutableDictionary *)value
{
    [config autorelease];
    config = [value retain];
}

- (void)encodeWithCoder:(NSCoder *)coder
{

//    NSLog(@"current version = %d, saving version = %d",[Cast version],[coder versionForClassName:@"Cast"]); 		// ¶¶ hack

    [coder encodeObject:[self nativeText]];
    [coder encodeObject:[self characters]];
    [coder encodeObject:[self config]];
}

- (id)initWithCoder:(NSCoder *)coder
{
    unsigned int vers;

    self = [super init];

    vers = [coder versionForClassName:@"Cast"];

//    NSLog(@"new version = %d, saved version = %d",[Cast version],vers);			// ¶¶ hack
    
    [self setNativeText:[coder decodeObject]];
    [self setCharacters:[coder decodeObject]];

    if (vers > 0)
    {
        // Decode new stuff in version 1 (1.0b1 files had version 0)
        [self setConfig:[coder decodeObject]];
    }
    else
    {
        // Set defaults for new stuff in version 1
        [self setConfig:[NSMutableDictionary dictionaryWithObjectsAndKeys:
            [NSNumber numberWithBool:[[PREFERENCES objectForKey:SHOW_RULER] intValue]],SHOW_RULER,
            [NSNumber numberWithBool:[[PREFERENCES objectForKey:SAVE_SELECTION] intValue]],SAVE_SELECTION,
            nil]];
    }
    
    return self;
}

@end
