//
//  CharacterEntity.m
//  Narrator
//
//  Created by David Sinclair on 2007-10-16.
//  Copyright (c) 2007 Dejal Systems, LLC. All rights reserved.
//

#import "CharacterEntity.h"
#import "VoiceEntity.h"
#import "MainDefines.h"


@implementation CharacterEntity

@dynamic actorName, voiceName, parent, children;

- (void)awakeFromFetch;
{
    [super awakeFromFetch];
    
    [self addObserver:self
           forKeyPath:@"name"
              options:(NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld)
              context:NULL];
}

- (void)awakeFromInsert;
{
    [super awakeFromInsert];
    
    [self addObserver:self
           forKeyPath:@"name"
              options:(NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld)
              context:NULL];
}

+ (CharacterEntity *)insertCharacterWithIdentifier:(NSString *)characterIdentifier name:(NSString *)characterName color:(NSColor *)characterColor
                                   voiceIdentifier:(NSString *)voiceIdentifier inContext:(NSManagedObjectContext *)context;
{
    CharacterEntity *character = [NSEntityDescription insertNewObjectForEntityForName:@"Character" inManagedObjectContext:context];
    
    character.identifier = characterIdentifier;
    character.name = characterName;
    
    if (characterColor)
        character.color = characterColor;
    
    if (voiceIdentifier)
        character.voice.identifier = voiceIdentifier;
    
    return character;
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context;
{
    if ([keyPath isEqual:@"name"])
    {
        [[NSNotificationCenter defaultCenter] postNotificationName:@"CharacterNameChanged" object:object];
    }
}

//- (NSString *)rateTitle;
//{
//    if ([self isSilentReadAlong])
//        return NSLocalizedString(@"Duration:", @"Title of the rate slider for the silent read-along actor.");
//    else
//        return NSLocalizedString(@"Rate:", @"Normal title of the rate slider.");
//}

- (BOOL)isSilent;
{
    return [self.voice isSilent];
}

- (BOOL)isSilentReadAlong;
{
    return [self.voice isSilentReadAlong];
}

- (BOOL)isSilentSkipOver;
{
    return [self.voice isSilentSkipOver];
}

@end

