//
//  MTCDUtilities.m
//  MTCoreData
//
//  Created by David Sinclair on 2007-10-05.
//  From the Intro to Core Data part 3 article in MacTech magazine, Nov 2005.
//

// Move these into the Shared sources in due course.
// I have added the count method, and removed -release calls since not needed with GC.


#import "MTCDUtilities.h"


@implementation MTCDUtilities

// DJS: This one I wrote:

+ (NSUInteger)countObjectsForEntityNamed:(NSString *)name inContext:(NSManagedObjectContext *)context;
{
    NSEntityDescription *entity = [NSEntityDescription entityForName:name inManagedObjectContext:context];
    NSFetchRequest *req = [[NSFetchRequest alloc] init];
    NSError *error = nil;
    
    [req setEntity:entity];
    
    NSUInteger count = [context countForFetchRequest:req error:&error];
    
    if (error)
    {
        NSException *exception = [NSException exceptionWithName:MTCoreDataException reason:[error localizedDescription] userInfo:nil];
        [exception raise];
    }
    
    return count;
}

+ (NSArray *)objectsForEntityNamed:(NSString *)name inContext:(NSManagedObjectContext *)context;
{
    NSEntityDescription *entity = [NSEntityDescription entityForName:name inManagedObjectContext:context];
    NSFetchRequest *req = [[NSFetchRequest alloc] init];
    NSError *error = nil;
    
//    NSLog(@"name: %@; entity: %@", name, entity);         // log
    
    [req setEntity:entity];
    
    NSArray *array = [context executeFetchRequest:req error:&error];
    
    if (!array)
    {
        NSException *exception = [NSException exceptionWithName:MTCoreDataException reason:[error localizedDescription] userInfo:nil];
        [exception raise];
    }
    
    return array;
}

// DJS: This one I wrote:

+ (NSManagedObject *)objectForEntityNamed:(NSString *)name inContext:(NSManagedObjectContext *)context;
{
    NSArray *array = [MTCDUtilities objectsForEntityNamed:name inContext:context];
    
    if ([array count] > 1)
    {
        NSException *exception = [NSException exceptionWithName:MTTooManyEntitiesReturnedException reason:@"Too many instances retrieved for criteria" userInfo:nil];
        [exception raise];
    }
    
    if (![array count])
        return nil;
    
    return (NSManagedObject *)[array objectAtIndex:0];
}

+ (NSArray *)objectsForEntityNamed:(NSString *)name matchingKey:(NSString *)key andValue:(id)value inContext:(NSManagedObjectContext *)context;
{
    NSString *predString = [NSString stringWithFormat:@"%@ == %%@", key];
    NSPredicate *pred = [NSPredicate predicateWithFormat:predString, value];
    NSEntityDescription *entity = [NSEntityDescription entityForName:name inManagedObjectContext:context];
    NSFetchRequest *req = [[NSFetchRequest alloc] init];
    
    [req setEntity:entity];
    [req setPredicate:pred];
    
    NSError *error = nil;
    NSArray *array = [context executeFetchRequest:req error:&error];
    
    if (!array)
    {
        NSException *exception = [NSException exceptionWithName:MTCoreDataException reason:[error localizedDescription] userInfo:nil];
        [exception raise];
    }
    
    return array;
}

+ (NSManagedObject *)objectForEntityNamed:(NSString *)name matchingKey:(NSString *)key andValue:(id)value inContext:(NSManagedObjectContext *)context;
{
    NSArray *array = [MTCDUtilities objectsForEntityNamed:name matchingKey:key andValue:value inContext:context];
    
    if ([array count] > 1)
    {
        NSException *exception = [NSException exceptionWithName:MTTooManyEntitiesReturnedException reason:@"Too many instances retrieved for criteria" userInfo:nil];
        [exception raise];
    }
    
    if (![array count])
        return nil;
    
    return (NSManagedObject *)[array objectAtIndex:0];
}

+ (NSArray *)objectsForEntityNamed:(NSString *)name matchingKeysAndValues:(NSDictionary *)keyValues usingOR:(BOOL)useOR inContext:(NSManagedObjectContext *)context;
{
    NSEnumerator *enumerator = [keyValues keyEnumerator];
    NSPredicate *pred = nil;
    NSString *key = nil;
    
    while ((key = [enumerator nextObject]))
    {
        NSString *predString = [NSString stringWithFormat:@"%@ == %%@", key];
        
        if (!pred)
            pred = [NSPredicate predicateWithFormat:predString, [keyValues objectForKey:key]];
        else
        {
            NSPredicate *tempPred = [NSPredicate predicateWithFormat:predString, [keyValues objectForKey:key]];
            NSArray *array = [NSArray arrayWithObjects:tempPred, pred, nil];
            
            if (useOR)
                pred = [NSCompoundPredicate orPredicateWithSubpredicates:array];
            else
                pred = [NSCompoundPredicate andPredicateWithSubpredicates:array];
        }
    }
    
    NSEntityDescription *entity = [NSEntityDescription entityForName:name inManagedObjectContext:context];
    NSFetchRequest *req = [[NSFetchRequest alloc] init];
    
    [req setEntity:entity];
    [req setPredicate:pred];
    
    NSError *error = nil;
    NSArray *array = [context executeFetchRequest:req error:&error];
    
    if (!array)
    {
        NSException *exception = [NSException exceptionWithName:MTCoreDataException reason:[error localizedDescription] userInfo:nil];
        [exception raise];
    }
    
    return array;
}

@end
