//
//  VoiceEntity.m
//  Narrator
//
//  Created by David Sinclair on 2007-11-09.
//  Copyright 2007 Dejal Systems, LLC. All rights reserved.
//

#import "VoiceEntity.h"
#import "MainDefines.h"


@implementation VoiceEntity

@dynamic identifier, name, age, gender, language, pitch, rate, inflection, volume, person, parent, children, rateTitle;

- (void)awakeFromFetch;
{
    [super awakeFromFetch];
    
//    [self addObserver:self
//           forKeyPath:@"identifier"
//              options:(NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld)
//              context:NULL];
}

- (void)awakeFromInsert;
{
    [super awakeFromInsert];
    
    [self resetProperty:nil];
    
//    [self addObserver:self
//           forKeyPath:@"identifier"
//              options:(NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld)
//              context:NULL];
}

- (void)finalize;
{
//	[self removeObserver:self forKeyPath:@"identifier"];
    
	[super finalize];
}

// Resets a specified synthesizer property, or all attributes and properties if nil.

- (void)resetProperty:(NSString *)propertyKey;
{
    if (!propertyKey)
    {
        if ([self isSilent])
        {
            self.name = self.identifier;
            self.age = nil;
            self.language = @"en";
            self.gender = @"";
        }
        else
        {
            NSDictionary *attributes = [NSSpeechSynthesizer attributesForVoice:self.identifier];
            
            self.name = [attributes objectForKey:NSVoiceName];
            self.age = [attributes objectForKey:NSVoiceAge];
            self.language = [attributes objectForKey:NSVoiceLocaleIdentifier];
            
            NSString *voiceGender = [attributes objectForKey:NSVoiceGender];
            
            // Set the gender in a localization-friendly way:
            if ([voiceGender isEqualToString:NSVoiceGenderMale])
                self.gender = NSLocalizedString(@"Male", @"Male gender");
            else if ([voiceGender isEqualToString:NSVoiceGenderFemale])
                self.gender = NSLocalizedString(@"Female", @"Female gender");
            else
                self.gender = NSLocalizedString(@"Neuter", @"Neuter gender");
        }
        
        if ([self isSilentSkipOver])
            self.rateTitle = NSLocalizedString(@"Duration:", @"Title of the rate slider for the silent read-along actor.");
        else
            self.rateTitle = NSLocalizedString(@"Rate:", @"Normal title of the rate slider.");
    }
    
    // Use the default system voice as the basis for the silent actors, or the chosen actor:
    NSSpeechSynthesizer *synthesizer = [[NSSpeechSynthesizer alloc] initWithVoice:[self isSilent] ? [NSSpeechSynthesizer defaultVoice] : self.identifier];
    
    if (!propertyKey || [propertyKey isEqualToString:@"rate"])
    {
        if ([self isSilentSkipOver])
            self.rate = [NSNumber numberWithInt:0];
        else
            self.rate = [synthesizer objectForProperty:NSSpeechRateProperty error:nil];
        
        if (!self.rate)
            self.rate = [NSNumber numberWithFloat:160.0];
    }
    
    if (!propertyKey || [propertyKey isEqualToString:@"pitch"])
    {
        self.pitch = [synthesizer objectForProperty:NSSpeechPitchBaseProperty error:nil];
        
        if (!self.pitch)
            self.pitch = [NSNumber numberWithFloat:40.0];
    }
    
    if (!propertyKey || [propertyKey isEqualToString:@"inflection"])
    {
        self.inflection = [synthesizer objectForProperty:NSSpeechPitchModProperty error:nil];
        
        if (!self.inflection)
            self.inflection = [NSNumber numberWithFloat:45.0];
    }
    
    if (!propertyKey || [propertyKey isEqualToString:@"volume"])
    {
        if ([self isSilent])
            self.volume = [NSNumber numberWithInt:0];
        else
            self.volume = [synthesizer objectForProperty:NSSpeechVolumeProperty error:nil];
        
        if (!self.volume)
            self.volume = [NSNumber numberWithFloat:1.0];
    }
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context;
{
    if ([keyPath isEqual:@"identifier"])
    {
        [self resetProperty:nil];
//        [openingBalanceInspectorField setObjectValue:[change objectForKey:NSKeyValueChangeNewKey]];
    }
    
//    [super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
}

/*
- (NSString *)displayGender;
{
    [self willAccessValueForKey:@"gender"];
    NSString *gender = [self primitiveValueForKey:@"gender"];
    [self didAccessValueForKey:@"gender"];
    
    return [gender substringFromString:@"VoiceGender"];
}
*/

- (BOOL)isSilent;
{
    return [self.identifier hasPrefix:@"com.dejal.narrator.Silent"];
}

- (BOOL)isSilentReadAlong;
{
    return [self.identifier isEqualToString:@"com.dejal.narrator.SilentReadAlong"];
}

- (BOOL)isSilentSkipOver;
{
    return [self.identifier isEqualToString:@"com.dejal.narrator.SilentSkipOver"];
}

@end

