//
//  CharacterAttachmentCell.m
//  Narrator
//
//  Created by David Sinclair on 2007-11-05.
//  Copyright 2007 Dejal Systems, LLC. All rights reserved.
//

#import "CharacterAttachmentCell.h"


@implementation CharacterAttachmentCell

@synthesize identifier, delegate;

// Returns an image representation of the cell, using the specified name and color; handy for a placeholder:

+ (NSImage *)imageWithName:(NSString *)newName color:(NSColor *)newColor;
{
    CharacterAttachmentCell *cell = [self new];
    
    cell.identifier = @"Placeholder";
    cell.name = newName;
    cell.color = newColor;
    
    NSImage *image = [[NSImage alloc] initWithSize:[cell cellSize]];
    
    [image lockFocus];
    [cell drawWithFrame:[image alignmentRect]];
    [image unlockFocus];
    
    return image;
}


// these don't seem to be needed....

//- (id)initWithCoder:(NSCoder *)coder;
//{
//    if ((self = [super initWithCoder:coder]))
//    {
//        if ([coder allowsKeyedCoding])
//            self.identifier = [coder decodeObjectForKey:@"Identifier"];
//        else
//            self.identifier = [coder decodeObject];
//    }
//    
//    return self;
//}
//
//- (void)encodeWithCoder:(NSCoder *)coder;
//{
//    [super encodeWithCoder:coder];
//    
//    if ([coder allowsKeyedCoding])
//        [coder encodeObject:self.identifier forKey:@"Identifier"];
//    else
//        [coder encodeObject:self.identifier];
//}

/*
- (NSRect)cellFrameForTextContainer:(NSTextContainer *)textContainer  
               proposedLineFragment:(NSRect)lineFrag
                      glyphPosition:(NSPoint)position  
                     characterIndex:(NSUInteger)charIndex;
{
//    NSLog(@"line frag %@", NSStringFromRect(lineFrag));
    return NSMakeRect(0, 0, lineFrag.size.width - 10, 16);
}
*/

- (NSDictionary *)drawingAttributes;
{
    return [NSDictionary dictionaryWithObject:@"" forKey:@"NSFontAttributeName"];
}

- (NSSize)cellSize;
{
    NSSize size = [self.name sizeWithAttributes:self.drawingAttributes];
    
//    size.height += 4;
    size.width += 20;
    
    return size;
}

- (NSBezierPath *)backgroundBezierPathForRect:(NSRect)rect;
{
    NSBezierPath *bezierPath = [NSBezierPath bezierPath];
    
    rect = NSInsetRect(rect, 3.0, 1.0);
    
    [bezierPath moveToPoint:NSMakePoint(NSMinX(rect), NSMinY(rect))];
    [bezierPath lineToPoint:NSMakePoint(NSMaxX(rect) - 6.0, NSMinY(rect))];
    [bezierPath lineToPoint:NSMakePoint(NSMaxX(rect), NSMinY(rect) + (rect.size.height / 2.0))];
    [bezierPath lineToPoint:NSMakePoint(NSMaxX(rect) - 6.0, NSMaxY(rect))];
    [bezierPath lineToPoint:NSMakePoint(NSMinX(rect), NSMaxY(rect))];
    [bezierPath appendBezierPathWithArcFromPoint:NSMakePoint(NSMinX(rect) + 3.0, NSMinY(rect) + (rect.size.height / 2.0)) toPoint:NSMakePoint(NSMinX(rect), NSMinY(rect)) radius:8.0];
    [bezierPath closePath];
    
    [bezierPath setLineJoinStyle:NSRoundLineJoinStyle];
    [bezierPath setLineCapStyle:NSRoundLineCapStyle];
    [bezierPath setLineWidth:2.0];
    
    return bezierPath;
}

// Convenience method to draw the cell into an image:

- (void)drawWithFrame:(NSRect)frame;
{
    NSBezierPath *bezierPath = [self backgroundBezierPathForRect:frame];
    
    if (self.color)
        [self.color set];
    else
        [[NSColor yellowColor] set];
    
    [bezierPath fill];
    
    [[NSColor grayColor] set];
    [bezierPath stroke];
    
    NSRect textRect = frame;
    textRect.origin.x += 8.0;
//    textRect.origin.y += 2.0;
    
    [self.name drawInRect:textRect withAttributes:self.drawingAttributes];
}

- (void)drawWithFrame:(NSRect)cellFrame
               inView:(NSView *)controlView  
       characterIndex:(NSUInteger)charIndex
        layoutManager:(NSLayoutManager *)layoutManager;
{
    [self drawWithFrame:cellFrame];
}

- (NSString *)name;
{
    if (![name length])
        name = [delegate characterNameForIdentifier:self.identifier];
    
    if (![name length])
        self.name = @"Unknown";
    
    return name;
}

- (void)setName:(NSString *)value;
{
    name = value;
}

- (NSColor *)color;
{
    if (!color)
        color = [delegate characterColorForIdentifier:self.identifier];
    
    return color;
}

- (void)setColor:(NSColor *)value;
{
    color = value;
}

@end

